class DisplayBeingData{
  
  // List used as constants
  int displayNormal=0;
  int displayDeathType=1;
  int displayDeathCause=2;
  int displayZombieOrigin=3;
  int displayZombieGeneration=4;
  int displayZombieAge=5;
  
  // the colors. No problem if they overlap each other as 
  // they have no logic significance (contrary to those defined 
  // in ZombieInfection)
  color colorDefault = color(25);
  color colorNuke = color(252,235,60);
  color colorMine = color(167,84,5);
  color colorBomb = color(255,64,0);
  color colorAge = color(30,80,40);
  color colorWalled = color(225,225,230);
  
  
  // Number of modes we allow to cycle through
  int modesNumber=4;
  
  // Important to know when we have just changed the mode
  int oldMode;
  int mode;
  int oldSecond;
  int modeSecond;
  
  // the various lists linking a parameter to a color
  int typeList[][]={{ishuman,human},
                    {issoldier,soldier},
                    {iszombie,zombie},
                    {isweakenedZombie,weakenedZombie},
                    {ismadZombie,madZombie}};
  
  int deathCauseList[][]={{weaponNuke,colorNuke},
                          {weaponTroopers,soldier},
                          {weaponQuickBomb,colorBomb},
                          {weaponBomb,colorBomb},
                          {weaponSlowBomb,colorBomb},
                          {weaponMine,colorMine},
                          {weaponBigMine,colorMine},
                          {weaponDirt,colorWalled},
                          {weaponConcrete,colorWalled},
                          {weaponInoculate,weakenedZombie},
                          {weaponBioweapon,madZombie},
                          {deathAge,colorAge}};
  
  int deathCauseSecond[]={colorNuke,
                          colorBomb,
                          colorMine,
                          colorAge,
                          soldier,
                          colorWalled};
                          
  int deathTypeSecond[]={human,
                         soldier,
                         zombie,
                         weakenedZombie,
                         madZombie};  
  
  int zombieTypeSecond[]={zombie,
                          weakenedZombie,
                          madZombie};
  
  DisplayBeingData(){
    mode=displayNormal;
    oldMode=mode;
    modeSecond=0;
    oldSecond=modeSecond;
    
  }
  
  void cycleMode(){
    mode++;
    if (mode>=modesNumber) mode=0;
    modeSecond=0;
  }
  
  void cycleSecond(){
    int secondListTemp[];
    secondListTemp=getSecondList(mode);
    modeSecond++;
    
    if (secondListTemp!=null && modeSecond>secondListTemp.length)
      modeSecond=0;
    //println(modeSecond);
  }
  
  int[] getSecondList(int mode){
    if (mode==displayDeathType) return deathCauseSecond;
    else if (mode==displayDeathCause) return deathTypeSecond;
    else if (mode==displayZombieOrigin) return zombieTypeSecond;
    return null;
  }
  
  // true if mode has just changed.
  boolean changed(){
    if ((mode!=oldMode)||(modeSecond!=oldSecond)) return true;
    else return false;
  }
  
  void doneChanges(){
    oldMode=mode;
    oldSecond=modeSecond;
  }
  
  // that should be temporary and removed soon.
  boolean modeNormal(){
    if (mode==displayNormal) return true;
    else return false;
  }
  
  // Selects the correct color in the list, if it can find it.
  color displayColor(int parameter, int tempList [][]){
    color colour=dead;
    for (int j=0; j<tempList.length; j++){
      if (parameter == tempList[j][0]) {
        colour=tempList[j][1];
        break;
      }
    }
    return colour;
  }
  
  // Returns true if has been killed according to selected method
  boolean secondCondition(int criteria,int secondList[],int listTemp[][]){
    color deathCause = displayColor(criteria,listTemp);
    
    if (modeSecond==0) return true;
    else if (deathCause == secondList[modeSecond-1]) return true;
    return false;
  }
  
  void display(Being being){
    color colour;
    
    // we want to display what it was before dying
    if (mode==displayDeathType){
      if (being.type==isdead && secondCondition(being.deathCause,deathCauseSecond,deathCauseList))
          colour = displayColor(being.originalType,typeList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
    
    // We want to display the death cause
    else if (mode==displayDeathCause){
      if (being.type==isdead && secondCondition(being.originalType,deathTypeSecond,typeList)) 
          colour = displayColor(being.deathCause,deathCauseList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
    
    // We want to display what the zombie was
    else if (mode==displayZombieOrigin){
      if (being instanceof Zombie && secondCondition(being.type,zombieTypeSecond,typeList))
          colour = displayColor(being.originalType,typeList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
  }
}
