int screenStart=0;
int screenGame=1;

int menuNone=-1;
int menuGeneral=0;
int menuAction=1;
int menuStrategy=2;
int menuOption=3;

int Button_Data_Normal =0;
int Button_Data_Unavailable =1;
int Button_Data_Selected =2;

int menuDisplayWeaponNumber=10;
int listRefMultiplier=100;

// holds informations regarding the display and speed of the simulation
class GameSequence{
  boolean freeze;
  boolean oldFreeze;
  boolean turnBased;
  boolean finished,win;
  int turnLengthSetting;
  int turnLength;
  int initialTurnLengthSetting;
  float initialTurnLengthRatio;
  int speed;
  int speedRate;
  int screenRef;
  int menu; // not used now
  int previousStartMenu; // usefull to know if can use previous parameters or need to load default.
  
  int listSpeed[]={120,40,20,15,10};
  String listSpeedName[]={s_Speed_VerySlow,s_Speed_Slow,s_Speed_Normal,s_Speed_Quick,s_Speed_VeryQuick};
  int listTurnLength[]={50,100,200,300,500};
  float listInitialTurnLengthRatio[]={0.1,0.3,0.5,1,2,3};
  
  
  GameSequence(){
    screenRef = screenStart;
    menu = menuGeneral;
    previousStartMenu = menuGeneral;
    freeze=true;
    oldFreeze=freeze;
    turnBased=true;
    finished=false;
    setTurnLength(defaultTurnLength);
    setInitialTurnLength(defaultInitialTurnLengthRatio);
    blink = new Blink();

    // Moved here from game initialisation, because will be used in first screen
    world = new World();
    mission = new Mission(world); 
  }
  
  void setSpeedRate(){
    if (speed>=0 && speed<listSpeed.length) speedRate=listSpeed[speed];
    else speedRate = listSpeed[defaultSpeed];
  }
  
  void setSpeedName(String buttonName){
    String name;
    if (speed>=0 && speed<listSpeed.length && cp5.controller(buttonName)!=null){
      cp5.controller(buttonName).setValueLabel(listSpeedName[speed]);
    }
  }
  
  void setInitialTurnLengthName(String buttonName){
    String name="*"+str(initialTurnLengthRatio)+" ("+str(int(initialTurnLengthRatio*turnLength))+")";  
    if (cp5.controller(buttonName)!=null) cp5.controller(buttonName).setValueLabel(name);
  }
  
  void setInitialTurnLength(int value){
    initialTurnLengthSetting=value;
    if (initialTurnLengthSetting>=0 && initialTurnLengthSetting<listInitialTurnLengthRatio.length){
      initialTurnLengthRatio=listInitialTurnLengthRatio[initialTurnLengthSetting];
    }
    else {
      initialTurnLengthSetting=defaultInitialTurnLengthRatio;
      initialTurnLengthRatio=listInitialTurnLengthRatio[initialTurnLengthSetting];
    }
  }
  
  void setTurnLength(int value){
    turnLengthSetting =value;
    if (turnLengthSetting>=0 && turnLengthSetting<listTurnLength.length){
      turnLength = listTurnLength[turnLengthSetting];
    }
    else {
      turnLengthSetting =defaultTurnLength;
      turnLength = listTurnLength[turnLengthSetting];
    }
  }
  
  void setTurnLengthName(String buttonName){
    cp5.controller(buttonName).setValueLabel(str(turnLength));
  }
  
  void initStartScreen(){
    startScreen = new StartScreen();
  }
 
  void initGame(){
    finished = false;
    // does not start at 0, otherwise infection would have spread too much
    setSpeedRate();
    tick=0;
    
    //println(turnLength*initialTurnLengthRatio);
    clock=turnLength-int (turnLength*initialTurnLengthRatio);
    if (gameSequence.turnBased) gameSequence.freeze = false;
    else gameSequence.freeze=true;
  
  
  // Initiates the gameScreen
   
   world.clean();// data must be there because will be linked to the screens
   if (gameScreen!=null){
     gameScreen.erase();
     gameScreen=null;
   } 
   gameScreen = new GameScreen();
   
   world.init(gameScreen.playArea,num);
   mission.clean(); // After world init, because need to get info about the world

  }
  
  void won(String s){
    if (!finished){
      mission.score();
      finished=true;
      freeze=true;
      win=true;
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionWin,s,colorWin);
    }
  }
  
  void lost(String s){
    if (!finished){
      mission.score();
      finished=true;
      win=false;
      freeze=true;
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionLoose,s,colorLoose);
    }
  }
  
  void warning(String s,int type){
    if (!finished){
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionWarning,s,mission.getEventColor(type));
    }
  }
  
  void display(){
    blink.forward();
    if (gameSequence.screenRef == screenStart){
      startScreen.display();
    }
    else if (gameSequence.screenRef == screenGame){
      gameSequence.runGame();
      gameScreen.display();
    }
  }
  
  int checkTurnLength(int _clock){
    int clock=_clock;
    clock++;
    if (turnBased && !finished){
      if (clock>=turnLength) {
        freeze=true;
        gameScreen.textPanel.menu.buttonStatus("Menu_PanelStrategy_NextTurn",Button_Data_Normal);
        clock=0;
      }
    }
    return clock;
  }
  
  void runGame(){
    int changed;
    // Display specific data or cleans everything special being data have been displayed
    // Do action if game has just been restarted and was not on normal display, or if game as just been paused, 
    // or game is pause and just changed type of display
    if ((!freeze && freeze!=oldFreeze && !world.displayBeingNormal()) ||
       (freeze && (freeze!=oldFreeze || world.displayBeingChanged()))){
      
      world.updateBeingsDisplay();
      oldFreeze=freeze;
    }
    
    // Move one more step forward
    if (tick*10%speedRate==0){  
      if (!freeze){
        world.oneStepForward();  
        clock = checkTurnLength(clock);
        mission.checkEvent();
      }
      else world.oneStepFrozen();
    }
    tick++;
    if (tick>50000) tick=0; //probably not usefull, as it should loop. But who knows...
  }
}

class World{
  WeaponStock weaponStock;
  Being[] beings; // The structure that stores survivors, zombies and dead people
  ArrayList devices;  // devices left on the map, such as bombs,...
  ArrayList effects; //temporary effects (visual or events) on the game
  DisplayBeingData displayBeingData; // class used to display some data about beings
  ArrayList datas;    // data stored for the graph
  
  PlayArea playArea;  // Used so that it knows where to draw and check the objects
  int time;
  int initPop;
    
  int numKilledHumans;
  int numKilledZombies;
  int numInoculatedHumans;
  int numZombies;
  int numDead; 
  int numSoldier;
  int numHuman;

  World(){
    weaponStock= new WeaponStock(this);
    displayBeingData= new DisplayBeingData();
    datas= new ArrayList();
  }
  
  void clean(){
    datas= new ArrayList();
  }
  
  void init(PlayArea _playArea,int _initPop){
    time=0;
    initPop=_initPop;
    playArea=_playArea;
    
    devices = new ArrayList();
    effects = new ArrayList();
    weaponStock.reinit();
    
    // create the beings
    beings = new Being[8000];        
    for(int i=0; i<min(8000,initPop); i++){ 
      beings[i] = new Human(this); 
      beings[i].position();
      beings[i].drawBeing(); 
    }
    // and infects the 4 first one that have been created.
    /*beings[0].infect(0,iszombie);
    beings[1].infect(0,iszombie);
    beings[2].infect(0,iszombie);
    beings[3].infect(0,iszombie);*/
    
    // Prepare the data
    // Very important !! They are created in the same order as "ishuman, iszombie,..."
    numKilledHumans=0;
    numKilledZombies=0;
    numInoculatedHumans=0;
    numZombies=0;
    numDead=0; 
    numSoldier=0;
    numHuman=initPop;
        
    Data data;
    data = new Data(human,"Humans");
    data.add(numHuman);
    datas.add(data);
    data = new Data(soldier,"Soldiers");
    data.add(numSoldier);
    datas.add(data);
    data = new Data(zombie,"Zombies");
    data.add(numZombies);
    datas.add(data);
    data = new Data(dead,"Dead");
    data.add(numDead);
    datas.add(data);
    data = new Data(panicHuman,"Nuked H.");
    data.add(numKilledHumans);
    datas.add(data);
    data = new Data(activeZombie,"Nuked Z.");
    data.add(numKilledZombies);
    datas.add(data);
    data = new Data(weakenedZombie,"Inoc. H.");
    data.add(numInoculatedHumans);
    datas.add(data);
  }
  
  // Simulates the world one step forward
  void oneStepForward(){
    beingOneStep(false);
    deviceOneStep(false);
    effectOneStep(false);    
    weaponOneStep();
    dataOneStep();
    time++;
  }
  
  // Update the drawing of the workd, when frozen
  void oneStepFrozen(){
    beingOneStep(true);
    deviceOneStep(true); 
  }
  
  void updateBeingsDisplay(){
    for(int i=0; i<num; i++){
        beings[i].drawBeingColor(empty);
        beings[i].drawBeingOutside();
    }
    displayBeingData.doneChanges();
  }
  
    
  boolean displayBeingNormal(){
    return displayBeingData.modeNormal();
  }
  
  boolean displayBeingChanged(){
    return displayBeingData.changed();
  }
  
  // Moves and draws the beings one at a time
  // Should also be runned when the game is paused to draw changes when bombing and game is paused
  private void beingOneStep(boolean freeze){
    for(int i=0; i<num; i++){ 
      if (beings[i].changed!=-1) {
        beings[i]=changeBeing(beings[i]);
        beings[i].drawBeingOutside();
      }
      if (beings[i].update) beings[i].drawBeingOutside();
      if (!freeze) beings[i].move();
    }
  }
  
  // Draws and make device act one step forward
  // Should used when paused, as they devices can blink
  private void deviceOneStep(boolean freeze){
    for (int i=0; i<devices.size();i++){
      Device device = (Device) devices.get(i);
      device.drawDevice();
      if (!freeze) device.act();
      if (device.destroyed) devices.remove(i);
    }
  }
  
  // Draws and make effect act one step forward
  // Should used when paused, as they devices can blink
  private void effectOneStep(boolean freeze){
    for (int i=0; i<effects.size();i++){
      Effect effect = (Effect) effects.get(i);
      effect.drawEffect();
      if (!freeze) effect.act();
      if (effect.destroyed) {
        println("Removed Bombing");
        effects.remove(i);
      }
    }
  }
  
  // Should only be called when game is not paused
  private void weaponOneStep(){
    //Restock Ammos
    weaponStock.reStock();
  }
  
  private void dataOneStep(){
    // counts the number of zombies and population
      // might be improved to avoid counting everything frame, by only make the difference
      numZombies=0;
      numDead=0; 
      numSoldier=0;
      numHuman=0;
      for(int j=0;j<num;j++){
        if(beings[j] instanceof Zombie) numZombies++;
        if(beings[j] instanceof Dead) numDead++;
        if(beings[j] instanceof Soldier) numSoldier++;
        if(beings[j] instanceof Human) numHuman++;
      }   
      // Stores the data and draw the graph
      Data data;
      data = (Data) datas.get(dataHuman);
      data.add(numHuman);
      data = (Data) datas.get(dataZombie);
      data.add(numZombies);
      data = (Data) datas.get(dataDead);
      data.add(numDead);
      data = (Data) datas.get(dataKilledHumans);
      data.add(numKilledHumans);
      data = (Data) datas.get(dataKilledZombies);
      data.add(numKilledZombies);
      data = (Data) datas.get(dataSoldier);
      data.add(numSoldier);
      data = (Data) datas.get(dataInoculatedHumans);
      data.add(numInoculatedHumans);
  }
}

class Blink{
  float colorLoop;
  
  Blink(){
    colorLoop=0;
  }
  
  void forward(){
    colorLoop+=0.01;
    if (colorLoop>1000) colorLoop=0;
  }
  
  // returns true when a full cycle has been done
  boolean checkCycle(float speed){
    if (colorLoop*speed > 2*PI) return true;
    return false;
  }
  
  // blinks following the sinus fonction
  color gradual(color color1,color color2,float speed){
    float blink;
    int cRed,cGreen,cBlue;
    
    blink = (1+sin(colorLoop*speed-PI/2))/2;
    cRed = (int) (blink*red(color1)+(1-blink)*red(color2));
    cGreen = (int) (blink*green(color1)+(1-blink)*green(color2));
    cBlue = (int) (blink*blue(color1)+(1-blink)*blue(color2));
    
    return color(cRed,cGreen,cBlue);
  }
  
  // returns either color1 or color2
  color flash(color color1,color color2, float speed){
    if (cos(colorLoop*speed)>0) return color1;
    else return color2;
  }
  
  // returns the color that is glowing
  color glow(color color1, float speed){
    float blink;
    int cRed,cGreen,cBlue;
    
    blink = (1+sin(colorLoop*speed-PI/2))/2;
    cRed = (int) (blink*red(color1));
    cGreen = (int) (blink*green(color1));
    cBlue = (int) (blink*blue(color1));
    
    return color(cRed,cGreen,cBlue);
  }
}

