int infoPanelRef_Statistics = 0;
int infoPanelRef_Mission = 1;
int infoPanelRef_Score = 2;
int infoPanelRef_TransitionWin = 3;
int infoPanelRef_TransitionLoose = 4;
int infoPanelRef_TransitionWarning = 5;

class GameScreen{
  TextPanel textPanel;
  PlayArea playArea;
  InfoPanel infoPanel;
  HoverInfo hoverInfo;
  
  GameScreen(){
    
    int fieldHeight=(hoverTextSize+hoverTextInterline)+hoverHeightMargin*2;
    int w = width-1; // -1 because want to see the line
    int h1 = 0;
    int h2 = height-panelHeight-infoHeight-fieldHeight-1;// -1 because y starts at 0
    // Builds the map
    playArea = new PlayArea(0,h1,w,h2);
    playArea.drawMap();
    
    // Builds the bottom panel, where data will be displayed
    h1+=h2;
    h2=panelHeight;
    if (gameSequence.turnBased == true) textPanel = new TextPanelStrategy(0,h1,w,h2);
    else textPanel = new TextPanelAction(0,h1,w,h2); 
    
    h1+=h2;
    h2=infoHeight;
    infoPanel = new InfoPanel(0,h1,w,h2-1,world); // -1 because want to see the line (bottom drawn outside)
    
    h1+=h2;
    h2=fieldHeight;
    hoverInfo = new HoverInfo(0,h1,w,h2,hoverWidthMargin,hoverHeightMargin);
  }
  
  void display(){
    if (gameSequence.finished && !(textPanel instanceof TextPanelFinished)){
      int _x=textPanel.tX;
      int _y=textPanel.tY;
      int _w=textPanel.tWidth;
      int _h=textPanel.tHeight;
      textPanel.erase();
      textPanel = new TextPanelFinished(_x,_y,_w,_h);
    }
    if (textPanel!=null) textPanel.display();
    if (infoPanel!=null) infoPanel.display();
    if (hoverInfo!=null) {
      boolean result = hoverInfo.display(textPanel.menu);
      if (!result) hoverInfo.display(infoPanel.menu);
    }
  }
  
  void erase(){
    if (textPanel!=null) textPanel.erase();
    if (infoPanel!=null) infoPanel.erase();
  }
}

// This is the panel that will display the text
class TextPanel{
  int tX,tY,tWidth,tHeight;
  PFont f;
  Menu menu;
  
  TextPanel(int _tX,int _tY, int _tWidth,int _tHeight){
    tX = _tX;
    tY = _tY;
    tWidth= _tWidth;
    tHeight= _tHeight;
    f=fBold;
    menu = loadMenu();
  }
  
  Menu loadMenu(){
    return null;
  }
  
  // cleans the panel
  void preparePanel(){
    //fill(colorLine);
    stroke(colorLine);
    fill(colorBackground1);
    //rect(tX,tY,tWidth,tHeight);
    rect(tX,tY,tWidth,tHeight); 
  }
  
  // displays the figures
  void display(){
  }
  
  void erase(){
    if (menu!=null) menu.eraseMenu();
  } 
}

class TextPanelFinished extends TextPanel{
  
  TextPanelFinished(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
  }
  
  Menu loadMenu(){
    return new MenuPanelFinished(tX,tY,tWidth,tHeight);
  }
  
  void display(){
    String s;
    color color1,color2;
    
    preparePanel();
    textFont(f,14);
    noStroke();
    
    /*int textY=tY+(panelHeight+14-5)/2;
    if (gameSequence.win){
      color1=colorWin;
      //color2=colorBackground1;
      s = s_Text_PanelFinished_Win;
    }
    else {
      color1=colorLoose;
      //color2=colorBackground1;
      s = s_Text_PanelFinished_Loose;
    }
    
    fill(color1);
    textAlign(CENTER);
    text(s,tX+tWidth/2,textY);*/
  }
}

// This is the panel that will display the text
class TextPanelStrategy extends TextPanel{
  
  TextPanelStrategy(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
  }
  
  Menu loadMenu(){
    return new MenuPanelStrategy(tX,tY,tWidth,tHeight);
  }
  
  // displays the figures
  void display(){
    String s;
    
    preparePanel();
    
    textFont(f,14);
    noStroke();
    
    int textY=tY+(panelHeight+14-5)/2;

    fill(colorText);

    int textX=panelButtonWidth+5+1;
    if (world.weaponStock.ammo<world.weaponStock.cost()) fill(colorDisabled);
    else fill(colorTextHighlight);
    s=world.weaponStock.name();
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,panelTextWidth,20);
   
    textX+=panelTextWidth;
    fill(colorText);
    if (limitedAmmo) s= s_Text_Panel_Cost+ world.weaponStock.cost();
    else s= s_Text_Panel_CostUnlimited;
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,costTextWidth,20);
   
    textX+=costTextWidth+panelButtonWidth+5+1;
    fill(colorText);
    if (limitedAmmo) s= s_Text_Panel_Ammo + world.weaponStock.ammo;
    else s= s_Text_Panel_AmmoUnlimited;
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,tWidth-textX-(menuButtonWidth+5),20);
  }
}

class TextPanelAction extends TextPanel{
  String weaponNameLeft,weaponNameRight;
  int costLeft,costRight;
  
  TextPanelAction(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
    world.weaponStock.weaponCycle=0;
    weaponNameLeft=world.weaponStock.name();
    costLeft=world.weaponStock.cost();
    world.weaponStock.weaponCycle=1;
    weaponNameRight=world.weaponStock.name();
    costRight=world.weaponStock.cost();
  }
  
  Menu loadMenu(){
    return new MenuPanelAction(tX,tY,tWidth,tHeight);
  }
  
  // displays the figures
  void display(){
    String s;
    
    preparePanel();
    
    textFont(f,14);
    noStroke();
    
    int textY=tY+(panelHeight+14-5)/2;
    if (gameSequence.freeze){
      fill(blink.gradual(colorBackground1,colorText,5));
      s = s_Text_PanelAction_Paused;
      textAlign(CENTER);
      text(s,tX+tWidth/2,textY);
    }
    else {
      fill(colorText);    
      int textX=5;
      s= s_Text_PanelAction_Left;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX += textWidth(s)+5;
      if (world.weaponStock.ammo<costLeft) fill(colorDisabled);
      else fill(colorTextHighlight);
      s=weaponNameLeft;
      textAlign(LEFT);
      text(s,textX,textY);
     
      textX += textWidth(s)+15;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Cost+ costLeft;
      else s= s_Text_Panel_CostUnlimited;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX+= textWidth(s)+menuButtonWidth;
      s= s_Text_PanelAction_Right;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX += textWidth(s)+5;
      if (world.weaponStock.ammo<costRight) fill (colorDisabled);
      else fill(colorTextHighlight);
      s=weaponNameRight;
      textAlign(LEFT);
      text(s,textX,textY);
     
      textX+=textWidth(s)+15;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Cost+ costRight;
      else s= s_Text_Panel_CostUnlimited;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX+=panelButtonWidth;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Ammo + world.weaponStock.ammo;
      else s= s_Text_Panel_AmmoUnlimited;
      textAlign(CENTER,BOTTOM);
      text(s,textX,textY-17,tWidth-textX-(menuButtonWidth+5),20);
    }
  }
}

class MenuPanelStrategy extends Menu{
  int x,y,w,h;
    
  MenuPanelStrategy(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-5;
    cp5.addButton("Menu_PanelStrategy_NextTurn")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_NextTurn)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_NextTurn",desc_Menu_PanelStrategy_NextTurn,desc_Menu_PanelStrategy_NextTurn_Unavailable));
    buttonStatus("Menu_PanelStrategy_NextTurn",Button_Data_Unavailable);
    
    bW = 5;
    cp5.addButton("Menu_PanelStrategy_PreviousWeapon")
       .setPosition(bW,bH)
       .setSize(panelButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_PreviousWeapon)
       .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_PreviousWeapon",desc_Menu_PanelStrategy_PreviousWeapon));
    
    bW += panelButtonWidth+2+panelTextWidth+costTextWidth;
    cp5.addButton("Menu_PanelStrategy_NextWeapon")
       .setPosition(bW,bH)
       .setSize(panelButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_NextWeapon)
       .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_NextWeapon",desc_Menu_PanelStrategy_NextWeapon));
  }
}

class MenuPanelFinished extends Menu{
  int x,y,w,h;
    
  MenuPanelFinished(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-textMargin;
    cp5.addButton("Menu_PanelFinished_KeepGoing")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_KeepGoing)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_KeepGoing",desc_Menu_PanelFinished_KeepGoing));
    
    bW = textMargin;
    cp5.addButton("Menu_PanelFinished_BackMenu")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_BackMenu)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_BackMenu",desc_Menu_PanelFinished_BackMenu));
    
    bW += menuButtonWidth+textMargin;
    cp5.addButton("Menu_PanelFinished_Restart")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_Restart)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_Restart",desc_Menu_PanelFinished_Restart));
  }
}

class MenuPanelInfo extends Menu{
  int x,y,w,h;
    
  MenuPanelInfo(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;

    createMenuBis();
  }
  
  void display(){
    fill(colorBackground1);
    stroke(colorLine);
    rect(x,y,w,h);
  }
  
  void createMenuBis(){
    int bH = y+textMargin;
    int bW = x+textMargin;
    cp5.addButton("Menu_PanelInfo_Stat")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Stat)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Stat",desc_Menu_PanelInfo_Stat));
    
    bH += menuButtonHeight+1;
    cp5.addButton("Menu_PanelInfo_Mission")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Mission)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Mission",desc_Menu_PanelInfo_Mission));
    
    bH += menuButtonHeight+1;
    cp5.addButton("Menu_PanelInfo_Score")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Score)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Score",desc_Menu_PanelInfo_Score));
  }
  

}

class MenuPanelAction extends Menu{
  int x,y,w,h;
    
  MenuPanelAction(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-5;
    cp5.addButton("Menu_PanelAction_Pause")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelAction_Start)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelAction_Pause",desc_Menu_PanelAction_Pause));
  }
}

class InfoPanel{
  InfoArea infoArea;
  Menu menu;
  World wld;
  int infoX, infoY, infoW, infoH;
  int menuWidth;
  int type;
    
  InfoPanel(int x, int y, int w, int h, World _wld){
    menuWidth=menuButtonWidth+2*textMargin;
    menu = new MenuPanelInfo(x,y,menuWidth,h);
    infoX=x+menuWidth;
    infoY=y;
    infoW=w-infoX;
    infoH=h;
    wld=_wld;
    changeInfoPanel(infoPanelRef_Mission);
  }
  
  void display(){
    menu.display();
    infoArea.display();
  }
  
  void erase(){
    if (menu!=null) menu.eraseMenu();
  }
  
  void changeInfoPanel(int infoPanelRef){
    type = infoPanelRef;
    if (infoPanelRef==infoPanelRef_Statistics) {
      infoArea = new InfoAreaStatistics(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Stat");
    }
    else if (infoPanelRef==infoPanelRef_Mission) {
      infoArea = new InfoAreaMission(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Mission");
    }
    else if (infoPanelRef==infoPanelRef_Score) {
      infoArea = new InfoAreaScore(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Score");
    }
    else {
      infoArea = null;
      menu.deselectButtons();
    }
  }
  
  void transitionInfoPanel(int infoPanelRef,String s,color colour){
    type = infoPanelRef;
    if (infoPanelRef==infoPanelRef_TransitionWin) {
      infoArea = new InfoAreaFinished(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Victory,s,colour,infoPanelRef_Score);
      menu.deselectButtons();
    }
    else if (infoPanelRef==infoPanelRef_TransitionLoose) {
      infoArea = new InfoAreaFinished(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Failure,s,colour,infoPanelRef_Score);
      menu.deselectButtons();
    }
    else if (infoPanelRef==infoPanelRef_TransitionWarning) {
      infoArea = new InfoAreaTransition(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Warning,s,colour,infoPanelRef_Mission);
      menu.deselectButtons();
    }
    else {
      infoArea = null;
      menu.deselectButtons();
    }
  }
}

class InfoArea{
  int x,y,h,w;
  World wld;
  PFont fText;
  int tSize = hoverTextSize;
  int tLeading = hoverTextInterline;
  int hText,maxLine;
  
  InfoArea(){
  }
  
  InfoArea(int _x, int _y, int _w, int _h, World _wld){
    wld = _wld;
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    hText=tSize+tLeading;
    maxLine=h/(hText-2*textMargin);
    fText=fNormal;
  }
  
  void cleanDisplay(){
    fill(colorBackground2);
    stroke(colorLine);
    rect(x,y,w,h);
  }
  
  void display(){
    cleanDisplay();
  }
  
  // returns the x according to the position ref.
  // there are 8 possible starting positions for the text
  // It is on purpose that it is fixed width : I don't want the text to widden too much if the screen is.
  int xPosition(int _ref){
    int ref=max(0,min(7,_ref));
    return x+textMargin+ref*infoWidthPosition;
  }
  
  // returns the y according to the line ref.
  int yPosition(int _ref){
    int ref=max(0,min(maxLine,_ref));
    return y+textMargin+ref*hText;
  }
  
  int xWidth(int _ref){
    int ref=max(1,min(8,_ref));
    return infoWidthPosition*ref;
  }
  
  boolean checkInside(int _x, int _y){
    if (_x>=x && _x<=x+w && _y>=y && _y<=y+h) return true;
    else return false;
  }
}

class InfoAreaStatistics extends InfoArea{
  Graph graph;
  
  InfoAreaStatistics(int _x, int _y, int _w, int _h,  World _wld){
    super(_x,_y,_w,_h,_wld);
    graph = new Graph(x+1,y+1,w-2,h-2,wld.datas);
    graph.bgColor(colorBackground2);
    graph.drColor(colorText);
    graph.draw();
  }
  
  void display(){
    
    if (clock%2==0) {
      cleanDisplay();
      graph.draw();
    }
  }
}

class InfoAreaMission extends InfoArea{
  
  InfoAreaMission(int _x, int _y, int _w, int _h, World _wld){
    super(_x,_y,_w,_h,_wld);
  }
  
  void display(){
    cleanDisplay();
    int j=0;
   
    // prepares the font
    textFont(fText,tSize);
    textAlign(LEFT,CENTER);
    int number = mission.events.size();
     
    fill(colorWin);
    text(s_InfoMission_WinningCondition,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Win){
        displayEvent(event,j);
        j++;
      }
    }
    
    j++;
    fill(colorLoose);
    text(s_InfoMission_LoosingCondition,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Loose){
        displayEvent(event,j);
        j++;
      }
    }
    
    j++;
    fill(colorMisc);
    text(s_InfoMission_Misc,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Misc){
        displayEvent(event,j);
        j++;
      }
    }
  }
  
  void displayEvent(Event event,int j){
    String s;
    color color1, color2,color3;
    
    if (event.type==EventType_Misc) s=event.description;
    else s=event.condition.description();
    
    if (event.active){
      if (event.blink && !gameSequence.finished){
        color1=colorDisabled;
        color2=blink.flash(colorText,colorDisabled,5);
        color3=color2;
      }
      else {
        color1=colorDisabled;
        color2=colorText;
        color3=colorDisabled;
      }
    }
    else {
      if (event.done){
        color1=colorDisabled;
        if (event.type==EventType_Misc && event.score()==0) color2=colorDisabled;
        else color2=mission.getEventColor(event.type);
        color3=mission.getEventColor(event.type);
      }
      else {
        color1=colorDisabled;
        color2=colorDisabled;
        color3=color2;
      }
    }
    fill(color2);
    text(s,xPosition(2),yPosition(j),xWidth(4),hText);
    String s2= event.progression()+"";
    text(s2,xPosition(6),yPosition(j),xWidth(1),hText);
    fill(color1);
    text(event.target(),xPosition(6)+textWidth(s2),yPosition(j),xWidth(1),hText);
    fill(color3);
    if (event.score()!=0) text(event.score()+"pt",xPosition(7),yPosition(j),xWidth(1),hText);
    
  }
}

class InfoAreaScore extends InfoArea{
  
  String message;
  
  InfoAreaScore(int _x, int _y, int _w, int _h,World _wld){
    super(_x,_y,_w,_h,_wld);
  }
  
  void display(){
    cleanDisplay();
    
    
    if (!gameSequence.finished){
      textFont(fText,tSize);
      fill(colorText);
      textAlign(CENTER,CENTER);
      text(s_InfoScore_Unfinished,x,y,w,h);
    }
    else {
      ScoreItem item;
      textFont(fBold,tSize);
      fill(colorText);
      textAlign(CENTER,CENTER);
      int j=0;
      text(s_InfoScore_Number,xPosition(3),yPosition(j),xWidth(1),hText);
      text(s_InfoScore_Multiplier,xPosition(4),yPosition(j),xWidth(1),hText);
      text(s_InfoScore_Score,xPosition(5),yPosition(j),xWidth(1),hText);
      j++;
      textFont(fText,tSize);
      for (int i=0;i<mission.scoreItems.size();i++){
        item = (ScoreItem) mission.scoreItems.get(i);
        if (i==mission.scoreItems.size()-1){
          if (gameSequence.win)fill(colorWin);
          else fill(colorLoose);
        }//fill(colorText);
        textAlign(LEFT,CENTER);
        text(item.itemText()+" : ",xPosition(0),yPosition(j),xWidth(3),hText);
        textAlign(CENTER,CENTER);
        text(item.value(),xPosition(3),yPosition(j),xWidth(1),hText);
        text(item.multiplier(),xPosition(4),yPosition(j),xWidth(1),hText);
        text(item.score(),xPosition(5),yPosition(j),xWidth(1),hText);
        j++;
      }
      
      fill(colorText);
      textFont(fBold,tSize);
      text(s_InfoScore_FinalScore,xPosition(4),yPosition(j),xWidth(1),hText);
      //fill(colorText);
      //fill(blink.gradual(colorText,colorBackground2,5));
      text(mission.finalScore+"",xPosition(5),yPosition(j),xWidth(1),hText);
      textAlign(LEFT,CENTER);
      textFont(fText,tSize);
    }
  }
}

class InfoAreaTransition extends InfoArea{
  String s1,s2;
  color colour;
  Blink blinkLocal;
  int nextType;
  
  InfoAreaTransition(int _x, int _y, int _w, int _h, World _wld,String _s1,String _s2,color _colour,int _nextType){
    super(_x,_y,_w,_h,_wld);
    s1=_s1;
    s2=_s2;
    colour=_colour;
    blinkLocal = new Blink();
    nextType=_nextType;
  }
  
  void display(){
    cleanDisplay();
    
    textAlign(CENTER,CENTER);
    textFont(fText,tSize*2);
    fill(blinkLocal.gradual(colour,colorBackground2,12));
    text(s1,x,y,w,hText*4);
    textFont(fText,(tSize*3)/2);
    fill(colour);
    text(s2,x,yPosition(4),w,hText*2);
    checkNextPanel();
    
    blinkLocal.forward();
  }
  
  void checkNextPanel(){
    if (blinkLocal.checkCycle(6)) gameScreen.infoPanel.changeInfoPanel(nextType);
  }
}

class InfoAreaFinished extends InfoAreaTransition{
  
  InfoAreaFinished(int _x, int _y, int _w, int _h, World _wld,String _s1,String _s2,color _colour,int _nextType){
    super(_x,_y,_w,_h,_wld,_s1,_s2,_colour,_nextType);
  }
  
  void display(){
    super.display();
    textFont(fText,tSize);
    fill(colorText);
    textAlign(CENTER,CENTER);
    text(s_Menu_PanelInfo_FinishedScore,x,yPosition(7),w,hText);
  }
  
  void checkNextPanel(){
  }
}

// This class is the area where things are happening
class PlayArea{
  int gX,gY,gWidth,gHeight;
  
  PlayArea(int _gX, int _gY, int _gWidth, int _gHeight){
    gX = _gX;
    gY = _gY;
    gWidth=_gWidth;
    gHeight=_gHeight;
  }
  
  // Creates and draw the map (without the beings)
  void drawMap(){
    int x,y,w,h;
    
    fill(wall);
    rect(0,0,gWidth,gHeight); // Fills the whole area with walls
    fill(wall);
    stroke(empty);
    for (int i=0; i<100; i++){ // Builds the passages (100 rectangles)
      x=(int)random(gWidth/scale)*scale-(gWidth/8/scale)*scale;
      y=(int)random(gHeight/scale)*scale-(gHeight/8/scale)*scale;
      w=((int)random(gWidth/4/scale)*scale+2)*scale-1;
      h=((int)random(gWidth/4/scale)*scale+2)*scale-1;
      for (int j=0;j<scale;j++) rect(x+j,y+j,w-2*j,h-2*j);
    }
 
    fill(empty);
    //noStroke();
    stroke(empty);
    for (int i=0; i<45; i++){ // Builds 45 rooms
      x=(int)random(gWidth/scale)*scale;
      y=(int)random(gHeight/scale)*scale;
      w=((int)random(40)+10)*scale-1;
      h=((int)random(40)+10)*scale-1;
      rect(x,y,w,h); 
      //x=(int)random(gWidth-1)+1;
      //y=(int)random(gHeight-1)+1;
    }
  }
  
  // Check if the given coordinates are inside the playArea
  boolean inside(int x, int y){
    if (x>=gX && x<gX+gWidth && y>=gY && y<gY+gHeight) return true;
    return false;
  }
  
  // gives an x value that is inside the playArea 
  int setX(){
    return gX+(int)random(gWidth/scale)*scale;
  }
  
  // gives an y value that is inside the playArea
  int setY(){
    return gY+(int)random(gHeight/scale)*scale;
  }
}

