// The classes in the file define the generic menu -> everywhere where there are some buttons
// It is later used in GameScreen and InitScreen


// Class that will hold some extra info about buttons usefull for the display of comment,
// and maybe manage the interaction.
class ButtonInfo {
  String name;
  String description; // description when overing the button
  String descriptionUnavailable; // description when overing the button when unavailable
  int data; // data related to the status of the button (highlighted, disabled,...)
  int data2; // some more data. Such as weapons ref
  
  ButtonInfo(String _name, String _description){
    name=_name;
    description=_description;
    descriptionUnavailable="";
    data=Button_Data_Normal;
    data2=0;
  }
  
  ButtonInfo(String _name, String _description, String _descriptionUnavailable){
    name=_name;
    description=_description;
    descriptionUnavailable=_descriptionUnavailable;
    data=Button_Data_Normal;
    data2=0;
  }
  
  
  ButtonInfo(String _name, String _description, String _descriptionUnavailable, int _data){
    name=_name;
    description=_description;
    descriptionUnavailable=_descriptionUnavailable;
    data=_data;
    data2=0;
  }
  
  String name(){
    return name;
  }
  
  String description(){
    if ((data == Button_Data_Normal) || (data == Button_Data_Selected)) return description;
    else if ( data == Button_Data_Unavailable) return descriptionUnavailable;
    else return desc_Button_Unknown;
  }
}

class HoverInfo{
  int x, y, w, h,wMargin,hMargin;
  PFont fText;
  int tSize = 12;
  int tLeading = tSize+3;
  
  HoverInfo(int _x, int _y, int _w, int _h, int _wMargin, int _hMargin){
    x= _x;
    y= _y;
    w= _w;
    h= _h;
    wMargin= _wMargin;
    hMargin= _hMargin;
    
    fText=fNormal; 
  }
  
  boolean display(Menu menu){
    noStroke();
    fill(colorHoverField);
    rect(x,y,w,h);
    
    textFont(fText,tSize);
    textLeading(tLeading);
    fill(colorHoverText);
    if (fps){
      textAlign(LEFT,CENTER);
      text("fps: "+decimalFormat.format(frameRate),x+hoverHeightMargin,y+hMargin,w-2*wMargin,h-2*hMargin);
    }
    if (menu!=null){
      if (cp5.isMouseOver()){
        String s ="";
        for (int i=0;i<menu.itemLength();i++){
          if (cp5.isMouseOver(cp5.getController(menu.itemName(i)))){
            s = menu.itemDescription(i);
            break;
          }
        }
        if (s!=""){
          textAlign(CENTER,CENTER);
          text(s,x+wMargin,y+hMargin,w-2*wMargin,h-2*hMargin);
          return true;
        }
      }
    }
    return false;
  }
}

// This class creates generic menus
class Menu{
  
  ArrayList buttonInfo;
  
  Menu(){
    buttonInfo= new ArrayList();
    createMenu();
  }

  void createMenu(){
  }

  int itemLength(){
    return buttonInfo.size();
  }

  void eraseMenu(){
    for (int i=0; i<itemLength();i++) {
      ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
      // Added condition bellow, because in mutliList, many buttons are erased automatically
      // So trying to erase them again causes errors.
      // Could be improved If I knew the type returned by cp5.controller();
      if (cp5.controller(bInfo.name)!=null) cp5.controller(bInfo.name).remove();
    }
  }
  
  // Finds the ButtonInfo in the arrayList which as the same name. Otherwise, returns null
  ButtonInfo findItem(String _name){
      ButtonInfo bInfo = null;
      
      for (int i=0; i<buttonInfo.size(); i++){
        bInfo = (ButtonInfo) buttonInfo.get(i);
        if (bInfo.name.equals(_name)) {
          return bInfo;
        }
      }
      return null;
  }
  
  // Changes the color of the botton, according to its status
  // Stores its status as well
  void buttonStatus(String buttonName, int bStatus){
    ButtonInfo bInfo = findItem(buttonName);
    if (bInfo!=null) {
      bInfo.data = bStatus;
    }
    
    Controller button = cp5.controller(buttonName);
    if (button != null){
      if (bStatus==Button_Data_Selected) {
        //button.unlock();
        button.setColorBackground(color(125,125,0));
        button.setColorForeground(color(175,175,0));
        button.setColorActive(color(200,200,50));
      }
      else if (bStatus==Button_Data_Normal) {
        //button.unlock();
        button.setColorBackground(color(2,52,77));
        button.setColorForeground(color(1,108,158));
        button.setColorActive(color(0,180,234));
      }
      else if (bStatus==Button_Data_Unavailable) {
        //button.lock(); // Actually, I don't want to lock, otherwise, I can't know that I am hovering them.
        button.setColorBackground(color(30,30,30));
        button.setColorForeground(color(60,60,60));
        button.setColorActive(color(60,60,60));
      }
    }
  }
  
  void selectOneButton(String s){
    for (int i=0; i<buttonInfo.size();i++){
      if (s.equals(itemName(i))) buttonStatus(itemName(i),Button_Data_Selected);
      else buttonStatus(itemName(i),Button_Data_Normal);
    }
  }
  
  void deselectButtons(){
    for (int i=0; i<buttonInfo.size();i++){
      buttonStatus(itemName(i),Button_Data_Normal);
    }
  }
  
  void updateListButtons(String name,int listSize, int ref){
    ButtonInfo bInfo;
    for (int i = 0; i<listSize ; i++){
       if (cp5.controller(name+" "+i)!=null) {
         bInfo = findItem(name+" "+i);
         if (i == ref) buttonStatus(name+" "+i,Button_Data_Selected);
         else buttonStatus(name+" "+i,Button_Data_Normal);   
       }  
    }
  }
  
  // Returns the name of a button
  String itemName(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.name();
  }
  
  // Retunrs the description of a button
  String itemDescription(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.description();
  }
  
  // Returns the data value of a button
  int itemData(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.data;
  }
  
  // Allows the menu to display things that needs to be refreshed. Such as text or other things.
  void display(){
  }
  
  // set default value to menu. Nothing to do in that specific menu, but more in others
  void defaultValues(){
  }
  
  // get the values from the menu. Nothing to do in that specific menu, but more in others
  void getValues(){
  }
}
