int EventType_Misc = 0;
int EventType_Win = 1;
int EventType_Loose = 2;
int ConditionResult_True = 0;
int ConditionResult_False = 1;
int ConditionResult_Warning = 2;

int scoreHumansAlive = 2;
int scoreHumansKilled = -5;
int scoreZombiesAlive = -200;
int scoreZombiesAliveMini = -2;
int scoreZombiesKilled = 1;


class Mission{
  ArrayList events;
  ArrayList scoreItems;
  ArrayList missionDefinitions;
  World wld;
  int finalScore;
  int specialScore,specialScoreNumber;
  int victoryScore;
  int missionRef;
  String victoryMessage;
  
  
  Mission(World _wld){
    wld=_wld;
    missionDefinitions = loadDefinitions();
    missionRef=0;
    setMission(missionRef);
    specialScore=0;
    specialScoreNumber=0;
    victoryScore=0;
    victoryMessage="";
  }
  
  void clean(){
    specialScore=0;
    specialScoreNumber=0;
    victoryScore=0;
    victoryMessage="";
    scoreItems=null;
    setMission(missionRef);
  }
  
  ArrayList loadDefinitions(){
    ArrayList mDefs = new ArrayList();
    
    mDefs.add(new MissionVanilla(this,wld));
    mDefs.add(new MissionRegular(this,wld));
    mDefs.add(new MissionSecond(this,wld));
    mDefs.add(new MissionThird(this,wld));
    mDefs.add(new MissionMadScientist(this,wld));
    
    return mDefs;
  }
  
  void setMission(int i){
    
    if (missionDefinitions!=null){
      int ref = max(0,min(i,missionDefinitions.size()-1));
      missionRef=ref;
      MissionDefinition missionDef = (MissionDefinition) missionDefinitions.get(ref);
      events = missionDef.create();
    }
    else events=null;
  }
  
  // Check every conditions.
  void checkEvent(){
    Event event;
    for (int i=0;i<events.size();i++){
      event = (Event) events.get(i);
      event.check();
    }
  }
  
  color getEventColor(int eventType){
    if (eventType==EventType_Win)  return colorWin;
    else if (eventType==EventType_Loose) return colorLoose;
    else return colorMisc;
  }
  
  void score(){
    
    finalScore=0;
    scoreItems = new ArrayList();
    
    ScoreItem item;
    item = new ScoreItem(s_Score_HumansAlive,wld.numHuman,scoreHumansAlive);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_HumansKilled,wld.numKilledHumans+wld.numInoculatedHumans,scoreHumansKilled);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_ZombiesAlive,wld.numZombies,multiplierZombie(wld.numZombies));  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_ZombiesKilled,wld.numKilledZombies,scoreZombiesKilled);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItemSpecial(s_Score_Special,specialScoreNumber,specialScore);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItemVictory(s_Score_MissionAccomplishment,victoryMessage,victoryScore);  
    scoreItems.add(item);
    finalScore+=item.score;
    
  }
  
  void specialScore(int _specialScore){
    specialScore+=_specialScore;
    specialScoreNumber++;
  }
  
  void victoryScore(int _victoryScore,boolean _victory){
    victoryScore=_victoryScore;
    if (_victory) victoryMessage = s_Score_Won;
    else victoryMessage = s_Score_Lost;
  }
  
  int multiplierZombie(int numZombies){
    if (numZombies>0) return (int) min(scoreZombiesAlive/sqrt(wld.numZombies),scoreZombiesAliveMini);
    else return 0; 
  }
}

class MissionDefinition{
  String name,description;
  Mission miss;
  World wld;
  
  MissionDefinition(Mission _miss,World _wld){
    miss=_miss;
    wld=_wld;
    name="";
    description="";
  }
  
  ArrayList create(){
    return null;
  }
  
}

class MissionRegular extends MissionDefinition{
  
  MissionRegular(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Regular;
    description=desc_Mission_Regular;
  }
  
  ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,20),new ActionLost(-2000),miss);
    events.add(event);
    event = new Event(new CondTime(wld,900),new ActionBombZombies(wld,-1000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionMadScientist extends MissionDefinition{
  
  MissionMadScientist(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_MadScientist;
    description=desc_Mission_MadScientist;
  }
  
  ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,3),miss);
    events.add(event);
    event = new Event(new CondTime(wld,90),new ActionInfect(wld,1),miss);
    events.add(event);
    event = new Event(new CondTime(wld,180),new ActionInfect(wld,1),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionLost(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,0),new ActionWin(-2000),miss);
    events.add(event);
    event = new Event(new CondTime(wld,500),new ActionBombZombies(wld,1000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionSecond extends MissionDefinition{
  
  MissionSecond(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Second;
    description=desc_Mission_Second;
  }
  
  ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondTime(wld,90),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondTime(wld,180),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,5),new ActionLost(-2000),miss);
    events.add(event);
    event = new Event(new CondHumanKilledMax(wld,5),new ActionLost(-2000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionThird extends MissionDefinition{
  
  MissionThird(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Third;
    description=desc_Mission_Third;
  }
  
  ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondZombieMax(wld,30),new ActionLost(-2000),miss);
    events.add(event);

    
    return events;
  }
}

class MissionVanilla extends MissionDefinition{
  
  MissionVanilla(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Vanilla;
    description=desc_Mission_Vanilla;
  }
  
  ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,0),new ActionLost(-2000),miss);
    events.add(event);

    
    return events;
  }
}

class ScoreItem{
  String itemText;
  int value, multiplier, score;
  
  ScoreItem(){
  }
  
  ScoreItem(String _text,int _value,int _multiplier){
    itemText= _text;
    value=_value;
    multiplier=_multiplier;
    score=value*multiplier;
  }
  
  String itemText(){
    return itemText;
  }
  
  String value(){
    String s=value+"";
    return s;
  }
  
  String multiplier(){
    String s="( x "+multiplier+" )";
    return s;
  }
  
  String score(){
    String s=score+"";
    return s;
  }
}

class ScoreItemSpecial extends ScoreItem{
  
  ScoreItemSpecial(String _text,int _value, int _score){
    itemText= _text;
    value=_value;
    score=_score;
  }
  
  String multiplier(){
    return "";
  }
  
}

class ScoreItemVictory extends ScoreItem{
  String sValue;
  
  ScoreItemVictory(String _text,String _value, int _score){
    itemText= _text;
    sValue=_value;
    score=_score;
  }
  
  String value(){
    return sValue;
  }
  
  String multiplier(){
    return "";
  }
  
}


class Event{
  Condition condition;
  Action action;
  boolean active; // If active, event will be checked
  boolean done; // Tells that this event has been executed once already
  boolean blink; // When displayed, that event should be blinking
  int type;
  String description,description2;
  Mission miss;
  
  Event(Condition _condition,Action _action,Mission _mission){
    condition = _condition;
    action = _action;
    active = true;
    blink = false;
    done = false;
    miss=_mission;
    type=action.type();
    description=condition.description()+" - "+action.description();
    description2=condition.descWarning();
  }
  
  // Check every conditions. Sends "true" i
  void check(){
    if (active){
      int result = condition.evaluate();
      if (result == ConditionResult_True) {
        active=false;
        if (!gameSequence.finished || (type != EventType_Loose && type != EventType_Win)) done=true;
        action.act(this);
      }
      else if (!blink && result == ConditionResult_Warning){ // starts blinking if condition is met
        blink = true;
        gameSequence.warning(description2,type);
      }
      else if (blink && result == ConditionResult_False){ // allows to stop blinking if the condition is not met anymore
        blink = false;
      }
    }
  }
  
  int progression(){
    return condition.progression();
  }
  
  int score(){
    return action.score();
  }
  
  String target(){
    return condition.target();
  }
}

class Action{
  String description;
  int type;
  int score;
  
  Action(){
    type=EventType_Misc;
    description=s_Event_None;
    score=0;
  }
  
  void act(Event event){
  }
  
  int type(){
    return type;
  }
  
  int score(){
    return score;
  }
  
  String description(){
    return description;
  }
  
  
}

class ActionInfect extends Action{
  int number;
  World wld;
  
  ActionInfect(World _wld,int _number){
    wld =_wld;
    number=_number;
    type= EventType_Misc;
    description=s_Event_Infect_1+number+s_Event_Infect_2;
  }
  
  void act(Event event){
    int j=0;
    int i=0;
    
    while (i<number && j<wld.initPop){
      while (!(wld.beings[j] instanceof Human)&&j<wld.initPop){
        j++;
      } 
      if (j<wld.initPop){
        wld.beings[j].infect(0,iszombie);
        i++;
        j++;
      }
    }
  }
}

class ActionWin extends Action{
  
  ActionWin(int _score){
    type=EventType_Win;
    score=_score;
  }
  
  void act(Event event){
    event.miss.victoryScore(score,true);
    gameSequence.won(event.condition.descDone());
  }
}

class ActionLost extends Action{
  
  ActionLost(int _score){
    type=EventType_Loose;
    score=_score;
  }
  
  void act(Event event){
    event.miss.victoryScore(score,false);
    gameSequence.lost(event.condition.descDone());
  }
}

class ActionBombZombies extends Action{
  World wld;
 
  ActionBombZombies(World _wld,int _score){
    wld=_wld;
    type=EventType_Misc;
    score=_score;
    description=s_Event_BombZombies;
  }
  
  void act(Event event){
    wld.effects.add(new EffectBombing(wld,1,100));
    if (score!=0) event.miss.specialScore(score);
  }
}

class Condition{
  World wld;
  String description;
  String descWarning;
  String descDone;
  int progression;
  String target;
  
  Condition(World _wld){
    wld = _wld;
    description=s_Event_None;
    descWarning=s_Event_None;
    descDone=s_Event_None;
    progression=0;
    target="";
  }
  
  int evaluate(){
    return ConditionResult_False;
  }
  
  String description(){
    return description;
  }
  
  String descWarning(){
    return descWarning;
  }
  
  String descDone(){
    return descDone;
  }
  
  int progression(){
    return progression;
  }
  
  String target(){
    return target;
  }
}

// Condition returns true if more than proportion of the population has been turned into zombies
class CondZombieDead extends Condition{
  
  CondZombieDead(World _wld){
    super(_wld);
    description=s_Event_Condition_ZombieDead;
    descDone=d_Event_Condition_ZombieDead;
    target="";
  }
  
  int evaluate(){
    //Data data = (Data) wld.datas.get(dataZombie);
    progression = wld.numZombies;
    if (wld.time>1 && wld.numZombies==0) return ConditionResult_True;
    return ConditionResult_False;
  }
}

// Condition returns true if more than x zombies
class CondZombieMax extends Condition{
  int number;
  int warning;
  
  CondZombieMax(World _wld, int _proportion){
    super(_wld);
    number=_proportion*wld.initPop/100;
    warning=number*80/100;
    description=s_Event_Condition_ZombieMax_1+number+s_Event_Condition_ZombieMax_2;
    descWarning=w_Event_Condition_ZombieMax_1+number+w_Event_Condition_ZombieMax_2;
    descDone=d_Event_Condition_ZombieMax_1+number+d_Event_Condition_ZombieMax_2;
    target="/"+number;
  }
  
  int evaluate(){
    progression=wld.numZombies;
    if (wld.numZombies>number) return ConditionResult_True;
    else if (wld.numZombies>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

// Condition returns true if killed more than x humans
class CondHumanKilledMax extends Condition{
  int number;
  int warning;
  
  CondHumanKilledMax(World _wld, int _proportion){
    super(_wld);
    number=_proportion*wld.initPop/100;
    warning = number*80/100;
    description=s_Event_Condition_HumanKilledMax_1+number+s_Event_Condition_HumanKilledMax_2;
    descWarning=w_Event_Condition_HumanKilledMax_1+number+w_Event_Condition_HumanKilledMax_2;
    descDone=d_Event_Condition_HumanKilledMax_1+number+d_Event_Condition_HumanKilledMax_2;
    target="/"+number;
  }
  
  int evaluate(){
    progression=wld.numKilledHumans+wld.numInoculatedHumans;
    if (wld.numKilledHumans+wld.numInoculatedHumans>number) return ConditionResult_True;
    else if (wld.numKilledHumans+wld.numInoculatedHumans>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

// Condition returns true if a proportion of the population is dead
class CondHumanDead extends Condition{
  int number;
  int warning;
  
  CondHumanDead(World _wld,int _proportion){
    super(_wld);
    
    number=_proportion*wld.initPop/100;
    warning=(number*150+wld.initPop*10)/100;
    description=s_Event_Condition_HumanDead_1+number+s_Event_Condition_HumanDead_2;
    descWarning=w_Event_Condition_HumanDead_1+number+w_Event_Condition_HumanDead_2;
    descDone=d_Event_Condition_HumanDead_1+number+d_Event_Condition_HumanDead_2;
    target="/"+number;
  }
  
  int evaluate(){
    progression=wld.numHuman;
    if (wld.numHuman<=number) return ConditionResult_True;
    else if (wld.numHuman<=warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

class CondTime extends Condition{
  int time;
  int warning;
  
  CondTime(World _wld,int _time){
    super(_wld);
    time = _time;
    warning=time*80/100;
    description=s_Event_Condition_Time_1+time+s_Event_Condition_Time_2;
    descWarning=w_Event_Condition_Time_1+time+w_Event_Condition_Time_2;
    descDone=d_Event_Condition_Time_1+time+d_Event_Condition_Time_2;
    target="/"+time;
  }
  
  int evaluate(){
    progression=wld.time;
    if (wld.time>time) return ConditionResult_True;
    else if (wld.time>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}
