import processing.core.*; 
import processing.xml.*; 

import controlP5.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class ZombieInfection extends PApplet {

// Zombie Infection Simulation
// Kevan Davis, 16/8/03

// Modified by John Gilbertson 25/09/03
// just making zombies more dangerous, edited level creation code, increased max zombies, made zombies less likely to group etc...

// Modified by Fabien Vidal 05/12/2012 v.13 (11/05/12 V.1.0)
// Heavy work on the structure. Multiple weapons, UI, missions and graph
// 
//

 // Library used to add controllers (buttons, slider,...)

//boolean freeze; // pauses the simulation
int num=4000;
int human;
int panicHuman;
int zombie;
int activeZombie;
int weakenedZombie;
int activeWeakZombie;
int madZombie;
int activeMadZombie;
int dead;
int wall;
int sand;
int concrete;
int empty;
int hit;
int soldier;
int emptySoldier;
int shootingSoldier;
int clock;
int tick;
int scale=2;

String version = "1.3";

int colorWin;
int colorLoose;
int colorMisc;
int colorText;
int colorTextHighlight;
int colorDisabled;

int colorBackground1;
int colorBackground2;
int colorLine;
int colorHoverField;
int colorHoverText;
int colorIntroTitle;

int colorLoopLength =50;

int defaultTurnLength=2;
int defaultInitialTurnLengthRatio=2;
boolean limitedAmmo=true;
int targetFrameRate = 60;
int defaultSpeed=4;
int defaultActionSpeed=2;

boolean fps=false;

int ishuman=0;
int iszombie=1;
int isdead=2;
int issoldier=3;
int isweakenedZombie=4;
int ismadZombie=5;

int seeSomething=-1;
int seeNothing=0;
int seeZombie=1;
int seeHuman=2;
int seeWall=3;
int seePanicHuman=4;
int seeSoldier=5;
int seeConcrete=6;

int dataHuman=0;
int dataSoldier=1;
int dataZombie=2;
int dataDead=3;
int dataKilledHumans=4;
int dataKilledZombies=5;
int dataInoculatedHumans=6;

int gWidth,gHeight;
int playWidthMini=700;
int playHeightMini=350;
int panelHeight=30;
int infoHeight=140;
int hoverTextSize=12;
int hoverTextInterline=3;
int hoverHeightMargin=5;
int hoverWidthMargin=20;

int infoWidthPosition=76;

int textMargin=5;

int infoTitleWidth = 120;
int infoTextWidth = 400;
int infoTargetWidth = 100;
int InfoPositionWidth = 76; // In the info screen is devided in 8 positions

int menuTitleBorder=50;
int menuTitleSize=50;
int menuButtonWidth=70;
int menuButtonHeight=19;
int panelButtonWidth=40;
int panelTextWidth=150;
int costTextWidth =70;

StartScreen startScreen;
GameScreen gameScreen;
Mission mission;
GameSequence gameSequence;
World world;
Blink blink; // will be used for some text effetc.
 

DecimalFormat decimalFormat;
ControlP5 cp5;
PFont fNormal;
PFont fBold;
PFont fTitle;

// Things to move in the World class
//ArrayList datas;    // data stored for the graph

public void setup() { 
  
  // defines a format to print floats (to print FPS, later)
  decimalFormat = (DecimalFormat)DecimalFormat.getInstance(Locale.FRENCH);
  decimalFormat.applyPattern("0");
 
  frameRate(targetFrameRate);
  
  size(playWidthMini,playHeightMini+panelHeight+infoHeight+hoverTextSize+hoverTextInterline+hoverHeightMargin*2);
  
  fNormal=loadFont("ArialMT-11.vlw");
  fBold=loadFont("Arial-BoldMT-14.vlw");
  fTitle=loadFont("KristenITC-Regular-19.vlw");
  
  human=color(200,0,200);
  panicHuman=color(255,120,255);
  zombie=color(0,150,0);
  activeZombie=color(0,255,0);
  weakenedZombie=color(0,200,150);
  activeWeakZombie=color(0,255,220);
  madZombie=color(200,100,0);
  activeMadZombie=color(255,200,0);
  wall=color(70,70,80);
  concrete=color(120,120,140);
  sand=color(160,120,100);
  empty=color(0,0,0);
  dead=color(128,30,30);
  hit=color(128,128,0);
  shootingSoldier=color(100,255,255);
  soldier=color(0,130,255);
  emptySoldier=color(0,50,200);
  
  colorWin=color(30,255,0);
  colorLoose=color(255,30,0);
  colorMisc=color(255,128,0);
  colorText=color(239,228,176);
  colorDisabled=color(150,150,140);
  colorTextHighlight=color(250,250,240);
  colorBackground1=color(1,21,31);
  colorBackground2=color(1,41,62);
  colorLine=color(1,10,20);
  colorHoverField=color(40,40,50);
  colorHoverText=color(255);
  colorIntroTitle=human;
   
  // This will contain the different controllers
  cp5 = new ControlP5(this);
  
  gameSequence = new GameSequence(); 
  gameSequence.initStartScreen();
} 
 

public void draw() { 
  gameSequence.display();
}



// Function used to change the class of the being.
// To be used befor running the move action.
public Being changeBeing(Being being) {
  int x,y,changed;
  
  if (being.changed == isdead){
      return new Dead(being);
  }
  if (being.changed == iszombie){
      return new Zombie(being, being.generation);
  }
  if (being.changed == isweakenedZombie){
      return new WeakenedZombie(being, being.generation);
  }
  if (being.changed == ismadZombie){
      return new MadZombie(being, being.generation);
  }
  if (being.changed == issoldier){
      return new Soldier(being);
  }
  return being;
}

// Generic being class. Is not meant to be used
class Being { 
  int xpos, ypos, dir;
  int xdest,ydest;
  int attracted=0;
  int attractionStrength=0;
  int generation=0;  // generation of the zombie that infected the being
  int type, active; // type : zombie, ... active : >0 panic
  int originalType; // What it was before being a that type. Mostly usefull for zombies, and dead
  int deathCause=deathAlive; // will be used to store the origin of the death (will be passed on when changing object being to dead one
  int changed;
  int seeList[][] = {};
  int ignoreList[] = {};
  boolean update=false;  //true if needs to refresh the display when paused
  World wld;
  
  Being(){
  }
  
  Being(World _wld){
    wld=_wld;
  }

  // With planned structure (array of beings), can not die
  // should be killed by another object.
  public void die(int _deathCause){
    deathCause=_deathCause;
    changed=isdead;
    setBeing(xpos,ypos,dead);
  }

  // Places a being at a random position on an empty area
  public void position(){
    for (int ok=0; ok<100; ok++){
      xpos = wld.playArea.setX(); 
      ypos = wld.playArea.setY();//(int)random(playArea.gHeight);
      if (get(xpos,ypos)==empty) { ok = 100; }
    }
  }
  
  public void setBeing(int x, int y,int colour) {
    
    for (int i=0;i<scale;i++){
      for (int j=0;j<scale;j++){
        set(x+i,y+j,colour);
      }
    }
    //set(x,y,colour);
  }
  
  public int lookPixel(int seeList[][],int ignoreList[],int x, int y){
    int colour;
    
    colour = get(x,y);
    if (!wld.playArea.inside(x,y)) return seeWall; // would probably have to change that
    for (int j=0; j<seeList.length; j++){
      if (colour == seeList[j][0]) return seeList[j][1];
    }
    
    // Checks if sees anything that is not in ignoreList
    int k;
    for ( k=0; k<ignoreList.length;k++){
      if (colour == ignoreList[k]) break;
    }
    if (k==ignoreList.length) return seeSomething; 
    
    // saw nothing
    return seeNothing;
  }
  
  // look in a given direction, up to a certain distance
  // return result depending on first object seen
  // beings look at what is drawn on the screen
  public int look(int seeList[][],int ignoreList[],int d,int dist){
    int x,y; 
    int colour;
        
    x = xpos;
    y = ypos;
    for(int i=0; i<dist; i++){
      x += directionX(d);
      y += directionY(d);
      colour = lookPixel(seeList, ignoreList,x,y);
      if (colour!=seeNothing) return colour;      
    }
    return seeNothing; // saw nothing
  }
  
  public int directionX(int dir){
    switch(dir){
      case 1:
      case 3:
        return 0;
      case 2:
        return 1*scale;
      case 4:
        return -1*scale;
    }
    return 0;
  }
  
  public int directionY(int dir){
    switch(dir){
      case 1:
        return -1*scale;
      case 2:
      case 4:
        return 0;
      case 3:
        return +1*scale;
    }
    return 0;
  }
  
  public void attract(int x, int y, int tempAttracted, int tempStrength){
    //if (tempStrength>attractionStrength||attracted==0){
      xdest=x;
      ydest=y;
      attracted=tempAttracted;
      attractionStrength=tempStrength;
    //}
  }
  
  // If the being is being attracted, will turn toward 
  // or away from  the attractor
  public void turnToTarget(){
    int dx = xpos-xdest;
    int dy = ypos-ydest;
    if (dx!=0 && dy!=0){
      if (abs(dx)>abs(dy) || (abs(dx)==abs(dy) && (int)random(1)==0)){
        if ((dx>0 && attracted > 0)||(dx<0 && attracted<0)) dir = 4;
        else dir =2;
      }
      else {
        if ((dy>0 && attracted > 0)||(dy<0 && attracted<0)) dir = 1;
        else dir = 3;
      }
    }
    if (attracted > 0) attracted --;
    else if (attracted <0) attracted ++;
  }

    // Turns the being into a zombie
  public void infect(int _generation, int _type){
    changed = _type;
    generation = _generation;
  }
  
  public void drawBeingColor(int colour){
    setBeing(xpos,ypos,colour);
  }
  // when drawing the being, out of the move() method. More options available
  public void drawBeingOutside(){
    if (!gameSequence.freeze || world.displayBeingData.modeNormal()){
      drawBeing();
    }
    else world.displayBeingData.display(this);
  }
  public void drawBeing(){}
  public void move(){}
  public void equip(){}
}

class Human extends Being {
  int belief, maxBelief;
  int rest;
  int seeList[][] = {  {panicHuman,seePanicHuman},
                       {zombie,seeZombie},
                       {activeZombie,seeZombie},
                       {weakenedZombie,seeZombie},
                       {activeWeakZombie,seeZombie}};
  int ignoreList[] = {empty,dead,hit};
  
  Human(){
  }
  
  Human(World _wld){
    super(_wld);
    dir = (int)random(4)+1;
    type = ishuman;
    originalType=ishuman;
    active = 0;
    belief=500;
    maxBelief=500;
    rest=40;
    changed=-1;
  }
  
  Human(Being being){
    xpos = being.xpos;
    ypos = being.ypos;
    dir = being.dir;
    type = ishuman;
    originalType=being.type;
    active = 0;
    belief = 500;
    maxBelief = 500;
    rest = 40;
    changed = -1;
    wld=being.wld;
  }
  
  public void infect(int _generation, int _type){
    changed = _type;
    generation = _generation;
    
    if (_type==isweakenedZombie) wld.numInoculatedHumans++;
  }
  
  public void equip(){
    changed = issoldier;
  }
  
  public void drawBeing(){
    if (active > 0) setBeing(xpos, ypos, panicHuman);
    else setBeing(xpos, ypos, human);
    if (update) update=false;
  }
  
  public void move(){
        
    int r = (int)random(3); //being move once every 4 ticks, except panic who run every
    if (active>0 || r==1){ // if panicHuman or just moves by one
      setBeing(xpos, ypos, empty);
      if(belief<=0){
        active=0;
        maxBelief-=100;
        belief=0;
        rest=0;
      }
      
      // if sees nothing in direction, moves   
      if (look(seeList,ignoreList,dir,1)==seeNothing) {
        xpos += directionX(dir);
        ypos += directionY(dir);
      }
      
      // otherwise turns
      else {
        dir = (int)random(4)+1; 
      }

      drawBeing();
      if (active>0) {active--;}
    }

    // everybody looks in direction
    int target = look(seeList,ignoreList,dir,10);

    // Saw a zombie !! Believes in them, and turns around
    if (target==1){
      maxBelief=500;
      belief=500;
      active=10;
      rest=0;
      dir = dir + 2; 
      if (dir>4) dir = dir - 4; 
    }
    
    // Saw someone panicking !!
    else if(target==seePanicHuman){ 
      active=10;
      if(belief>0){
        belief--;
      }
      else if(belief==0 && maxBelief!=0 && rest>=40){
        belief=maxBelief;
        rest=0;
      }
      else if(belief==0 && rest<40){
        rest++;
      }
    }
    
    // Saw neither a zombie, nor some panicking and not active
    else if(active==0) {
      rest++;
    }
    
    // pretty likely to change direction
    if ((int)random(2)==1) {
      
      // that line makes them turn toward the attractor
      if (attracted !=0) turnToTarget(); 
      else dir = (int)random(4)+1;
    }
  }
  
}

// Class of the corpses lying around. 
// They stay there doing nothing...
class Dead extends Being {
  
  int seeList[][]={};
  int ignoreList[] = {empty,hit};
  
  Dead(Being being){
    xpos = being.xpos;
    ypos = being.ypos;
    type = isdead;
    originalType = being.type;
    deathCause=being.deathCause;
    changed = -1;
    wld=being.wld;
  }
  
  public void drawBeing(){
    
    if (lookPixel(seeList,ignoreList,xpos,ypos)==seeNothing) 
        setBeing(xpos,ypos,dead);
    if (update) update=false;
  }
  
  public void move(){
    drawBeing();
  } 
  
  // Dead can't be infected nor die.
  public void die(int deathCause){}
}

class Zombie extends Being {
  int zombielife;
  int seeList[][] = {  {wall,seeWall},
                       {sand,seeWall},
                       {concrete,seeConcrete},
                       {panicHuman, seeHuman},
                       {human, seeHuman},
                       {shootingSoldier, seeHuman},
                       {soldier, seeHuman},
                       {emptySoldier,seeHuman}};
  int ignoreList[] = {empty,dead,hit};
  
  Zombie(){
  }
  
  Zombie(World _wld){
    super(_wld);
  }
  
  Zombie(Being being,int _generation){
    xpos=being.xpos;
    ypos=being.ypos;
    dir=being.dir;
    type=iszombie;
    originalType=being.type;
    active = 0;
    zombielife=(int)random(200)+800;
    generation= _generation+1;
    changed=-1;
    wld=being.wld;
  }
  
  public void drawBeing(){
    if (active > 0) setBeing(xpos, ypos, activeZombie);
    else setBeing(xpos, ypos, zombie);
  }
  
  public void attack(Being being){
    being.infect(generation,iszombie);
  }
  
   public void move(){
     int target; // store data from look()
   //Zombies move once every 4 ticks
    
     int r = (int)random(3); 
     if (r==1) {
        setBeing(xpos, ypos, empty);
        
        // if sees nothing in direction, moves
        target=look(seeList,ignoreList,dir,1);   
        if (target==seeNothing){
          xpos += directionX(dir);
          ypos += directionY(dir);
        }
        
        // otherwise turns
        else {
          if (target!=seeHuman) dir = (int)random(4)+1;
        }
        
        // draws the zombie at its new position
        drawBeing();
      }
  
      // now, checks what can be seen
      target = look(seeList,ignoreList,dir,10);
      
      // if sees human, activates
      if (target==seeHuman) active = 10; 
      
      // if sees a wall, might destroy it
      else if (target==seeWall)  { 
        if((int)random(4)==1){
          setBeing(xpos+directionX(dir),ypos+directionY(dir),empty); 
        }
      }
      else if (target==seeConcrete) {
        if((int)random(16)==1){
          setBeing(xpos+directionX(dir),ypos+directionY(dir),empty); 
        }
      }
      
      // might turn if is not active
      if (attracted !=0) turnToTarget(); 
      else if (active==0){ 
        if((int)random(10)==1){
          if((int)random(2)==0) dir = dir+1;
          else dir = dir-1;
          if(dir==5) dir=1;
          if(dir==0) dir=4;
        }
      }
      if (active>0) {active--;}

      
      // look for victims
      int victim = look(seeList,ignoreList,dir,1);
      if (victim == seeHuman){
        int ix = xpos+directionX(dir);
        int iy = ypos+directionY(dir);
        for(int i=0; i<num; i++) {
          if (ix==wld.beings[i].xpos && iy==wld.beings[i].ypos){
            attack(wld.beings[i]);
            break;
          }
        } 
      }
      
      
      // Zombies too can die...
      zombielife--;
      if(zombielife<=0) die(deathAge);
  }
}

class Soldier extends Human{
  int ammoInit = 8;
  int ammo;
  
  int seeList[][] = {  {emptySoldier,seeSoldier},
                       {zombie,seeZombie},
                       {activeZombie,seeZombie},
                       {weakenedZombie,seeZombie},
                       {activeWeakZombie,seeZombie}};
  
  Soldier(int x, int y,World _wld){
    super(_wld);
    xpos=x;
    ypos=y;
    active=0;
    ammo = ammoInit;
    type = issoldier;
    originalType = issoldier;
    changed=-1;
  }
  
  Soldier(Being being){
    xpos=being.xpos;
    ypos=being.ypos;
    ammo = ammoInit;
    type = issoldier;
    originalType=being.type;
    active=0;
    changed=-1;
    wld=being.wld;
  }
  
  public void equip(){
    ammo = ammoInit;
    update=true;
  }
  
  public void drawBeing(){
    if (ammo<=ammoInit/2) setBeing(xpos, ypos, emptySoldier);
    else if (active > 0) setBeing(xpos, ypos, shootingSoldier);
    else setBeing(xpos, ypos, soldier);
    if (update) update=false;
  }
  
  public void move(){
    int dirBack;
    
    int r = (int)random(3); //being move once every 4 ticks, except panic who run every
    if (active>0 || r==1){ // if panicHuman or just moves by one
      setBeing(xpos, ypos, empty);
      
      // If just shot moves back
      if (active>0){
          dirBack=dir+2;
          if (dirBack>4) dirBack -=4;
          if (look(seeList,ignoreList,dirBack,1)==seeNothing){
            xpos += directionX(dirBack);
            ypos += directionY(dirBack);
          }
      }
      else if (look(seeList,ignoreList,dir,1)==seeNothing) {
        xpos += directionX(dir);
        ypos += directionY(dir);
      }
      
      // otherwise turns
      else dir = (int)random(4)+1; 
      drawBeing();
      if (active>0) {active--;}
    }
    
    int target = look(seeList,ignoreList,dir,10);

    // Saw a zombie !! Shoots him
    // Not efficient at all (CPU)!! 
    if (target==seeZombie){      
      for (int i=0;i<num;i++){
        int x=xpos;
        int y=ypos;
        for (int j=0;j<10;j++){
          x += directionX(dir);
          y += directionY(dir);
          if (wld.beings[i] instanceof Zombie && wld.beings[i].xpos==x 
              && wld.beings[i].ypos==y){
            if (ammo>0) {
              wld.beings[i].die(weaponTroopers);
              ammo --;
            }
          }
        }
      }
      active=5;
    }
    
    // Sees if there is there is soldier around and shares ammo 
    for (int i=0;i<4;i++){
      if (look(seeList,ignoreList,i,1)==seeSoldier){
        int ix = xpos+directionX(i);
        int iy = ypos+directionY(i);
        for(int j=0; j<num; j++) {
          if (wld.beings[j] instanceof Soldier &&
              ix==wld.beings[j].xpos && iy==wld.beings[j].ypos){
            Soldier soldier = (Soldier) wld.beings[j];
            int ammoDiff = ammo-soldier.ammo;
            if (ammoDiff>1){
              soldier.ammo+=ammoDiff/2;
              ammo-=ammoDiff/2;
            }
            break;
          }
        }        
      }
    }  
    
    // pretty likely to change direction (same as human)
    if ((int)random(2)==1) {
      if (attracted !=0) turnToTarget(); 
      else dir = (int)random(4)+1;
    }
  }
}

// Zombie that gets weaker every new generation. Will also attack near active zombies.
class WeakenedZombie extends Zombie{
  int seeList[][] = {  {wall,seeWall},
                       {sand,seeWall},
                       {concrete,seeConcrete},
                       {panicHuman, seeHuman},
                       {human, seeHuman},
                       {zombie, seeHuman},
                       {activeZombie, seeHuman},
                       {shootingSoldier, seeHuman},
                       {soldier, seeHuman},
                       {emptySoldier, seeHuman}};
  
  WeakenedZombie(Being being,int _generation){
    xpos=being.xpos;
    ypos=being.ypos;
    dir=being.dir;
    type=isweakenedZombie;
    originalType=being.type;
    active = 0;
    generation= _generation+1;
    zombielife=(int)random(50)+(150-generation*30);
    changed=-1;
    wld=being.wld;
  }
  
  public void attack(Being being){
    being.infect(generation,isweakenedZombie);
  }
  
  // Same color when attack, but slightly different when normal
  public void drawBeing(){
    if (active > 0) setBeing(xpos, ypos, activeWeakZombie);
    else setBeing(xpos, ypos, weakenedZombie);
    if (update) update=false;
  }
  
  public int look(int _seeList[][],int _ignoreList[],int d,int dist){
    return super.look(seeList,ignoreList,d,dist);
  }
}

class MadZombie extends Zombie{
 
  int seeList[][] = {  {wall,seeWall},
                       {sand,seeWall},
                       {concrete,seeConcrete},
                       {zombie, seeHuman},
                       {activeZombie, seeHuman},
                       {weakenedZombie, seeHuman},
                       {activeWeakZombie,seeHuman}};
  
  MadZombie(Being being,int _generation){
    xpos=being.xpos;
    ypos=being.ypos;
    dir=being.dir;
    type=ismadZombie;
    originalType=being.type;
    active = 0;
    generation= _generation+1;
    zombielife=(int)random(50)+(120);
    changed=-1;
    wld=being.wld;
  }
  
  public void attack(Being being){
    being.infect(generation,ismadZombie);
  }
  
  // Same color when attack, but slightly different when normal
  public void drawBeing(){
    if (active > 0) {
      setBeing(xpos, ypos, activeMadZombie);
    }
    else setBeing(xpos, ypos, madZombie);
    if (update) update=false;
  }
  
  // Rather awkward way of doing, but otherwise, it is the
  // seeList defined in class Zombie that is being used.
  public int look(int _seeList[][],int _ignoreList[],int d,int dist){
    return super.look(seeList,ignoreList,d,dist);
  }
}



class DisplayBeingData{
  
  // List used as constants
  int displayNormal=0;
  int displayDeathType=1;
  int displayDeathCause=2;
  int displayZombieOrigin=3;
  int displayZombieGeneration=4;
  int displayZombieAge=5;
  
  // the colors. No problem if they overlap each other as 
  // they have no logic significance (contrary to those defined 
  // in ZombieInfection)
  int colorDefault = color(25);
  int colorNuke = color(252,235,60);
  int colorMine = color(167,84,5);
  int colorBomb = color(255,64,0);
  int colorAge = color(30,80,40);
  int colorWalled = color(225,225,230);
  
  
  // Number of modes we allow to cycle through
  int modesNumber=4;
  
  // Important to know when we have just changed the mode
  int oldMode;
  int mode;
  int oldSecond;
  int modeSecond;
  
  // the various lists linking a parameter to a color
  int typeList[][]={{ishuman,human},
                    {issoldier,soldier},
                    {iszombie,zombie},
                    {isweakenedZombie,weakenedZombie},
                    {ismadZombie,madZombie}};
  
  int deathCauseList[][]={{weaponNuke,colorNuke},
                          {weaponTroopers,soldier},
                          {weaponQuickBomb,colorBomb},
                          {weaponBomb,colorBomb},
                          {weaponSlowBomb,colorBomb},
                          {weaponMine,colorMine},
                          {weaponBigMine,colorMine},
                          {weaponDirt,colorWalled},
                          {weaponConcrete,colorWalled},
                          {weaponInoculate,weakenedZombie},
                          {weaponBioweapon,madZombie},
                          {deathAge,colorAge}};
  
  int deathCauseSecond[]={colorNuke,
                          colorBomb,
                          colorMine,
                          colorAge,
                          soldier,
                          colorWalled};
                          
  int deathTypeSecond[]={human,
                         soldier,
                         zombie,
                         weakenedZombie,
                         madZombie};  
  
  int zombieTypeSecond[]={zombie,
                          weakenedZombie,
                          madZombie};
  
  DisplayBeingData(){
    mode=displayNormal;
    oldMode=mode;
    modeSecond=0;
    oldSecond=modeSecond;
    
  }
  
  public void cycleMode(){
    mode++;
    if (mode>=modesNumber) mode=0;
    modeSecond=0;
  }
  
  public void cycleSecond(){
    int secondListTemp[];
    secondListTemp=getSecondList(mode);
    modeSecond++;
    
    if (secondListTemp!=null && modeSecond>secondListTemp.length)
      modeSecond=0;
    //println(modeSecond);
  }
  
  public int[] getSecondList(int mode){
    if (mode==displayDeathType) return deathCauseSecond;
    else if (mode==displayDeathCause) return deathTypeSecond;
    else if (mode==displayZombieOrigin) return zombieTypeSecond;
    return null;
  }
  
  // true if mode has just changed.
  public boolean changed(){
    if ((mode!=oldMode)||(modeSecond!=oldSecond)) return true;
    else return false;
  }
  
  public void doneChanges(){
    oldMode=mode;
    oldSecond=modeSecond;
  }
  
  // that should be temporary and removed soon.
  public boolean modeNormal(){
    if (mode==displayNormal) return true;
    else return false;
  }
  
  // Selects the correct color in the list, if it can find it.
  public int displayColor(int parameter, int tempList [][]){
    int colour=dead;
    for (int j=0; j<tempList.length; j++){
      if (parameter == tempList[j][0]) {
        colour=tempList[j][1];
        break;
      }
    }
    return colour;
  }
  
  // Returns true if has been killed according to selected method
  public boolean secondCondition(int criteria,int secondList[],int listTemp[][]){
    int deathCause = displayColor(criteria,listTemp);
    
    if (modeSecond==0) return true;
    else if (deathCause == secondList[modeSecond-1]) return true;
    return false;
  }
  
  public void display(Being being){
    int colour;
    
    // we want to display what it was before dying
    if (mode==displayDeathType){
      if (being.type==isdead && secondCondition(being.deathCause,deathCauseSecond,deathCauseList))
          colour = displayColor(being.originalType,typeList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
    
    // We want to display the death cause
    else if (mode==displayDeathCause){
      if (being.type==isdead && secondCondition(being.originalType,deathTypeSecond,typeList)) 
          colour = displayColor(being.deathCause,deathCauseList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
    
    // We want to display what the zombie was
    else if (mode==displayZombieOrigin){
      if (being instanceof Zombie && secondCondition(being.type,zombieTypeSecond,typeList))
          colour = displayColor(being.originalType,typeList);
      else colour=colorDefault;
      being.drawBeingColor(colour);
    }
  }
}
class Effect{
  World wld;
  boolean destroyed;
  
  Effect(){
    destroyed=false;
  }
  
  Effect(World _wld){
    wld=_wld;
  }
  
  public void act(){
  }
  
  public void drawEffect(){
  }
  
}

class EffectBombing extends Effect{
  int quantity;
  int error;
  Weapon weapon;
  
  EffectBombing(World _wld, int _quantity, int _error){
    wld=_wld;
    quantity=_quantity;
    error=_error;
    weapon = new Weapon(wld);
  }
  
  // Plays the effect. When returns true, the effect is removed from the list.
  public void act(){
    int x,y;
    int i=0;
    int j=0;
    while (i<quantity && j<wld.initPop){
      if (wld.beings[j] instanceof Zombie) {
        x=wld.beings[j].xpos+(int) random(error)-error/2;
        y=wld.beings[j].ypos+(int) random(error)-error/2;
        weapon.fire(x,y);
        i++;
        j++;
      }
      else j++;
    }
    if (j== wld.initPop) destroyed=true;
  }
}
int screenStart=0;
int screenGame=1;

int menuNone=-1;
int menuGeneral=0;
int menuAction=1;
int menuStrategy=2;
int menuOption=3;

int Button_Data_Normal =0;
int Button_Data_Unavailable =1;
int Button_Data_Selected =2;

int menuDisplayWeaponNumber=10;
int listRefMultiplier=100;

// holds informations regarding the display and speed of the simulation
class GameSequence{
  boolean freeze;
  boolean oldFreeze;
  boolean turnBased;
  boolean finished,win;
  int turnLengthSetting;
  int turnLength;
  int initialTurnLengthSetting;
  float initialTurnLengthRatio;
  int speed;
  int speedRate;
  int screenRef;
  int menu; // not used now
  int previousStartMenu; // usefull to know if can use previous parameters or need to load default.
  
  int listSpeed[]={120,40,20,15,10};
  String listSpeedName[]={s_Speed_VerySlow,s_Speed_Slow,s_Speed_Normal,s_Speed_Quick,s_Speed_VeryQuick};
  int listTurnLength[]={50,100,200,300,500};
  float listInitialTurnLengthRatio[]={0.1f,0.3f,0.5f,1,2,3};
  
  
  GameSequence(){
    screenRef = screenStart;
    menu = menuGeneral;
    previousStartMenu = menuGeneral;
    freeze=true;
    oldFreeze=freeze;
    turnBased=true;
    finished=false;
    setTurnLength(defaultTurnLength);
    setInitialTurnLength(defaultInitialTurnLengthRatio);
    blink = new Blink();

    // Moved here from game initialisation, because will be used in first screen
    world = new World();
    mission = new Mission(world); 
  }
  
  public void setSpeedRate(){
    if (speed>=0 && speed<listSpeed.length) speedRate=listSpeed[speed];
    else speedRate = listSpeed[defaultSpeed];
  }
  
  public void setSpeedName(String buttonName){
    String name;
    if (speed>=0 && speed<listSpeed.length && cp5.controller(buttonName)!=null){
      cp5.controller(buttonName).setValueLabel(listSpeedName[speed]);
    }
  }
  
  public void setInitialTurnLengthName(String buttonName){
    String name="*"+str(initialTurnLengthRatio)+" ("+str(PApplet.parseInt(initialTurnLengthRatio*turnLength))+")";  
    if (cp5.controller(buttonName)!=null) cp5.controller(buttonName).setValueLabel(name);
  }
  
  public void setInitialTurnLength(int value){
    initialTurnLengthSetting=value;
    if (initialTurnLengthSetting>=0 && initialTurnLengthSetting<listInitialTurnLengthRatio.length){
      initialTurnLengthRatio=listInitialTurnLengthRatio[initialTurnLengthSetting];
    }
    else {
      initialTurnLengthSetting=defaultInitialTurnLengthRatio;
      initialTurnLengthRatio=listInitialTurnLengthRatio[initialTurnLengthSetting];
    }
  }
  
  public void setTurnLength(int value){
    turnLengthSetting =value;
    if (turnLengthSetting>=0 && turnLengthSetting<listTurnLength.length){
      turnLength = listTurnLength[turnLengthSetting];
    }
    else {
      turnLengthSetting =defaultTurnLength;
      turnLength = listTurnLength[turnLengthSetting];
    }
  }
  
  public void setTurnLengthName(String buttonName){
    cp5.controller(buttonName).setValueLabel(str(turnLength));
  }
  
  public void initStartScreen(){
    startScreen = new StartScreen();
  }
 
  public void initGame(){
    finished = false;
    // does not start at 0, otherwise infection would have spread too much
    setSpeedRate();
    tick=0;
    
    //println(turnLength*initialTurnLengthRatio);
    clock=turnLength-PApplet.parseInt (turnLength*initialTurnLengthRatio);
    if (gameSequence.turnBased) gameSequence.freeze = false;
    else gameSequence.freeze=true;
  
  
  // Initiates the gameScreen
   
   world.clean();// data must be there because will be linked to the screens
   if (gameScreen!=null){
     gameScreen.erase();
     gameScreen=null;
   } 
   gameScreen = new GameScreen();
   
   world.init(gameScreen.playArea,num);
   mission.clean(); // After world init, because need to get info about the world

  }
  
  public void won(String s){
    if (!finished){
      mission.score();
      finished=true;
      freeze=true;
      win=true;
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionWin,s,colorWin);
    }
  }
  
  public void lost(String s){
    if (!finished){
      mission.score();
      finished=true;
      win=false;
      freeze=true;
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionLoose,s,colorLoose);
    }
  }
  
  public void warning(String s,int type){
    if (!finished){
      gameScreen.infoPanel.transitionInfoPanel(infoPanelRef_TransitionWarning,s,mission.getEventColor(type));
    }
  }
  
  public void display(){
    blink.forward();
    if (gameSequence.screenRef == screenStart){
      startScreen.display();
    }
    else if (gameSequence.screenRef == screenGame){
      gameSequence.runGame();
      gameScreen.display();
    }
  }
  
  public int checkTurnLength(int _clock){
    int clock=_clock;
    clock++;
    if (turnBased && !finished){
      if (clock>=turnLength) {
        freeze=true;
        gameScreen.textPanel.menu.buttonStatus("Menu_PanelStrategy_NextTurn",Button_Data_Normal);
        clock=0;
      }
    }
    return clock;
  }
  
  public void runGame(){
    int changed;
    // Display specific data or cleans everything special being data have been displayed
    // Do action if game has just been restarted and was not on normal display, or if game as just been paused, 
    // or game is pause and just changed type of display
    if ((!freeze && freeze!=oldFreeze && !world.displayBeingNormal()) ||
       (freeze && (freeze!=oldFreeze || world.displayBeingChanged()))){
      
      world.updateBeingsDisplay();
      oldFreeze=freeze;
    }
    
    // Move one more step forward
    if (tick*10%speedRate==0){  
      if (!freeze){
        world.oneStepForward();  
        clock = checkTurnLength(clock);
        mission.checkEvent();
      }
      else world.oneStepFrozen();
    }
    tick++;
    if (tick>50000) tick=0; //probably not usefull, as it should loop. But who knows...
  }
}

class World{
  WeaponStock weaponStock;
  Being[] beings; // The structure that stores survivors, zombies and dead people
  ArrayList devices;  // devices left on the map, such as bombs,...
  ArrayList effects; //temporary effects (visual or events) on the game
  DisplayBeingData displayBeingData; // class used to display some data about beings
  ArrayList datas;    // data stored for the graph
  
  PlayArea playArea;  // Used so that it knows where to draw and check the objects
  int time;
  int initPop;
    
  int numKilledHumans;
  int numKilledZombies;
  int numInoculatedHumans;
  int numZombies;
  int numDead; 
  int numSoldier;
  int numHuman;

  World(){
    weaponStock= new WeaponStock(this);
    displayBeingData= new DisplayBeingData();
    datas= new ArrayList();
  }
  
  public void clean(){
    datas= new ArrayList();
  }
  
  public void init(PlayArea _playArea,int _initPop){
    time=0;
    initPop=_initPop;
    playArea=_playArea;
    
    devices = new ArrayList();
    effects = new ArrayList();
    weaponStock.reinit();
    
    // create the beings
    beings = new Being[8000];        
    for(int i=0; i<min(8000,initPop); i++){ 
      beings[i] = new Human(this); 
      beings[i].position();
      beings[i].drawBeing(); 
    }
    // and infects the 4 first one that have been created.
    /*beings[0].infect(0,iszombie);
    beings[1].infect(0,iszombie);
    beings[2].infect(0,iszombie);
    beings[3].infect(0,iszombie);*/
    
    // Prepare the data
    // Very important !! They are created in the same order as "ishuman, iszombie,..."
    numKilledHumans=0;
    numKilledZombies=0;
    numInoculatedHumans=0;
    numZombies=0;
    numDead=0; 
    numSoldier=0;
    numHuman=initPop;
        
    Data data;
    data = new Data(human,"Humans");
    data.add(numHuman);
    datas.add(data);
    data = new Data(soldier,"Soldiers");
    data.add(numSoldier);
    datas.add(data);
    data = new Data(zombie,"Zombies");
    data.add(numZombies);
    datas.add(data);
    data = new Data(dead,"Dead");
    data.add(numDead);
    datas.add(data);
    data = new Data(panicHuman,"Nuked H.");
    data.add(numKilledHumans);
    datas.add(data);
    data = new Data(activeZombie,"Nuked Z.");
    data.add(numKilledZombies);
    datas.add(data);
    data = new Data(weakenedZombie,"Inoc. H.");
    data.add(numInoculatedHumans);
    datas.add(data);
  }
  
  // Simulates the world one step forward
  public void oneStepForward(){
    beingOneStep(false);
    deviceOneStep(false);
    effectOneStep(false);    
    weaponOneStep();
    dataOneStep();
    time++;
  }
  
  // Update the drawing of the workd, when frozen
  public void oneStepFrozen(){
    beingOneStep(true);
    deviceOneStep(true); 
  }
  
  public void updateBeingsDisplay(){
    for(int i=0; i<num; i++){
        beings[i].drawBeingColor(empty);
        beings[i].drawBeingOutside();
    }
    displayBeingData.doneChanges();
  }
  
    
  public boolean displayBeingNormal(){
    return displayBeingData.modeNormal();
  }
  
  public boolean displayBeingChanged(){
    return displayBeingData.changed();
  }
  
  // Moves and draws the beings one at a time
  // Should also be runned when the game is paused to draw changes when bombing and game is paused
  private void beingOneStep(boolean freeze){
    for(int i=0; i<num; i++){ 
      if (beings[i].changed!=-1) {
        beings[i]=changeBeing(beings[i]);
        beings[i].drawBeingOutside();
      }
      if (beings[i].update) beings[i].drawBeingOutside();
      if (!freeze) beings[i].move();
    }
  }
  
  // Draws and make device act one step forward
  // Should used when paused, as they devices can blink
  private void deviceOneStep(boolean freeze){
    for (int i=0; i<devices.size();i++){
      Device device = (Device) devices.get(i);
      device.drawDevice();
      if (!freeze) device.act();
      if (device.destroyed) devices.remove(i);
    }
  }
  
  // Draws and make effect act one step forward
  // Should used when paused, as they devices can blink
  private void effectOneStep(boolean freeze){
    for (int i=0; i<effects.size();i++){
      Effect effect = (Effect) effects.get(i);
      effect.drawEffect();
      if (!freeze) effect.act();
      if (effect.destroyed) {
        println("Removed Bombing");
        effects.remove(i);
      }
    }
  }
  
  // Should only be called when game is not paused
  private void weaponOneStep(){
    //Restock Ammos
    weaponStock.reStock();
  }
  
  private void dataOneStep(){
    // counts the number of zombies and population
      // might be improved to avoid counting everything frame, by only make the difference
      numZombies=0;
      numDead=0; 
      numSoldier=0;
      numHuman=0;
      for(int j=0;j<num;j++){
        if(beings[j] instanceof Zombie) numZombies++;
        if(beings[j] instanceof Dead) numDead++;
        if(beings[j] instanceof Soldier) numSoldier++;
        if(beings[j] instanceof Human) numHuman++;
      }   
      // Stores the data and draw the graph
      Data data;
      data = (Data) datas.get(dataHuman);
      data.add(numHuman);
      data = (Data) datas.get(dataZombie);
      data.add(numZombies);
      data = (Data) datas.get(dataDead);
      data.add(numDead);
      data = (Data) datas.get(dataKilledHumans);
      data.add(numKilledHumans);
      data = (Data) datas.get(dataKilledZombies);
      data.add(numKilledZombies);
      data = (Data) datas.get(dataSoldier);
      data.add(numSoldier);
      data = (Data) datas.get(dataInoculatedHumans);
      data.add(numInoculatedHumans);
  }
}

class Blink{
  float colorLoop;
  
  Blink(){
    colorLoop=0;
  }
  
  public void forward(){
    colorLoop+=0.01f;
    if (colorLoop>1000) colorLoop=0;
  }
  
  // returns true when a full cycle has been done
  public boolean checkCycle(float speed){
    if (colorLoop*speed > 2*PI) return true;
    return false;
  }
  
  // blinks following the sinus fonction
  public int gradual(int color1,int color2,float speed){
    float blink;
    int cRed,cGreen,cBlue;
    
    blink = (1+sin(colorLoop*speed-PI/2))/2;
    cRed = (int) (blink*red(color1)+(1-blink)*red(color2));
    cGreen = (int) (blink*green(color1)+(1-blink)*green(color2));
    cBlue = (int) (blink*blue(color1)+(1-blink)*blue(color2));
    
    return color(cRed,cGreen,cBlue);
  }
  
  // returns either color1 or color2
  public int flash(int color1,int color2, float speed){
    if (cos(colorLoop*speed)>0) return color1;
    else return color2;
  }
  
  // returns the color that is glowing
  public int glow(int color1, float speed){
    float blink;
    int cRed,cGreen,cBlue;
    
    blink = (1+sin(colorLoop*speed-PI/2))/2;
    cRed = (int) (blink*red(color1));
    cGreen = (int) (blink*green(color1));
    cBlue = (int) (blink*blue(color1));
    
    return color(cRed,cGreen,cBlue);
  }
}

int infoPanelRef_Statistics = 0;
int infoPanelRef_Mission = 1;
int infoPanelRef_Score = 2;
int infoPanelRef_TransitionWin = 3;
int infoPanelRef_TransitionLoose = 4;
int infoPanelRef_TransitionWarning = 5;

class GameScreen{
  TextPanel textPanel;
  PlayArea playArea;
  InfoPanel infoPanel;
  HoverInfo hoverInfo;
  
  GameScreen(){
    
    int fieldHeight=(hoverTextSize+hoverTextInterline)+hoverHeightMargin*2;
    int w = width-1; // -1 because want to see the line
    int h1 = 0;
    int h2 = height-panelHeight-infoHeight-fieldHeight-1;// -1 because y starts at 0
    // Builds the map
    playArea = new PlayArea(0,h1,w,h2);
    playArea.drawMap();
    
    // Builds the bottom panel, where data will be displayed
    h1+=h2;
    h2=panelHeight;
    if (gameSequence.turnBased == true) textPanel = new TextPanelStrategy(0,h1,w,h2);
    else textPanel = new TextPanelAction(0,h1,w,h2); 
    
    h1+=h2;
    h2=infoHeight;
    infoPanel = new InfoPanel(0,h1,w,h2-1,world); // -1 because want to see the line (bottom drawn outside)
    
    h1+=h2;
    h2=fieldHeight;
    hoverInfo = new HoverInfo(0,h1,w,h2,hoverWidthMargin,hoverHeightMargin);
  }
  
  public void display(){
    if (gameSequence.finished && !(textPanel instanceof TextPanelFinished)){
      int _x=textPanel.tX;
      int _y=textPanel.tY;
      int _w=textPanel.tWidth;
      int _h=textPanel.tHeight;
      textPanel.erase();
      textPanel = new TextPanelFinished(_x,_y,_w,_h);
    }
    if (textPanel!=null) textPanel.display();
    if (infoPanel!=null) infoPanel.display();
    if (hoverInfo!=null) {
      boolean result = hoverInfo.display(textPanel.menu);
      if (!result) hoverInfo.display(infoPanel.menu);
    }
  }
  
  public void erase(){
    if (textPanel!=null) textPanel.erase();
    if (infoPanel!=null) infoPanel.erase();
  }
}

// This is the panel that will display the text
class TextPanel{
  int tX,tY,tWidth,tHeight;
  PFont f;
  Menu menu;
  
  TextPanel(int _tX,int _tY, int _tWidth,int _tHeight){
    tX = _tX;
    tY = _tY;
    tWidth= _tWidth;
    tHeight= _tHeight;
    f=fBold;
    menu = loadMenu();
  }
  
  public Menu loadMenu(){
    return null;
  }
  
  // cleans the panel
  public void preparePanel(){
    //fill(colorLine);
    stroke(colorLine);
    fill(colorBackground1);
    //rect(tX,tY,tWidth,tHeight);
    rect(tX,tY,tWidth,tHeight); 
  }
  
  // displays the figures
  public void display(){
  }
  
  public void erase(){
    if (menu!=null) menu.eraseMenu();
  } 
}

class TextPanelFinished extends TextPanel{
  
  TextPanelFinished(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
  }
  
  public Menu loadMenu(){
    return new MenuPanelFinished(tX,tY,tWidth,tHeight);
  }
  
  public void display(){
    String s;
    int color1,color2;
    
    preparePanel();
    textFont(f,14);
    noStroke();
    
    /*int textY=tY+(panelHeight+14-5)/2;
    if (gameSequence.win){
      color1=colorWin;
      //color2=colorBackground1;
      s = s_Text_PanelFinished_Win;
    }
    else {
      color1=colorLoose;
      //color2=colorBackground1;
      s = s_Text_PanelFinished_Loose;
    }
    
    fill(color1);
    textAlign(CENTER);
    text(s,tX+tWidth/2,textY);*/
  }
}

// This is the panel that will display the text
class TextPanelStrategy extends TextPanel{
  
  TextPanelStrategy(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
  }
  
  public Menu loadMenu(){
    return new MenuPanelStrategy(tX,tY,tWidth,tHeight);
  }
  
  // displays the figures
  public void display(){
    String s;
    
    preparePanel();
    
    textFont(f,14);
    noStroke();
    
    int textY=tY+(panelHeight+14-5)/2;

    fill(colorText);

    int textX=panelButtonWidth+5+1;
    if (world.weaponStock.ammo<world.weaponStock.cost()) fill(colorDisabled);
    else fill(colorTextHighlight);
    s=world.weaponStock.name();
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,panelTextWidth,20);
   
    textX+=panelTextWidth;
    fill(colorText);
    if (limitedAmmo) s= s_Text_Panel_Cost+ world.weaponStock.cost();
    else s= s_Text_Panel_CostUnlimited;
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,costTextWidth,20);
   
    textX+=costTextWidth+panelButtonWidth+5+1;
    fill(colorText);
    if (limitedAmmo) s= s_Text_Panel_Ammo + world.weaponStock.ammo;
    else s= s_Text_Panel_AmmoUnlimited;
    textAlign(CENTER,BOTTOM);
    text(s,textX,textY-17,tWidth-textX-(menuButtonWidth+5),20);
  }
}

class TextPanelAction extends TextPanel{
  String weaponNameLeft,weaponNameRight;
  int costLeft,costRight;
  
  TextPanelAction(int _tX,int _tY, int _tWidth,int _tHeight){
    super(_tX, _tY, _tWidth,_tHeight);
    world.weaponStock.weaponCycle=0;
    weaponNameLeft=world.weaponStock.name();
    costLeft=world.weaponStock.cost();
    world.weaponStock.weaponCycle=1;
    weaponNameRight=world.weaponStock.name();
    costRight=world.weaponStock.cost();
  }
  
  public Menu loadMenu(){
    return new MenuPanelAction(tX,tY,tWidth,tHeight);
  }
  
  // displays the figures
  public void display(){
    String s;
    
    preparePanel();
    
    textFont(f,14);
    noStroke();
    
    int textY=tY+(panelHeight+14-5)/2;
    if (gameSequence.freeze){
      fill(blink.gradual(colorBackground1,colorText,5));
      s = s_Text_PanelAction_Paused;
      textAlign(CENTER);
      text(s,tX+tWidth/2,textY);
    }
    else {
      fill(colorText);    
      int textX=5;
      s= s_Text_PanelAction_Left;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX += textWidth(s)+5;
      if (world.weaponStock.ammo<costLeft) fill(colorDisabled);
      else fill(colorTextHighlight);
      s=weaponNameLeft;
      textAlign(LEFT);
      text(s,textX,textY);
     
      textX += textWidth(s)+15;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Cost+ costLeft;
      else s= s_Text_Panel_CostUnlimited;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX+= textWidth(s)+menuButtonWidth;
      s= s_Text_PanelAction_Right;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX += textWidth(s)+5;
      if (world.weaponStock.ammo<costRight) fill (colorDisabled);
      else fill(colorTextHighlight);
      s=weaponNameRight;
      textAlign(LEFT);
      text(s,textX,textY);
     
      textX+=textWidth(s)+15;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Cost+ costRight;
      else s= s_Text_Panel_CostUnlimited;
      textAlign(LEFT);
      text(s,textX,textY);
      
      textX+=panelButtonWidth;
      fill(colorText);
      if (limitedAmmo) s= s_Text_Panel_Ammo + world.weaponStock.ammo;
      else s= s_Text_Panel_AmmoUnlimited;
      textAlign(CENTER,BOTTOM);
      text(s,textX,textY-17,tWidth-textX-(menuButtonWidth+5),20);
    }
  }
}

class MenuPanelStrategy extends Menu{
  int x,y,w,h;
    
  MenuPanelStrategy(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  public void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-5;
    cp5.addButton("Menu_PanelStrategy_NextTurn")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_NextTurn)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_NextTurn",desc_Menu_PanelStrategy_NextTurn,desc_Menu_PanelStrategy_NextTurn_Unavailable));
    buttonStatus("Menu_PanelStrategy_NextTurn",Button_Data_Unavailable);
    
    bW = 5;
    cp5.addButton("Menu_PanelStrategy_PreviousWeapon")
       .setPosition(bW,bH)
       .setSize(panelButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_PreviousWeapon)
       .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_PreviousWeapon",desc_Menu_PanelStrategy_PreviousWeapon));
    
    bW += panelButtonWidth+2+panelTextWidth+costTextWidth;
    cp5.addButton("Menu_PanelStrategy_NextWeapon")
       .setPosition(bW,bH)
       .setSize(panelButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelStrategy_NextWeapon)
       .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelStrategy_NextWeapon",desc_Menu_PanelStrategy_NextWeapon));
  }
}

class MenuPanelFinished extends Menu{
  int x,y,w,h;
    
  MenuPanelFinished(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  public void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-textMargin;
    cp5.addButton("Menu_PanelFinished_KeepGoing")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_KeepGoing)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_KeepGoing",desc_Menu_PanelFinished_KeepGoing));
    
    bW = textMargin;
    cp5.addButton("Menu_PanelFinished_BackMenu")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_BackMenu)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_BackMenu",desc_Menu_PanelFinished_BackMenu));
    
    bW += menuButtonWidth+textMargin;
    cp5.addButton("Menu_PanelFinished_Restart")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelFinished_Restart)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelFinished_Restart",desc_Menu_PanelFinished_Restart));
  }
}

class MenuPanelInfo extends Menu{
  int x,y,w,h;
    
  MenuPanelInfo(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;

    createMenuBis();
  }
  
  public void display(){
    fill(colorBackground1);
    stroke(colorLine);
    rect(x,y,w,h);
  }
  
  public void createMenuBis(){
    int bH = y+textMargin;
    int bW = x+textMargin;
    cp5.addButton("Menu_PanelInfo_Stat")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Stat)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Stat",desc_Menu_PanelInfo_Stat));
    
    bH += menuButtonHeight+1;
    cp5.addButton("Menu_PanelInfo_Mission")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Mission)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Mission",desc_Menu_PanelInfo_Mission));
    
    bH += menuButtonHeight+1;
    cp5.addButton("Menu_PanelInfo_Score")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelInfo_Score)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelInfo_Score",desc_Menu_PanelInfo_Score));
  }
  

}

class MenuPanelAction extends Menu{
  int x,y,w,h;
    
  MenuPanelAction(int _x,int _y,int _w, int _h){
    //buttonInfo= new ArrayList(); // Because super consructor is called before
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    createMenuBis();
  }
  
  public void createMenuBis(){
    
    int bH = (panelHeight-menuButtonHeight)/2+y;
    int bW = w-menuButtonWidth-5;
    cp5.addButton("Menu_PanelAction_Pause")
       .setPosition(bW,bH)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_PanelAction_Start)
       ;
    buttonInfo.add(new ButtonInfo("Menu_PanelAction_Pause",desc_Menu_PanelAction_Pause));
  }
}

class InfoPanel{
  InfoArea infoArea;
  Menu menu;
  World wld;
  int infoX, infoY, infoW, infoH;
  int menuWidth;
  int type;
    
  InfoPanel(int x, int y, int w, int h, World _wld){
    menuWidth=menuButtonWidth+2*textMargin;
    menu = new MenuPanelInfo(x,y,menuWidth,h);
    infoX=x+menuWidth;
    infoY=y;
    infoW=w-infoX;
    infoH=h;
    wld=_wld;
    changeInfoPanel(infoPanelRef_Mission);
  }
  
  public void display(){
    menu.display();
    infoArea.display();
  }
  
  public void erase(){
    if (menu!=null) menu.eraseMenu();
  }
  
  public void changeInfoPanel(int infoPanelRef){
    type = infoPanelRef;
    if (infoPanelRef==infoPanelRef_Statistics) {
      infoArea = new InfoAreaStatistics(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Stat");
    }
    else if (infoPanelRef==infoPanelRef_Mission) {
      infoArea = new InfoAreaMission(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Mission");
    }
    else if (infoPanelRef==infoPanelRef_Score) {
      infoArea = new InfoAreaScore(infoX,infoY,infoW,infoH,wld);
      menu.selectOneButton("Menu_PanelInfo_Score");
    }
    else {
      infoArea = null;
      menu.deselectButtons();
    }
  }
  
  public void transitionInfoPanel(int infoPanelRef,String s,int colour){
    type = infoPanelRef;
    if (infoPanelRef==infoPanelRef_TransitionWin) {
      infoArea = new InfoAreaFinished(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Victory,s,colour,infoPanelRef_Score);
      menu.deselectButtons();
    }
    else if (infoPanelRef==infoPanelRef_TransitionLoose) {
      infoArea = new InfoAreaFinished(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Failure,s,colour,infoPanelRef_Score);
      menu.deselectButtons();
    }
    else if (infoPanelRef==infoPanelRef_TransitionWarning) {
      infoArea = new InfoAreaTransition(infoX,infoY,infoW,infoH,wld,s_Menu_PanelInfo_Warning,s,colour,infoPanelRef_Mission);
      menu.deselectButtons();
    }
    else {
      infoArea = null;
      menu.deselectButtons();
    }
  }
}

class InfoArea{
  int x,y,h,w;
  World wld;
  PFont fText;
  int tSize = hoverTextSize;
  int tLeading = hoverTextInterline;
  int hText,maxLine;
  
  InfoArea(){
  }
  
  InfoArea(int _x, int _y, int _w, int _h, World _wld){
    wld = _wld;
    x=_x;
    y=_y;
    h=_h;
    w=_w;
    hText=tSize+tLeading;
    maxLine=h/(hText-2*textMargin);
    fText=fNormal;
  }
  
  public void cleanDisplay(){
    fill(colorBackground2);
    stroke(colorLine);
    rect(x,y,w,h);
  }
  
  public void display(){
    cleanDisplay();
  }
  
  // returns the x according to the position ref.
  // there are 8 possible starting positions for the text
  // It is on purpose that it is fixed width : I don't want the text to widden too much if the screen is.
  public int xPosition(int _ref){
    int ref=max(0,min(7,_ref));
    return x+textMargin+ref*infoWidthPosition;
  }
  
  // returns the y according to the line ref.
  public int yPosition(int _ref){
    int ref=max(0,min(maxLine,_ref));
    return y+textMargin+ref*hText;
  }
  
  public int xWidth(int _ref){
    int ref=max(1,min(8,_ref));
    return infoWidthPosition*ref;
  }
  
  public boolean checkInside(int _x, int _y){
    if (_x>=x && _x<=x+w && _y>=y && _y<=y+h) return true;
    else return false;
  }
}

class InfoAreaStatistics extends InfoArea{
  Graph graph;
  
  InfoAreaStatistics(int _x, int _y, int _w, int _h,  World _wld){
    super(_x,_y,_w,_h,_wld);
    graph = new Graph(x+1,y+1,w-2,h-2,wld.datas);
    graph.bgColor(colorBackground2);
    graph.drColor(colorText);
    graph.draw();
  }
  
  public void display(){
    
    if (clock%2==0) {
      cleanDisplay();
      graph.draw();
    }
  }
}

class InfoAreaMission extends InfoArea{
  
  InfoAreaMission(int _x, int _y, int _w, int _h, World _wld){
    super(_x,_y,_w,_h,_wld);
  }
  
  public void display(){
    cleanDisplay();
    int j=0;
   
    // prepares the font
    textFont(fText,tSize);
    textAlign(LEFT,CENTER);
    int number = mission.events.size();
     
    fill(colorWin);
    text(s_InfoMission_WinningCondition,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Win){
        displayEvent(event,j);
        j++;
      }
    }
    
    j++;
    fill(colorLoose);
    text(s_InfoMission_LoosingCondition,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Loose){
        displayEvent(event,j);
        j++;
      }
    }
    
    j++;
    fill(colorMisc);
    text(s_InfoMission_Misc,xPosition(0),yPosition(j),xWidth(2),hText);
    for (int i=0;i<number;i++){
      Event event= (Event) mission.events.get(i);
      if (event.type==EventType_Misc){
        displayEvent(event,j);
        j++;
      }
    }
  }
  
  public void displayEvent(Event event,int j){
    String s;
    int color1, color2,color3;
    
    if (event.type==EventType_Misc) s=event.description;
    else s=event.condition.description();
    
    if (event.active){
      if (event.blink && !gameSequence.finished){
        color1=colorDisabled;
        color2=blink.flash(colorText,colorDisabled,5);
        color3=color2;
      }
      else {
        color1=colorDisabled;
        color2=colorText;
        color3=colorDisabled;
      }
    }
    else {
      if (event.done){
        color1=colorDisabled;
        if (event.type==EventType_Misc && event.score()==0) color2=colorDisabled;
        else color2=mission.getEventColor(event.type);
        color3=mission.getEventColor(event.type);
      }
      else {
        color1=colorDisabled;
        color2=colorDisabled;
        color3=color2;
      }
    }
    fill(color2);
    text(s,xPosition(2),yPosition(j),xWidth(4),hText);
    String s2= event.progression()+"";
    text(s2,xPosition(6),yPosition(j),xWidth(1),hText);
    fill(color1);
    text(event.target(),xPosition(6)+textWidth(s2),yPosition(j),xWidth(1),hText);
    fill(color3);
    if (event.score()!=0) text(event.score()+"pt",xPosition(7),yPosition(j),xWidth(1),hText);
    
  }
}

class InfoAreaScore extends InfoArea{
  
  String message;
  
  InfoAreaScore(int _x, int _y, int _w, int _h,World _wld){
    super(_x,_y,_w,_h,_wld);
  }
  
  public void display(){
    cleanDisplay();
    
    
    if (!gameSequence.finished){
      textFont(fText,tSize);
      fill(colorText);
      textAlign(CENTER,CENTER);
      text(s_InfoScore_Unfinished,x,y,w,h);
    }
    else {
      ScoreItem item;
      textFont(fBold,tSize);
      fill(colorText);
      textAlign(CENTER,CENTER);
      int j=0;
      text(s_InfoScore_Number,xPosition(3),yPosition(j),xWidth(1),hText);
      text(s_InfoScore_Multiplier,xPosition(4),yPosition(j),xWidth(1),hText);
      text(s_InfoScore_Score,xPosition(5),yPosition(j),xWidth(1),hText);
      j++;
      textFont(fText,tSize);
      for (int i=0;i<mission.scoreItems.size();i++){
        item = (ScoreItem) mission.scoreItems.get(i);
        if (i==mission.scoreItems.size()-1){
          if (gameSequence.win)fill(colorWin);
          else fill(colorLoose);
        }//fill(colorText);
        textAlign(LEFT,CENTER);
        text(item.itemText()+" : ",xPosition(0),yPosition(j),xWidth(3),hText);
        textAlign(CENTER,CENTER);
        text(item.value(),xPosition(3),yPosition(j),xWidth(1),hText);
        text(item.multiplier(),xPosition(4),yPosition(j),xWidth(1),hText);
        text(item.score(),xPosition(5),yPosition(j),xWidth(1),hText);
        j++;
      }
      
      fill(colorText);
      textFont(fBold,tSize);
      text(s_InfoScore_FinalScore,xPosition(4),yPosition(j),xWidth(1),hText);
      //fill(colorText);
      //fill(blink.gradual(colorText,colorBackground2,5));
      text(mission.finalScore+"",xPosition(5),yPosition(j),xWidth(1),hText);
      textAlign(LEFT,CENTER);
      textFont(fText,tSize);
    }
  }
}

class InfoAreaTransition extends InfoArea{
  String s1,s2;
  int colour;
  Blink blinkLocal;
  int nextType;
  
  InfoAreaTransition(int _x, int _y, int _w, int _h, World _wld,String _s1,String _s2,int _colour,int _nextType){
    super(_x,_y,_w,_h,_wld);
    s1=_s1;
    s2=_s2;
    colour=_colour;
    blinkLocal = new Blink();
    nextType=_nextType;
  }
  
  public void display(){
    cleanDisplay();
    
    textAlign(CENTER,CENTER);
    textFont(fText,tSize*2);
    fill(blinkLocal.gradual(colour,colorBackground2,12));
    text(s1,x,y,w,hText*4);
    textFont(fText,(tSize*3)/2);
    fill(colour);
    text(s2,x,yPosition(4),w,hText*2);
    checkNextPanel();
    
    blinkLocal.forward();
  }
  
  public void checkNextPanel(){
    if (blinkLocal.checkCycle(6)) gameScreen.infoPanel.changeInfoPanel(nextType);
  }
}

class InfoAreaFinished extends InfoAreaTransition{
  
  InfoAreaFinished(int _x, int _y, int _w, int _h, World _wld,String _s1,String _s2,int _colour,int _nextType){
    super(_x,_y,_w,_h,_wld,_s1,_s2,_colour,_nextType);
  }
  
  public void display(){
    super.display();
    textFont(fText,tSize);
    fill(colorText);
    textAlign(CENTER,CENTER);
    text(s_Menu_PanelInfo_FinishedScore,x,yPosition(7),w,hText);
  }
  
  public void checkNextPanel(){
  }
}

// This class is the area where things are happening
class PlayArea{
  int gX,gY,gWidth,gHeight;
  
  PlayArea(int _gX, int _gY, int _gWidth, int _gHeight){
    gX = _gX;
    gY = _gY;
    gWidth=_gWidth;
    gHeight=_gHeight;
  }
  
  // Creates and draw the map (without the beings)
  public void drawMap(){
    int x,y,w,h;
    
    fill(wall);
    rect(0,0,gWidth,gHeight); // Fills the whole area with walls
    fill(wall);
    stroke(empty);
    for (int i=0; i<100; i++){ // Builds the passages (100 rectangles)
      x=(int)random(gWidth/scale)*scale-(gWidth/8/scale)*scale;
      y=(int)random(gHeight/scale)*scale-(gHeight/8/scale)*scale;
      w=((int)random(gWidth/4/scale)*scale+2)*scale-1;
      h=((int)random(gWidth/4/scale)*scale+2)*scale-1;
      for (int j=0;j<scale;j++) rect(x+j,y+j,w-2*j,h-2*j);
    }
 
    fill(empty);
    //noStroke();
    stroke(empty);
    for (int i=0; i<45; i++){ // Builds 45 rooms
      x=(int)random(gWidth/scale)*scale;
      y=(int)random(gHeight/scale)*scale;
      w=((int)random(40)+10)*scale-1;
      h=((int)random(40)+10)*scale-1;
      rect(x,y,w,h); 
      //x=(int)random(gWidth-1)+1;
      //y=(int)random(gHeight-1)+1;
    }
  }
  
  // Check if the given coordinates are inside the playArea
  public boolean inside(int x, int y){
    if (x>=gX && x<gX+gWidth && y>=gY && y<gY+gHeight) return true;
    return false;
  }
  
  // gives an x value that is inside the playArea 
  public int setX(){
    return gX+(int)random(gWidth/scale)*scale;
  }
  
  // gives an y value that is inside the playArea
  public int setY(){
    return gY+(int)random(gHeight/scale)*scale;
  }
}

// The classes in the file define the generic menu -> everywhere where there are some buttons
// It is later used in GameScreen and InitScreen


// Class that will hold some extra info about buttons usefull for the display of comment,
// and maybe manage the interaction.
class ButtonInfo {
  String name;
  String description; // description when overing the button
  String descriptionUnavailable; // description when overing the button when unavailable
  int data; // data related to the status of the button (highlighted, disabled,...)
  int data2; // some more data. Such as weapons ref
  
  ButtonInfo(String _name, String _description){
    name=_name;
    description=_description;
    descriptionUnavailable="";
    data=Button_Data_Normal;
    data2=0;
  }
  
  ButtonInfo(String _name, String _description, String _descriptionUnavailable){
    name=_name;
    description=_description;
    descriptionUnavailable=_descriptionUnavailable;
    data=Button_Data_Normal;
    data2=0;
  }
  
  
  ButtonInfo(String _name, String _description, String _descriptionUnavailable, int _data){
    name=_name;
    description=_description;
    descriptionUnavailable=_descriptionUnavailable;
    data=_data;
    data2=0;
  }
  
  public String name(){
    return name;
  }
  
  public String description(){
    if ((data == Button_Data_Normal) || (data == Button_Data_Selected)) return description;
    else if ( data == Button_Data_Unavailable) return descriptionUnavailable;
    else return desc_Button_Unknown;
  }
}

class HoverInfo{
  int x, y, w, h,wMargin,hMargin;
  PFont fText;
  int tSize = 12;
  int tLeading = tSize+3;
  
  HoverInfo(int _x, int _y, int _w, int _h, int _wMargin, int _hMargin){
    x= _x;
    y= _y;
    w= _w;
    h= _h;
    wMargin= _wMargin;
    hMargin= _hMargin;
    
    fText=fNormal; 
  }
  
  public boolean display(Menu menu){
    noStroke();
    fill(colorHoverField);
    rect(x,y,w,h);
    
    textFont(fText,tSize);
    textLeading(tLeading);
    fill(colorHoverText);
    if (fps){
      textAlign(LEFT,CENTER);
      text("fps: "+decimalFormat.format(frameRate),x+hoverHeightMargin,y+hMargin,w-2*wMargin,h-2*hMargin);
    }
    if (menu!=null){
      if (cp5.isMouseOver()){
        String s ="";
        for (int i=0;i<menu.itemLength();i++){
          if (cp5.isMouseOver(cp5.getController(menu.itemName(i)))){
            s = menu.itemDescription(i);
            break;
          }
        }
        if (s!=""){
          textAlign(CENTER,CENTER);
          text(s,x+wMargin,y+hMargin,w-2*wMargin,h-2*hMargin);
          return true;
        }
      }
    }
    return false;
  }
}

// This class creates generic menus
class Menu{
  
  ArrayList buttonInfo;
  
  Menu(){
    buttonInfo= new ArrayList();
    createMenu();
  }

  public void createMenu(){
  }

  public int itemLength(){
    return buttonInfo.size();
  }

  public void eraseMenu(){
    for (int i=0; i<itemLength();i++) {
      ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
      // Added condition bellow, because in mutliList, many buttons are erased automatically
      // So trying to erase them again causes errors.
      // Could be improved If I knew the type returned by cp5.controller();
      if (cp5.controller(bInfo.name)!=null) cp5.controller(bInfo.name).remove();
    }
  }
  
  // Finds the ButtonInfo in the arrayList which as the same name. Otherwise, returns null
  public ButtonInfo findItem(String _name){
      ButtonInfo bInfo = null;
      
      for (int i=0; i<buttonInfo.size(); i++){
        bInfo = (ButtonInfo) buttonInfo.get(i);
        if (bInfo.name.equals(_name)) {
          return bInfo;
        }
      }
      return null;
  }
  
  // Changes the color of the botton, according to its status
  // Stores its status as well
  public void buttonStatus(String buttonName, int bStatus){
    ButtonInfo bInfo = findItem(buttonName);
    if (bInfo!=null) {
      bInfo.data = bStatus;
    }
    
    Controller button = cp5.controller(buttonName);
    if (button != null){
      if (bStatus==Button_Data_Selected) {
        //button.unlock();
        button.setColorBackground(color(125,125,0));
        button.setColorForeground(color(175,175,0));
        button.setColorActive(color(200,200,50));
      }
      else if (bStatus==Button_Data_Normal) {
        //button.unlock();
        button.setColorBackground(color(2,52,77));
        button.setColorForeground(color(1,108,158));
        button.setColorActive(color(0,180,234));
      }
      else if (bStatus==Button_Data_Unavailable) {
        //button.lock(); // Actually, I don't want to lock, otherwise, I can't know that I am hovering them.
        button.setColorBackground(color(30,30,30));
        button.setColorForeground(color(60,60,60));
        button.setColorActive(color(60,60,60));
      }
    }
  }
  
  public void selectOneButton(String s){
    for (int i=0; i<buttonInfo.size();i++){
      if (s.equals(itemName(i))) buttonStatus(itemName(i),Button_Data_Selected);
      else buttonStatus(itemName(i),Button_Data_Normal);
    }
  }
  
  public void deselectButtons(){
    for (int i=0; i<buttonInfo.size();i++){
      buttonStatus(itemName(i),Button_Data_Normal);
    }
  }
  
  public void updateListButtons(String name,int listSize, int ref){
    ButtonInfo bInfo;
    for (int i = 0; i<listSize ; i++){
       if (cp5.controller(name+" "+i)!=null) {
         bInfo = findItem(name+" "+i);
         if (i == ref) buttonStatus(name+" "+i,Button_Data_Selected);
         else buttonStatus(name+" "+i,Button_Data_Normal);   
       }  
    }
  }
  
  // Returns the name of a button
  public String itemName(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.name();
  }
  
  // Retunrs the description of a button
  public String itemDescription(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.description();
  }
  
  // Returns the data value of a button
  public int itemData(int i){
    ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
    return bInfo.data;
  }
  
  // Allows the menu to display things that needs to be refreshed. Such as text or other things.
  public void display(){
  }
  
  // set default value to menu. Nothing to do in that specific menu, but more in others
  public void defaultValues(){
  }
  
  // get the values from the menu. Nothing to do in that specific menu, but more in others
  public void getValues(){
  }
}
int EventType_Misc = 0;
int EventType_Win = 1;
int EventType_Loose = 2;
int ConditionResult_True = 0;
int ConditionResult_False = 1;
int ConditionResult_Warning = 2;

int scoreHumansAlive = 2;
int scoreHumansKilled = -5;
int scoreZombiesAlive = -200;
int scoreZombiesAliveMini = -2;
int scoreZombiesKilled = 1;


class Mission{
  ArrayList events;
  ArrayList scoreItems;
  ArrayList missionDefinitions;
  World wld;
  int finalScore;
  int specialScore,specialScoreNumber;
  int victoryScore;
  int missionRef;
  String victoryMessage;
  
  
  Mission(World _wld){
    wld=_wld;
    missionDefinitions = loadDefinitions();
    missionRef=0;
    setMission(missionRef);
    specialScore=0;
    specialScoreNumber=0;
    victoryScore=0;
    victoryMessage="";
  }
  
  public void clean(){
    specialScore=0;
    specialScoreNumber=0;
    victoryScore=0;
    victoryMessage="";
    scoreItems=null;
    setMission(missionRef);
  }
  
  public ArrayList loadDefinitions(){
    ArrayList mDefs = new ArrayList();
    
    mDefs.add(new MissionVanilla(this,wld));
    mDefs.add(new MissionRegular(this,wld));
    mDefs.add(new MissionSecond(this,wld));
    mDefs.add(new MissionThird(this,wld));
    mDefs.add(new MissionMadScientist(this,wld));
    
    return mDefs;
  }
  
  public void setMission(int i){
    
    if (missionDefinitions!=null){
      int ref = max(0,min(i,missionDefinitions.size()-1));
      missionRef=ref;
      MissionDefinition missionDef = (MissionDefinition) missionDefinitions.get(ref);
      events = missionDef.create();
    }
    else events=null;
  }
  
  // Check every conditions.
  public void checkEvent(){
    Event event;
    for (int i=0;i<events.size();i++){
      event = (Event) events.get(i);
      event.check();
    }
  }
  
  public int getEventColor(int eventType){
    if (eventType==EventType_Win)  return colorWin;
    else if (eventType==EventType_Loose) return colorLoose;
    else return colorMisc;
  }
  
  public void score(){
    
    finalScore=0;
    scoreItems = new ArrayList();
    
    ScoreItem item;
    item = new ScoreItem(s_Score_HumansAlive,wld.numHuman,scoreHumansAlive);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_HumansKilled,wld.numKilledHumans+wld.numInoculatedHumans,scoreHumansKilled);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_ZombiesAlive,wld.numZombies,multiplierZombie(wld.numZombies));  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItem(s_Score_ZombiesKilled,wld.numKilledZombies,scoreZombiesKilled);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItemSpecial(s_Score_Special,specialScoreNumber,specialScore);  
    scoreItems.add(item);
    finalScore+=item.score;
    item = new ScoreItemVictory(s_Score_MissionAccomplishment,victoryMessage,victoryScore);  
    scoreItems.add(item);
    finalScore+=item.score;
    
  }
  
  public void specialScore(int _specialScore){
    specialScore+=_specialScore;
    specialScoreNumber++;
  }
  
  public void victoryScore(int _victoryScore,boolean _victory){
    victoryScore=_victoryScore;
    if (_victory) victoryMessage = s_Score_Won;
    else victoryMessage = s_Score_Lost;
  }
  
  public int multiplierZombie(int numZombies){
    if (numZombies>0) return (int) min(scoreZombiesAlive/sqrt(wld.numZombies),scoreZombiesAliveMini);
    else return 0; 
  }
}

class MissionDefinition{
  String name,description;
  Mission miss;
  World wld;
  
  MissionDefinition(Mission _miss,World _wld){
    miss=_miss;
    wld=_wld;
    name="";
    description="";
  }
  
  public ArrayList create(){
    return null;
  }
  
}

class MissionRegular extends MissionDefinition{
  
  MissionRegular(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Regular;
    description=desc_Mission_Regular;
  }
  
  public ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,20),new ActionLost(-2000),miss);
    events.add(event);
    event = new Event(new CondTime(wld,900),new ActionBombZombies(wld,-1000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionMadScientist extends MissionDefinition{
  
  MissionMadScientist(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_MadScientist;
    description=desc_Mission_MadScientist;
  }
  
  public ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,3),miss);
    events.add(event);
    event = new Event(new CondTime(wld,90),new ActionInfect(wld,1),miss);
    events.add(event);
    event = new Event(new CondTime(wld,180),new ActionInfect(wld,1),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionLost(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,0),new ActionWin(-2000),miss);
    events.add(event);
    event = new Event(new CondTime(wld,500),new ActionBombZombies(wld,1000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionSecond extends MissionDefinition{
  
  MissionSecond(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Second;
    description=desc_Mission_Second;
  }
  
  public ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondTime(wld,90),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondTime(wld,180),new ActionInfect(wld,2),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,5),new ActionLost(-2000),miss);
    events.add(event);
    event = new Event(new CondHumanKilledMax(wld,5),new ActionLost(-2000),miss);
    events.add(event);
    
    return events;
  }
}

class MissionThird extends MissionDefinition{
  
  MissionThird(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Third;
    description=desc_Mission_Third;
  }
  
  public ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondZombieMax(wld,30),new ActionLost(-2000),miss);
    events.add(event);

    
    return events;
  }
}

class MissionVanilla extends MissionDefinition{
  
  MissionVanilla(Mission _miss,World _wld){
    super(_miss,_wld);
    name=s_Mission_Vanilla;
    description=desc_Mission_Vanilla;
  }
  
  public ArrayList create(){
    ArrayList events;
    
    events = new ArrayList();
    Event event;
    event = new Event(new CondTime(wld,0),new ActionInfect(wld,4),miss);
    events.add(event);
    event = new Event(new CondZombieDead(wld),new ActionWin(2000),miss);
    events.add(event);
    event = new Event(new CondHumanDead(wld,0),new ActionLost(-2000),miss);
    events.add(event);

    
    return events;
  }
}

class ScoreItem{
  String itemText;
  int value, multiplier, score;
  
  ScoreItem(){
  }
  
  ScoreItem(String _text,int _value,int _multiplier){
    itemText= _text;
    value=_value;
    multiplier=_multiplier;
    score=value*multiplier;
  }
  
  public String itemText(){
    return itemText;
  }
  
  public String value(){
    String s=value+"";
    return s;
  }
  
  public String multiplier(){
    String s="( x "+multiplier+" )";
    return s;
  }
  
  public String score(){
    String s=score+"";
    return s;
  }
}

class ScoreItemSpecial extends ScoreItem{
  
  ScoreItemSpecial(String _text,int _value, int _score){
    itemText= _text;
    value=_value;
    score=_score;
  }
  
  public String multiplier(){
    return "";
  }
  
}

class ScoreItemVictory extends ScoreItem{
  String sValue;
  
  ScoreItemVictory(String _text,String _value, int _score){
    itemText= _text;
    sValue=_value;
    score=_score;
  }
  
  public String value(){
    return sValue;
  }
  
  public String multiplier(){
    return "";
  }
  
}


class Event{
  Condition condition;
  Action action;
  boolean active; // If active, event will be checked
  boolean done; // Tells that this event has been executed once already
  boolean blink; // When displayed, that event should be blinking
  int type;
  String description,description2;
  Mission miss;
  
  Event(Condition _condition,Action _action,Mission _mission){
    condition = _condition;
    action = _action;
    active = true;
    blink = false;
    done = false;
    miss=_mission;
    type=action.type();
    description=condition.description()+" - "+action.description();
    description2=condition.descWarning();
  }
  
  // Check every conditions. Sends "true" i
  public void check(){
    if (active){
      int result = condition.evaluate();
      if (result == ConditionResult_True) {
        active=false;
        if (!gameSequence.finished || (type != EventType_Loose && type != EventType_Win)) done=true;
        action.act(this);
      }
      else if (!blink && result == ConditionResult_Warning){ // starts blinking if condition is met
        blink = true;
        gameSequence.warning(description2,type);
      }
      else if (blink && result == ConditionResult_False){ // allows to stop blinking if the condition is not met anymore
        blink = false;
      }
    }
  }
  
  public int progression(){
    return condition.progression();
  }
  
  public int score(){
    return action.score();
  }
  
  public String target(){
    return condition.target();
  }
}

class Action{
  String description;
  int type;
  int score;
  
  Action(){
    type=EventType_Misc;
    description=s_Event_None;
    score=0;
  }
  
  public void act(Event event){
  }
  
  public int type(){
    return type;
  }
  
  public int score(){
    return score;
  }
  
  public String description(){
    return description;
  }
  
  
}

class ActionInfect extends Action{
  int number;
  World wld;
  
  ActionInfect(World _wld,int _number){
    wld =_wld;
    number=_number;
    type= EventType_Misc;
    description=s_Event_Infect_1+number+s_Event_Infect_2;
  }
  
  public void act(Event event){
    int j=0;
    int i=0;
    
    while (i<number && j<wld.initPop){
      while (!(wld.beings[j] instanceof Human)&&j<wld.initPop){
        j++;
      } 
      if (j<wld.initPop){
        wld.beings[j].infect(0,iszombie);
        i++;
        j++;
      }
    }
  }
}

class ActionWin extends Action{
  
  ActionWin(int _score){
    type=EventType_Win;
    score=_score;
  }
  
  public void act(Event event){
    event.miss.victoryScore(score,true);
    gameSequence.won(event.condition.descDone());
  }
}

class ActionLost extends Action{
  
  ActionLost(int _score){
    type=EventType_Loose;
    score=_score;
  }
  
  public void act(Event event){
    event.miss.victoryScore(score,false);
    gameSequence.lost(event.condition.descDone());
  }
}

class ActionBombZombies extends Action{
  World wld;
 
  ActionBombZombies(World _wld,int _score){
    wld=_wld;
    type=EventType_Misc;
    score=_score;
    description=s_Event_BombZombies;
  }
  
  public void act(Event event){
    wld.effects.add(new EffectBombing(wld,1,100));
    if (score!=0) event.miss.specialScore(score);
  }
}

class Condition{
  World wld;
  String description;
  String descWarning;
  String descDone;
  int progression;
  String target;
  
  Condition(World _wld){
    wld = _wld;
    description=s_Event_None;
    descWarning=s_Event_None;
    descDone=s_Event_None;
    progression=0;
    target="";
  }
  
  public int evaluate(){
    return ConditionResult_False;
  }
  
  public String description(){
    return description;
  }
  
  public String descWarning(){
    return descWarning;
  }
  
  public String descDone(){
    return descDone;
  }
  
  public int progression(){
    return progression;
  }
  
  public String target(){
    return target;
  }
}

// Condition returns true if more than proportion of the population has been turned into zombies
class CondZombieDead extends Condition{
  
  CondZombieDead(World _wld){
    super(_wld);
    description=s_Event_Condition_ZombieDead;
    descDone=d_Event_Condition_ZombieDead;
    target="";
  }
  
  public int evaluate(){
    //Data data = (Data) wld.datas.get(dataZombie);
    progression = wld.numZombies;
    if (wld.time>1 && wld.numZombies==0) return ConditionResult_True;
    return ConditionResult_False;
  }
}

// Condition returns true if more than x zombies
class CondZombieMax extends Condition{
  int number;
  int warning;
  
  CondZombieMax(World _wld, int _proportion){
    super(_wld);
    number=_proportion*wld.initPop/100;
    warning=number*80/100;
    description=s_Event_Condition_ZombieMax_1+number+s_Event_Condition_ZombieMax_2;
    descWarning=w_Event_Condition_ZombieMax_1+number+w_Event_Condition_ZombieMax_2;
    descDone=d_Event_Condition_ZombieMax_1+number+d_Event_Condition_ZombieMax_2;
    target="/"+number;
  }
  
  public int evaluate(){
    progression=wld.numZombies;
    if (wld.numZombies>number) return ConditionResult_True;
    else if (wld.numZombies>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

// Condition returns true if killed more than x humans
class CondHumanKilledMax extends Condition{
  int number;
  int warning;
  
  CondHumanKilledMax(World _wld, int _proportion){
    super(_wld);
    number=_proportion*wld.initPop/100;
    warning = number*80/100;
    description=s_Event_Condition_HumanKilledMax_1+number+s_Event_Condition_HumanKilledMax_2;
    descWarning=w_Event_Condition_HumanKilledMax_1+number+w_Event_Condition_HumanKilledMax_2;
    descDone=d_Event_Condition_HumanKilledMax_1+number+d_Event_Condition_HumanKilledMax_2;
    target="/"+number;
  }
  
  public int evaluate(){
    progression=wld.numKilledHumans+wld.numInoculatedHumans;
    if (wld.numKilledHumans+wld.numInoculatedHumans>number) return ConditionResult_True;
    else if (wld.numKilledHumans+wld.numInoculatedHumans>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

// Condition returns true if a proportion of the population is dead
class CondHumanDead extends Condition{
  int number;
  int warning;
  
  CondHumanDead(World _wld,int _proportion){
    super(_wld);
    
    number=_proportion*wld.initPop/100;
    warning=(number*150+wld.initPop*10)/100;
    description=s_Event_Condition_HumanDead_1+number+s_Event_Condition_HumanDead_2;
    descWarning=w_Event_Condition_HumanDead_1+number+w_Event_Condition_HumanDead_2;
    descDone=d_Event_Condition_HumanDead_1+number+d_Event_Condition_HumanDead_2;
    target="/"+number;
  }
  
  public int evaluate(){
    progression=wld.numHuman;
    if (wld.numHuman<=number) return ConditionResult_True;
    else if (wld.numHuman<=warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}

class CondTime extends Condition{
  int time;
  int warning;
  
  CondTime(World _wld,int _time){
    super(_wld);
    time = _time;
    warning=time*80/100;
    description=s_Event_Condition_Time_1+time+s_Event_Condition_Time_2;
    descWarning=w_Event_Condition_Time_1+time+w_Event_Condition_Time_2;
    descDone=d_Event_Condition_Time_1+time+d_Event_Condition_Time_2;
    target="/"+time;
  }
  
  public int evaluate(){
    progression=wld.time;
    if (wld.time>time) return ConditionResult_True;
    else if (wld.time>warning) return ConditionResult_Warning;
    return ConditionResult_False;
  }
}
// This will dispay every information in the start screen, except the buttons 
// buttons will be (handled in GameSequence)
class StartScreen{

  Menu menu;
  HoverInfo hoverInfo;
  
  StartScreen(){
   
    if (gameSequence.previousStartMenu == menuAction) menu = new MenuAction();
    else if(gameSequence.previousStartMenu == menuStrategy) menu = new MenuStrategy();
    else menu = new MenuIntro();
    
    int fieldHeight=(hoverTextSize+hoverTextInterline)*3+hoverHeightMargin*2;
    hoverInfo = new HoverInfo(0,height-fieldHeight,width,fieldHeight,hoverWidthMargin,hoverHeightMargin);
  }
  
  public void display(){
    background(0);
    textAlign(CENTER);
    textFont(fTitle,menuTitleSize);
    fill(colorIntroTitle);
    text("Infection control",menuTitleBorder,menuTitleBorder,width-menuTitleBorder*2,menuTitleBorder*1.5f);
  
    // Check if the mouse is overing a button, and displays corresponding text
    hoverInfo.display(menu);
  }      
}

// This class creates the general menu on the introduction screen
class MenuIntro extends Menu{

  public void createMenu(){
    int n = 0;
    int x = width/2-menuButtonWidth/2;
    int y = (int) (menuTitleBorder*2+ menuTitleBorder*1.5f);
    int h = menuButtonHeight+1;
    
    cp5.addButton("Menu_Action")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Action)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action",desc_Menu_Action));
       
    n+=1; 
    cp5.addButton("Menu_Strategy")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Strategy)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy",desc_Menu_Strategy));
       
    n+=1; 
    cp5.addButton("Menu_Options")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Options)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Options",desc_Menu_Options)); 
  }

  public int itemLength(){
    return buttonInfo.size();
  }

  public void eraseMenu(){
    for (int i=0; i<itemLength();i++) {
      ButtonInfo bInfo = (ButtonInfo) buttonInfo.get(i);
      // Added condition bellow, because in mutliList, many buttons are erased automatically
      // So trying to erase them again causes errors.
      // Could be improved If I knew the type returned by cp5.controller();
      if (cp5.controller(bInfo.name)!=null) cp5.controller(bInfo.name).remove();
    }
  }
  
  // Finds the ButtonInfo in the arrayList which as the same name. Otherwise, returns null
  public ButtonInfo findItem(String _name){
      ButtonInfo bInfo = null;
      
      for (int i=0; i<buttonInfo.size(); i++){
        bInfo = (ButtonInfo) buttonInfo.get(i);
        if (bInfo.name.equals(_name)) {
          return bInfo;
        }
      }
      return null;
  }
}

// Creates the menu related to the actions.
class MenuAction extends Menu{
                     
  public void createMenu(){
    
    int n = 0;
    int x = width/2-menuButtonWidth/2;
    int y = (int) (menuTitleBorder*2+menuTitleBorder*1.5f);
    int h = menuButtonHeight+1;
    
    // Iniates the customList the way we want (different than strategy game)
    if (gameSequence.previousStartMenu != menuAction){
      CustomList wList = (CustomList) world.weaponStock.weaponsList.get(1);
      wList.setSize(2);
    }
     
    // Adds the buttons under each other
    cp5.addButton("Menu_Action_Start")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Action_Start)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action_Start",desc_Menu_Action_Start));
    
    n+=1;  
    n+=1;
    MultiList l = cp5.addMultiList("Menu_Action_Mission",x,y+n*h,menuButtonWidth,menuButtonHeight);
    buttonInfo.add(new ButtonInfo("Menu_Mission",desc_Menu_Mission));
    customizeMission(l,s_Menu_Mission,desc_Menu_Mission);
    
    n+=1;
    l = cp5.addMultiList("Menu_Action_Left",x,y+n*h,menuButtonWidth,menuButtonHeight);
    buttonInfo.add(new ButtonInfo("Menu_Action_Left",desc_Menu_Action_Left));
    customize(l,s_Menu_Action_Left,desc_Menu_Action_Left);
      
    n+=1;
    l = cp5.addMultiList("Menu_Action_Right",x,y+n*h,menuButtonWidth,menuButtonHeight);
    buttonInfo.add(new ButtonInfo("Menu_Action_Right",desc_Menu_Action_Right));
    customize(l,s_Menu_Action_Right,desc_Menu_Action_Right);
    
    n+=1;
    cp5.addToggle("Menu_Action_Ammo")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Action_Ammo_Unlimited)
       .getCaptionLabel().align(ControlP5.LEFT, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action_Ammo",desc_Menu_Action_Ammo));
    
    int temp = world.weaponStock.ammoInit;
    n+=1;
    cp5.addSlider("Menu_AmmoInit")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,6)
       .setNumberOfTickMarks(7)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_AmmoInit)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_AmmoInit",desc_Menu_AmmoInit,desc_Menu_Action_AmmoOff));
    
    temp = world.weaponStock.restockRate;
    n+=1;
    cp5.addSlider("Menu_AmmoRestock")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,world.weaponStock.listRestock.length-1)
       .setNumberOfTickMarks(world.weaponStock.listRestock.length)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_AmmoRestock)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_AmmoRestock",desc_Menu_AmmoRestock,desc_Menu_Action_AmmoOff));
    
    temp = gameSequence.speed; // for weird reason, action is triggered as a parallel process, so need to store data temporarilly
    n+=1;
    n+=1;
    cp5.addSlider("Menu_Action_Speed")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,gameSequence.listSpeed.length-1)
       .setNumberOfTickMarks(gameSequence.listSpeed.length)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_Action_Speed)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action_Speed",desc_Menu_Action_Speed));
    
    n+=1;
    n+=1;
    cp5.addButton("Menu_Action_Default")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Action_Default)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action_Default",desc_Menu_Action_Default));
    
    n+=1;
    cp5.addButton("Menu_Action_Back")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Action_Back)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Action_Back",desc_Menu_Action_Back));
    
    if (gameSequence.previousStartMenu == menuAction) previousValues();
    else defaultValues();
  }
  
  // Fills the multilist with the name of every weapons contained in the last list of weaponStock
  // Counts the number of buttons created.
  public void customize(MultiList l,String name,String description){
    
    int buttonNumber=0;
    
    l.closeDelay=5; // delay after which must close the list (in ms ?) when mouse stopped overing
    
    MultiListButton b;
    b = l.add(name,-1);
    buttonInfo.add(new ButtonInfo(name,description));
    
    WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(0);
    wList.menuListWeapons(name,b,buttonInfo,0);
  }
  
  public void customizeMission(MultiList l,String name,String description){
    
    int buttonNumber=0;
    
    l.closeDelay=5; // delay after which must close the list (in ms ?) when mouse stopped overing
    
    MultiListButton b;
    b = l.add(name,-1);
    buttonInfo.add(new ButtonInfo(name,description));
    
    for (int i=0; i<mission.missionDefinitions.size();i++){
      // Button for the name of the list
      MissionDefinition mDef = (MissionDefinition) mission.missionDefinitions.get(i);
      MultiListButton c = b.add(name+" "+i,i);
      buttonInfo.add(new ButtonInfo(name+" "+i,mDef.description));
      c.setLabel(mDef.name);
      c.setWidth(PApplet.parseInt (menuButtonWidth*1.3f));
    }
  }
  
  // Updates the visual aspect of the weapons in multilists according to status (selected, unavailable, normal) 
  // wNumber : of weapons in the multilist
  // wListCheck : list of weapons in stock
  // wListStock : list of weapons selected (here left, and right click
  // position : ref of weapon in wList
  public void updateWeaponButtons(WeaponsList wListCheck,WeaponsList wListStock){
    String name;
    ButtonInfo bInfo;
    int position;
    WeaponManager wManager = new WeaponManager();
    
    for (position=0; position<wListStock.length();position++){
    
      if (position == 0) name = s_Menu_Action_Left;
      else if (position == 1) name = s_Menu_Action_Right;
      else return;
      
      for (int i = 0; i<wListCheck.length() ; i++){
         if (cp5.controller(name+" "+i)!=null) {
           Weapon weapon = (Weapon) wManager.loadOneWeapon(wListCheck.get(i));
           bInfo = findItem(name+" "+i);
           // if weapon has been selected, highlight it
           if (i == wListStock.get(position)) buttonStatus(name+" "+i,Button_Data_Selected);
           // if weapon needs another weapon that is not selected, set it in grey
           else if (!weapon.checkRequiredWeapon(wListStock,position)) buttonStatus(name+" "+i,Button_Data_Unavailable);
           // otherwise, regular color
           else buttonStatus(name+" "+i,Button_Data_Normal);   
         }  
      }  
    }
  }
  
  // Selects the weapons at the given position
  public void selectWeapon(int value,int position){
    String name;
    ButtonInfo bInfo;
    
    if (position == 0) name = s_Menu_Action_Left;
    else if (position == 1) name = s_Menu_Action_Right;
    else return;
    bInfo = findItem(name+" "+value);
    
    if (bInfo!=null ){
      if (bInfo.data == Button_Data_Normal && value>=0){
        world.weaponStock.setCustomListValue(position,value); 
        updateWeaponButtons((WeaponsList) world.weaponStock.weaponsList.get(0),
                            (WeaponsList) world.weaponStock.weaponsList.get(1));
      }
    }
  }
  
  // Selects the Mission
  public void selectMission(int value){
    String name;
    ButtonInfo bInfo;
    
    name = s_Menu_Mission;
    bInfo = findItem(name+" "+value);
    
    if (bInfo!=null ){
      if (bInfo.data == Button_Data_Normal && value>=0){
        
        updateListButtons(name,mission.missionDefinitions.size(),value);
        mission.setMission(value); 
      }
    }
  }
  
  public void defaultValues(){
    selectMission(0);
    selectWeapon(weaponNuke,0);
    selectWeapon(weaponTroopers,1);
    cp5.getController("Menu_Action_Speed").setValue(defaultActionSpeed);
    gameSequence.setSpeedName("Menu_Action_Speed"); // not required : for some reason, the button will do it
    cp5.getController("Menu_AmmoRestock").setValue(defaultAmmoRestock);
    world.weaponStock.setRestockName("Menu_AmmoRestock");
    cp5.getController("Menu_AmmoInit").setValue(defaultAmmoStart);
    world.weaponStock.setAmmoInitName("Menu_AmmoInit");
    controlP5.Toggle toggle = (controlP5.Toggle) (cp5.getController("Menu_Action_Ammo"));
    toggle.setState(false);
    
  }
  
  public void previousValues(){
    updateListButtons(s_Menu_Mission,mission.missionDefinitions.size(),mission.missionRef);
    updateWeaponButtons((WeaponsList) world.weaponStock.weaponsList.get(0),
                        (WeaponsList) world.weaponStock.weaponsList.get(1));
    cp5.getController("Menu_Action_Speed").setValue(gameSequence.speed);
    gameSequence.setSpeedName("Menu_Action_Speed"); // not required : for some reason, the button will do it
    cp5.getController("Menu_AmmoRestock").setValue(world.weaponStock.restockRate);
    world.weaponStock.setRestockName("Menu_AmmoRestock");
    cp5.getController("Menu_AmmoInit").setValue(world.weaponStock.ammoInit);
    world.weaponStock.setAmmoInitName("Menu_AmmoInit");
    controlP5.Toggle toggle = (controlP5.Toggle) (cp5.getController("Menu_Action_Ammo"));
    toggle.setState(!limitedAmmo);
  }
  
  public void getValues(){
    gameSequence.previousStartMenu=menuAction;
    gameSequence.speed = PApplet.parseInt (cp5.getController("Menu_Action_Speed").getValue());
    controlP5.Toggle toggle = (controlP5.Toggle) (cp5.getController("Menu_Action_Ammo"));
    limitedAmmo = !toggle.getState();
 
    world.weaponStock.setRestock(PApplet.parseInt (cp5.getController("Menu_AmmoRestock").getValue()));
    world.weaponStock.setAmmo(PApplet.parseInt (cp5.getController("Menu_AmmoInit").getValue()));
  }
}

class MenuStrategy extends Menu{
                     
  public void createMenu(){
    int n = 0;
    int x = width/2-menuButtonWidth/2;
    int y = (int) (menuTitleBorder*2+menuTitleBorder*1.5f);
    int h = menuButtonHeight+1;
    
    if (gameSequence.previousStartMenu != menuStrategy){
      CustomList wList = (CustomList) world.weaponStock.weaponsList.get(1);
      wList.setSize(1);
    }
    
    cp5.addButton("Menu_Strategy_Start")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Strategy_Start)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy_Start",desc_Menu_Strategy_Start));   
    
    n+=1;
    n+=1;
    MultiList l = cp5.addMultiList("Menu_Strategy_Mission",x,y+n*h,menuButtonWidth,menuButtonHeight);
    buttonInfo.add(new ButtonInfo("Menu_Strategy_Mission",desc_Menu_Mission));
    customizeMission(l,s_Menu_Mission,desc_Menu_Mission);
    
    n+=1;
    l = cp5.addMultiList("Menu_Strategy_Weapons",x,y+n*h,menuButtonWidth,menuButtonHeight);
    buttonInfo.add(new ButtonInfo("Menu_Strategy_Weapons",desc_Menu_Strategy_Weapons));
    customize(l,s_Menu_Strategy_Weapons,desc_Menu_Strategy_Weapons);
    
    int temp = world.weaponStock.ammoInit;
    n+=1;
    cp5.addSlider("Menu_AmmoInit")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,6)
       .setNumberOfTickMarks(7)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_AmmoInit)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_AmmoInit",desc_Menu_AmmoInit,desc_Menu_Action_AmmoOff));
    
    temp = world.weaponStock.restockRate;
    n+=1;
    cp5.addSlider("Menu_AmmoRestock")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,6)
       .setNumberOfTickMarks(7)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_AmmoRestock)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_AmmoRestock",desc_Menu_AmmoRestock,desc_Menu_Action_AmmoOff));
    
    temp = gameSequence.turnLengthSetting;
    n+=1;
    n+=1;
    cp5.addSlider("Menu_Strategy_TurnLength")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,gameSequence.listTurnLength.length-1)
       .setNumberOfTickMarks(gameSequence.listTurnLength.length)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_Strategy_TurnLength)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy_TurnLength",desc_Menu_Strategy_TurnLength));
    
    temp = gameSequence.initialTurnLengthSetting;
    n+=1;
    cp5.addSlider("Menu_Strategy_InitialTurnLength")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setDecimalPrecision(0)
       .setRange(0,gameSequence.listInitialTurnLengthRatio.length-1)
       .setNumberOfTickMarks(gameSequence.listInitialTurnLengthRatio.length)
       .showTickMarks(false)
       .setValue(temp)
       .setCaptionLabel(s_Menu_Strategy_InitialTurnLength)
       .getCaptionLabel().align(ControlP5.LEFT_OUTSIDE, ControlP5.CENTER).setPaddingX(4)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy_InitialTurnLength",desc_Menu_Strategy_InitialTurnLength));
    
    n+=1;  
    n+=1;  
    cp5.addButton("Menu_Strategy_Default")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Strategy_Default)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy_Default",desc_Menu_Strategy_Default));
    
    n+=1;
    cp5.addButton("Menu_Strategy_Back")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Strategy_Back)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Strategy_Back",desc_Menu_Strategy_Back));
    
    if (gameSequence.previousStartMenu == menuStrategy) previousValues();
    else defaultValues();
  }
    
  public void customize(MultiList l,String name,String description){
    
    int buttonNumber=0;
    
    l.closeDelay=5; // delay after which must close the list (in ms ?) when mouse stopped overing
    
    MultiListButton b;
    b = l.add(name,-1);
    buttonInfo.add(new ButtonInfo(name,description));
    
    for (int i=0; i<world.weaponStock.weaponsList.size();i++){
      // Button for the name of the list
      WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(i);
      MultiListButton c = b.add(name+" "+i,i+1);
      buttonInfo.add(new ButtonInfo(name+" "+i,wList.description));
      c.setLabel(wList.name);
      c.setWidth(PApplet.parseInt (menuButtonWidth*1.3f));
      
      // Add weapons connected to that list
      // weaponStock.loadWeapons(i);
      if (wList.custm) wList = (WeaponsList) world.weaponStock.weaponsList.get(0);
      wList.menuListWeapons(name+" "+i,c,buttonInfo,(i+1)*listRefMultiplier);
      
      updateWeaponsNames(name+" "+i,(WeaponsList) world.weaponStock.weaponsList.get(i));
    }
  }
  
  // Crap programming : cut and pasted from menuAction
  public void customizeMission(MultiList l,String name,String description){
    
    int buttonNumber=0;
    
    l.closeDelay=5; // delay after which must close the list (in ms ?) when mouse stopped overing
    
    MultiListButton b;
    b = l.add(name,-1);
    buttonInfo.add(new ButtonInfo(name,description));
    
    for (int i=0; i<mission.missionDefinitions.size();i++){
      // Button for the name of the list
      MissionDefinition mDef = (MissionDefinition) mission.missionDefinitions.get(i);
      MultiListButton c = b.add(name+" "+i,i);
      buttonInfo.add(new ButtonInfo(name+" "+i,mDef.description));
      c.setLabel(mDef.name);
      c.setWidth(PApplet.parseInt (menuButtonWidth*1.3f));
    }
  }
  
  // crap Programming. in Action Strategy
  // Selects the Mission
  public void selectMission(int value){
    String name;
    ButtonInfo bInfo;
    
    name = s_Menu_Mission;
    bInfo = findItem(name+" "+value);
    
    if (bInfo!=null ){
      if (bInfo.data == Button_Data_Normal && value>=0){
        
        updateListButtons(name,mission.missionDefinitions.size(),value);
        mission.setMission(value); 
      }
    }
  }
  
  /*void updateListButtons(int listRef,String name){
    ButtonInfo bInfo;

    for (int i =0; i<world.weaponStock.weaponsList.size();i++){
      //bInfo = findItem(name+" "+i);
     if (i == listRef) buttonStatus(name+" "+i,Button_Data_Selected);
     else buttonStatus(name+" "+i,Button_Data_Normal);    
    }
  }*/
  
  public void updateWeaponsNames(String name,WeaponsList wList){
    WeaponManager wManager = new WeaponManager();
    
    if (wList.custm){
      for (int i=0;i<weaponNumber;i++){
        Weapon weapon = (Weapon) wManager.loadOneWeapon(i);
        cp5.getController(name+" "+i).setLabel(weapon.name);
      }
      for (int i=0; i<wList.length();i++){
        int j=0;
        while (j!=wList.get(i) && j<weaponNumber) j++;
        if (j==wList.get(i)){ 
          Weapon weapon = (Weapon) wManager.loadOneWeapon(j);
          cp5.getController(name+" "+j).setLabel((i+1)+" "+weapon.name);
        }
      }
    }
    else {  
      for (int i=0;i<wList.length();i++){
        Weapon weapon = (Weapon) wManager.loadOneWeapon(wList.get(i));
        if (i<=8) cp5.getController(name+" "+i).setLabel((i+1)+" "+weapon.name);
        else if (i==9) cp5.getController(name+" "+i).setLabel(0+" "+weapon.name);
        else cp5.getController(name+" "+i).setLabel(weapon.name);
      }
    }
  }
  
  public void updateWeaponButtons(String name, WeaponsList wList){
    ButtonInfo bInfo;
    WeaponManager wManager = new WeaponManager();
    
    if (!wList.custm){
      for (int i=0; i<wList.length(); i++){
        Weapon weapon = (Weapon) wManager.loadOneWeapon(wList.get(i));
        if (!weapon.checkRequiredWeapon(wList)) buttonStatus(name+" "+i,Button_Data_Unavailable);
        else buttonStatus(name+" "+i,Button_Data_Selected);
      }
    }
    else {
      WeaponsList wListStock = (WeaponsList) world.weaponStock.weaponsList.get(0);
      for (int i=0; i<wListStock.length(); i++){
        boolean found=false;
        for (int j=0; j<wList.length(); j++){
          //bInfo = findItem(name+" "+i); 
          if (wList.get(j) == wListStock.get(i)) {
            buttonStatus(name+" "+i,Button_Data_Selected);
            found = true;
            break;
          }
          if (!found){
            Weapon weapon = (Weapon) wManager.loadOneWeapon(wListStock.get(i));
            if (!weapon.checkRequiredWeapon(wList)) buttonStatus(name+" "+i,Button_Data_Unavailable);
            else buttonStatus(name+" "+i,Button_Data_Normal);
          }
        }
      }
      updateWeaponsNames(name,wList);
    }
  }
  
  public void updateAllWeaponButton(){
    for (int i=0;i<world.weaponStock.weaponsList.size();i++){
      WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(i);
      updateWeaponButtons(s_Menu_Strategy_Weapons+" "+i,wList);
    }
  }
  
  public void selectList(int listRef){
    String name=s_Menu_Strategy_Weapons;
    WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(listRef);
    if (wList.rand){
      ButtonInfo bInfo = findItem(name+" "+listRef);
      if (bInfo.data==Button_Data_Selected){
        wList.shuffle();
        updateWeaponsNames(name+" "+listRef,wList);
        updateWeaponButtons(name+" "+listRef,wList);
        world.weaponStock.loadWeapons(listRef);
        return;
      }  
    }
    updateListButtons(name,world.weaponStock.weaponsList.size(),listRef);
    updateWeaponButtons(name+" "+listRef,wList);
    world.weaponStock.loadWeapons(listRef);
  }
  
  public void clickedWeapon(int itemRef,int listRef){
    String name=s_Menu_Strategy_Weapons+" "+listRef;
    ButtonInfo bInfo;
    WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(listRef);
    
    if (wList.custm && itemRef>=0){
      CustomList cList = (CustomList) wList;
      bInfo = findItem(name+" "+itemRef);
      WeaponsList referenceList = (WeaponsList) world.weaponStock.weaponsList.get(0);
      int weaponRef = referenceList.get(itemRef);
   
      if (bInfo!=null ){
        if (bInfo.data == Button_Data_Normal){
          cList.addWeapon(weaponRef);
          updateWeaponButtons(name,wList);
        }
        else if (bInfo.data == Button_Data_Selected){
          cList.removeWeapon(weaponRef);
          updateWeaponButtons(name,wList);
        }
      }
    }
  }
  
  // Selects the weapons at the given position
  public void selectWeapon(int value,int position,int listRef){
    /*String name=s_Menu_Strategy_Weapons+" "+listRef;
    ButtonInfo bInfo;
    WeaponsList wList = (WeaponsList) weaponsStock.weaponsList.get(listRef);
    
    if (wList.custm){
      CustomList cList = (CustomList) wList;
      bInfo = findItem(name+" "+value);
    
      if (bInfo!=null ){
        if (bInfo.data == Button_Data_Normal && value>=0){
          weaponStock.setCustomListValue(position,value); 
          updateWeaponButtons(name+" "+value,(WeaponsList) weaponStock.weaponsList.get(1));
        }
      }
    }*/
  }
  
  
  
  public void defaultValues(){
    selectMission(0);
    selectList(5);
    updateAllWeaponButton();
    cp5.getController("Menu_Strategy_TurnLength").setValue(defaultTurnLength);
    gameSequence.setTurnLengthName("Menu_Strategy_TurnLength");
    cp5.getController("Menu_Strategy_InitialTurnLength").setValue(defaultInitialTurnLengthRatio);
    gameSequence.setInitialTurnLengthName("Menu_Strategy_InitialTurnLength");
    cp5.getController("Menu_AmmoRestock").setValue(defaultAmmoRestock);
    world.weaponStock.setRestockName("Menu_AmmoRestock");
    cp5.getController("Menu_AmmoInit").setValue(defaultAmmoStart);
    world.weaponStock.setAmmoInitName("Menu_AmmoInit");     
  }
  
  public void previousValues(){
    updateListButtons(s_Menu_Mission,mission.missionDefinitions.size(),mission.missionRef);
    selectList(world.weaponStock.listCycle);
    updateAllWeaponButton();
    cp5.getController("Menu_Strategy_TurnLength").setValue(gameSequence.turnLengthSetting);
    gameSequence.setTurnLengthName("Menu_Strategy_TurnLength");
    cp5.getController("Menu_Strategy_InitialTurnLength").setValue(gameSequence.initialTurnLengthSetting);
    gameSequence.setInitialTurnLengthName("Menu_Strategy_InitialTurnLength");
    cp5.getController("Menu_AmmoRestock").setValue(world.weaponStock.restockRate);
    world.weaponStock.setRestockName("Menu_AmmoRestock");
    cp5.getController("Menu_AmmoInit").setValue(world.weaponStock.ammoInit);
    world.weaponStock.setAmmoInitName("Menu_AmmoInit");
  }
  
  public void getValues(){
    gameSequence.previousStartMenu=menuStrategy;
    gameSequence.setTurnLength(PApplet.parseInt (cp5.getController("Menu_Strategy_TurnLength").getValue()));
    gameSequence.setInitialTurnLength(PApplet.parseInt (cp5.getController("Menu_Strategy_InitialTurnLength").getValue()));
    world.weaponStock.setRestock(PApplet.parseInt (cp5.getController("Menu_AmmoRestock").getValue()));
    world.weaponStock.setAmmo(PApplet.parseInt (cp5.getController("Menu_AmmoInit").getValue()));
  }
  
}

class MenuOptions extends Menu{
  /*String item[][] = {{"Menu_Option_Default",desc_Menu_Option_Default},
                     {"Menu_Option_Back",desc_Menu_Option_Back}};*/
                     
  public void createMenu(){
    int n = 0;
    int x = width/2-menuButtonWidth/2;
    int y = (int) (menuTitleBorder*2+menuTitleBorder*1.5f);
    int h = menuButtonHeight+1;
    
    cp5.addButton("Menu_Option_Default")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Option_Default)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Option_Default",desc_Menu_Option_Default));
    
    n+=1;
    cp5.addButton("Menu_Option_Back")
       .setPosition(x,y+n*h)
       .setSize(menuButtonWidth,menuButtonHeight)
       .setCaptionLabel(s_Menu_Option_Back)
       ;
    buttonInfo.add(new ButtonInfo("Menu_Option_Back",desc_Menu_Option_Back));
  }
}
public void mousePressed(){ 
  int mx=mouseX;
  int my=mouseY;
  
  if (gameSequence.screenRef == screenGame ){
    if (!gameSequence.finished){
      if (gameSequence.turnBased){ // If playing a strategy game
        if (mouseButton == LEFT){
          // Checks that we are clicking inside the playing area
          if (gameSequence.freeze & gameScreen.playArea.inside(mx,my)) world.weaponStock.fire(mx,my,limitedAmmo);
          // If clicks out of the playArea, unpauses the game, so there is not need to press space
          //if (!gameScreen.playArea.inside(mx,my)) gameSequence.freeze=false;
        }
        else if (mouseButton == RIGHT) world.weaponStock.cycle();
      }
      else { // if playing an action game
        if (!gameSequence.freeze && gameScreen.playArea.inside(mx,my)){
          if (mouseButton == LEFT) world.weaponStock.fireLeft(mx,my,limitedAmmo);
          else if (mouseButton == RIGHT) world.weaponStock.fireRight(mx,my,limitedAmmo);
        }
      }
    }
    else {
      InfoPanel  panel = gameScreen.infoPanel;
      if (panel.type==infoPanelRef_TransitionWin || panel.type==infoPanelRef_TransitionLoose){
        if (panel.infoArea.checkInside(mx,my)) panel.changeInfoPanel(((InfoAreaTransition)panel.infoArea).nextType );
      }
    }
  }
}


// Gets input from the keyboard
public void keyPressed() {

  if(key == ESC) key = 'm'; // Otherwise will leave when pressing esc
  if(key == 'm') {
    if ( gameSequence.screenRef == screenGame ){
      gameScreen.erase();
      gameSequence.initStartScreen();
      gameSequence.screenRef=screenStart;
    }
  } 
  if(key == '+' && num < 8000) { num += 100; key='z'; }
  if(key == '-' && num > 4000) { num -= 100; key='z'; }
  if(key == 'z' || key == 'Z'){
    gameSequence.freeze=true;
    gameSequence.initGame();
    gameSequence.screenRef=screenGame;
  }
  if(key == 't' || key == 'T') {
    if (!gameSequence.turnBased) {
      gameSequence.turnBased = true;
      limitedAmmo = true;
      gameSequence.freeze = false; // Otherwise, too easy to cheat...
    }
  }
  if(key == 'r' || key == 'R'){
    if (gameSequence.turnBased){
      gameSequence.turnBased = false;
      limitedAmmo = false;
      gameSequence.freeze = false;
    }
  } 
  if(key == ' '){
    if (gameSequence.freeze) gameSequence.freeze = false;
    else if (!gameSequence.turnBased || gameSequence.finished) gameSequence.freeze=true;
  }
  if(key == 'a' || key == 'A'){
    if (limitedAmmo) limitedAmmo = false;
    else limitedAmmo = true;
  }
  if(key == 'f' || key == 'F'){
    if (fps) fps =false;
    else fps = true;
  }
  if(key == 'b' || key == 'B') world.weaponStock.cycleBack();
  if(key == 'n' || key == 'N') world.weaponStock.cycle();
  //if(key == 'g' || key == 'G') weaponStock.cycleListBack();
  //if(key == 'h' || key == 'H') weaponStock.cycleList();
  if(key == 'd' || key == 'D') world.displayBeingData.cycleMode();
  if(key == 's' || key == 'S') world.displayBeingData.cycleSecond();
  
  if(key == 't' || key == 'T') println(gameSequence.speed);
  if (key >= '0' && key <='9') {
    int value = Integer.parseInt(str(key));
    world.weaponStock.setWeapon(value);
  };
  
}

// Intercepts clicking on the different button (some might not have been created yet)

public void Menu_Action(){
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuAction();
}

public void Menu_Strategy(){
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuStrategy();
}

public void Menu_Options(){
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuOptions();
}

public void Menu_Action_Back(){
  startScreen.menu.getValues();
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuIntro();
}

public void Menu_Action_Start(){
  startScreen.menu.getValues();
  
  gameSequence.turnBased = false;
  gameSequence.freeze = false;
  world.weaponStock.loadWeapons(1); // load the custom list for weapons
      
  startScreen.menu.eraseMenu();
  gameSequence.initGame();
  gameSequence.screenRef=screenGame;
}

public void Menu_Action_Speed(){
  gameSequence.speed = (int) cp5.controller("Menu_Action_Speed").getValue();
  gameSequence.setSpeedName("Menu_Action_Speed");
}

public void Menu_AmmoRestock(){
  world.weaponStock.setRestock((int) cp5.controller("Menu_AmmoRestock").getValue());
  world.weaponStock.setRestockName("Menu_AmmoRestock");
}

public void Menu_AmmoInit(){
  world.weaponStock.setAmmo((int) cp5.controller("Menu_AmmoInit").getValue());
  world.weaponStock.setAmmoInitName("Menu_AmmoInit");
}

public void Menu_Action_Left(ControlEvent event){
  
  MenuAction menuAction = (MenuAction) startScreen.menu;
  menuAction.selectWeapon((int)event.value(),0);
}

public void Menu_Action_Right(ControlEvent event){
  MenuAction menuAction = (MenuAction) startScreen.menu;
  menuAction.selectWeapon((int)event.value(),1);
}

public void Menu_Action_Mission(ControlEvent event){
  
  MenuAction menuAction = (MenuAction) startScreen.menu;
  menuAction.selectMission((int)event.value());
}

public void Menu_Action_Ammo(boolean unlimited){
  if (unlimited){
    cp5.getController("Menu_Action_Ammo").setCaptionLabel(s_Menu_Action_Ammo_Unlimited);
    startScreen.menu.buttonStatus("Menu_AmmoInit",Button_Data_Unavailable);
    startScreen.menu.buttonStatus("Menu_AmmoRestock",Button_Data_Unavailable);
  }
  else {
    cp5.getController("Menu_Action_Ammo").setCaptionLabel(s_Menu_Action_Ammo_Limited);
    startScreen.menu.buttonStatus("Menu_AmmoInit",Button_Data_Normal);
    startScreen.menu.buttonStatus("Menu_AmmoRestock",Button_Data_Normal);
  }
}

public void Menu_Action_Default(){
  startScreen.menu.defaultValues();
}

public void Menu_Strategy_Weapons(ControlEvent event){
  int value = (int)event.value();
  int listRef,weaponRef;
  MenuStrategy menuStrategy = (MenuStrategy) startScreen.menu;
  
  if (value>=0){
    if (value < listRefMultiplier){
      menuStrategy.selectList(value-1);
    }
    else {
      listRef=value/listRefMultiplier-1;
      weaponRef=value%listRefMultiplier;
      if (listRef>=0 && listRef<world.weaponStock.weaponsList.size()){
        WeaponsList wList = (WeaponsList) world.weaponStock.weaponsList.get(listRef);
        if (wList.custm) {
          menuStrategy.clickedWeapon(weaponRef,listRef);
        }
        menuStrategy.selectList(listRef);
      }
    }
  }
}

public void Menu_Strategy_Mission(ControlEvent event){
  
  MenuStrategy menuStrategy = (MenuStrategy) startScreen.menu;
  menuStrategy.selectMission((int)event.value());
}

public void Menu_Strategy_TurnLength(){
  gameSequence.setTurnLength(PApplet.parseInt (cp5.getController("Menu_Strategy_TurnLength").getValue()));
  gameSequence.setTurnLengthName("Menu_Strategy_TurnLength");
  gameSequence.setInitialTurnLengthName("Menu_Strategy_InitialTurnLength");
}

public void Menu_Strategy_InitialTurnLength(){
  gameSequence.setInitialTurnLength(PApplet.parseInt (cp5.getController("Menu_Strategy_InitialTurnLength").getValue()));
  gameSequence.setInitialTurnLengthName("Menu_Strategy_InitialTurnLength");
}

public void Menu_Strategy_Default(){
  startScreen.menu.defaultValues();
}

public void Menu_Strategy_Back(){
  startScreen.menu.getValues();
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuIntro();
}

public void Menu_Strategy_Start(){
  startScreen.menu.getValues();
  startScreen.menu.eraseMenu();
  
  gameSequence.turnBased = true;
  limitedAmmo = true;
  gameSequence.freeze = false;
  gameSequence.speed=defaultSpeed;
  gameSequence.setSpeedRate();
  gameSequence.initGame();
  
  gameSequence.screenRef=screenGame;
}

public void Menu_Option_Back(){
  startScreen.menu.eraseMenu();
  startScreen.menu = new MenuIntro();
}

public void Menu_PanelStrategy_NextTurn(){
  if (gameSequence.freeze) {
    gameSequence.freeze = false;
    gameScreen.textPanel.menu.buttonStatus("Menu_PanelStrategy_NextTurn",Button_Data_Unavailable);
  }
  //else if (!gameSequence.turnBased) gameSequence.freeze=true;
}

public void Menu_PanelStrategy_PreviousWeapon(){
  world.weaponStock.cycleBack();
}

public void Menu_PanelStrategy_NextWeapon(){
  world.weaponStock.cycle();
}

public void Menu_PanelAction_Pause(){
  if (gameSequence.freeze) {
    gameSequence.freeze = false;
    cp5.getController("Menu_PanelAction_Pause").setCaptionLabel(s_Menu_PanelAction_Pause);
  }
  else if (!gameSequence.turnBased) {
    gameSequence.freeze=true;
    cp5.getController("Menu_PanelAction_Pause").setCaptionLabel(s_Menu_PanelAction_Continue);
  }
}

public void Menu_PanelInfo_Stat(){
  if (gameScreen.infoPanel.type != infoPanelRef_Statistics) 
    gameScreen.infoPanel.changeInfoPanel(infoPanelRef_Statistics); 
}

public void Menu_PanelInfo_Mission(){
  if (gameScreen.infoPanel.type != infoPanelRef_Mission) {
    gameScreen.infoPanel.changeInfoPanel(infoPanelRef_Mission);
  } 
}

public void Menu_PanelInfo_Score(){
  if (gameScreen.infoPanel.type != infoPanelRef_Score) {
    gameScreen.infoPanel.changeInfoPanel(infoPanelRef_Score);
  } 
}

public void Menu_PanelFinished_BackMenu(){
  gameScreen.erase();
  gameSequence.initStartScreen();
  gameSequence.screenRef=screenStart;
}

public void Menu_PanelFinished_Restart(){
  gameSequence.freeze=true;
  gameSequence.initGame();
  gameSequence.screenRef=screenGame;
}

public void Menu_PanelFinished_KeepGoing(){
  if (gameSequence.freeze) {
    gameSequence.freeze = false;
    cp5.getController("Menu_PanelFinished_KeepGoing").setCaptionLabel(s_Menu_PanelFinished_Pause);
  }
  else {
    gameSequence.freeze=true;
    cp5.getController("Menu_PanelFinished_KeepGoing").setCaptionLabel(s_Menu_PanelFinished_KeepGoing);
  }
}
int deathAlive = -1;
int deathAge = -2;
int weaponNuke =0;
int weaponTroopers=1;
int weaponResupply=2;
int weaponSoldierAttractor=3;
int weaponSoldierRepulsor=4;
int weaponQuickBomb=5;
int weaponBomb=6;
int weaponSlowBomb=7;
int weaponMine=8;
int weaponBigMine=9;
int weaponMineField=10;
int weaponDirt=11;
int weaponConcrete=12;
int weaponHumanAttractor=13;
int weaponHumanRepulsor=14;
int weaponZombieAttractor=15;
int weaponZombieRepulsor=16;
int weaponInoculate=17;
int weaponBioweapon=18;
int weaponHeadquarter=19;

int weaponNumber = 20; // important for random !!

int defaultAmmoRestock = 3;
int defaultAmmoStart = 2;

// Returns true if the being is in the radius of the object
public boolean checkBeingInArea(Being being,int x, int y,int radius) {
  int dx=being.xpos-x;
  int dy=being.ypos-y;
  int diff=(dx*dx)+(dy*dy);
  return diff<radius*radius;
}

// Class containing a list of weapons. It does only load the reference of each weapon, 
// but not the class (thus does not know the name/ description/ caracteristics of the weapons)
// Kinf of an array with a name
class WeaponsList{
  String name;
  String description;
  int weapons[]={};
  boolean rand=false;
  boolean custm=false;
  
  WeaponsList(){}
  
  WeaponsList(String _name,String _description, int _weapons[]){
    name = _name;
    description = _description;
    weapons = _weapons;
  }
  
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }
  
  public void menuListWeapons(String name, MultiListButton b, ArrayList buttonInfo,int firstValue){
    WeaponManager wManager = new WeaponManager();
    int j=0;
    
    // Prints weapons in columns of 8 (menuDisplayWeaponNumber)
    while (length()-j > menuDisplayWeaponNumber){
      // If there are more than 8 create a button "more..." to display next column
      MultiListButton c = b.add(name+"_More_"+j,-1);
      buttonInfo.add(new ButtonInfo(name+"_More_"+j,desc_Menu_Action_More));
      c.setLabel(s_Menu_Action_More);
      c.setColorBackground(color(128,0,0));
      c.setWidth(PApplet.parseInt (menuButtonWidth*1.3f));
      for (int i = j; i<j+menuDisplayWeaponNumber;i++){
        Weapon weapon = (Weapon) wManager.loadOneWeapon(get(i));
        b.add(name+" "+i,i+firstValue)
          .setLabel(weapon.name)
          .setWidth(PApplet.parseInt (menuButtonWidth*1.3f))
          ;
        buttonInfo.add(new ButtonInfo(name+" "+i,weapon.description(),weapon.descriptionUnavailable()));
      }
      j+=menuDisplayWeaponNumber;
      b=c;
    }
    // Displays the last column
    for (int i = j; i<length();i++){
      Weapon weapon = (Weapon) wManager.loadOneWeapon(get(i));
      b.add(name+" "+i,i+firstValue)
        .setLabel(weapon.name)
        .setWidth(PApplet.parseInt (menuButtonWidth*1.3f))
        ;
      buttonInfo.add(new ButtonInfo(name+" "+i,weapon.description(),weapon.descriptionUnavailable()));
    }
  }
  
  public void shuffle(){
  }
}

class CustomList extends WeaponsList {
  int weapons[] = {weaponNuke};
  
  CustomList(){
    name=s_List_Custom;
    description=desc_List_Custom;
    custm=true;
  }
  
  //set the size (n) of the costumList, and fills it with the n first weapons
  public void setSize(int _size){
    weapons = new int[_size];
    for (int i = 0; i<_size;i++) weapons[i]=i;
  }
  
    
  // adds the weapon at the end of the array, if that weapon hasn't been added yet.
  public boolean addWeapon(int weaponRef){
    int i;
    int _weapons[];
    boolean changed =false;
    
    // test if the weapon is already in the list
    for (i=0; i<weapons.length;i++){
      if (weapons[i]==weaponRef) break;
    }
    
    //The whole array has been scrolled down, so the weapon was not there already 
    if (i==weapons.length){
      _weapons = new int[i+1];
      int j;
      for (j=0;j<i;j++){
        _weapons[j]=weapons[j];
      }
      _weapons[j]=weaponRef;
      weapons = _weapons;
      changed = true;
    }
    return changed;
  }
  
  // Remove the weapon from the array, if that weapon can be found
  public boolean removeWeapon(int weaponRef){
    int i;
    int _weapons[];
    boolean changed =false;
    
    // test if the weapon is already in the list
    for (i=0; i<weapons.length;i++){
      if (weapons[i]==weaponRef) break;
    }
    
    //The weapon has been spotted in the array (array was not fully scrolled down) 
    if (i<weapons.length){
      _weapons = new int[weapons.length-1];
      int j;
      // Add every elements befor the one to be removed
      for (j=0;j<i;j++){
        _weapons[j]=weapons[j];
      }
      
      // Add every elements after the one to be removed
      for (j=i;j<weapons.length-1;j++){
        _weapons[j]=weapons[j+1];
      }
    
      weapons = _weapons;
      changed = true;
    }
    return changed;
  }
  
  // Change the reference of the weapon at a given position in the array
  // Returns the value of the weapon that has been changed, so the button can have its color reset
  public int setWeapon(int position, int weaponRef){
    int changed=-1;
    
    if (position<=weapons.length){
      changed=weapons[position];
      weapons[position]=weaponRef;
   }
    
    return changed;
  }
  
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }  
}

// Lists every weapons
class FullHolster extends WeaponsList {
  int weapons[] = new int[weaponNumber];
  
  FullHolster(){
    name = s_List_FullArsenal;
    description = desc_List_FullArsenal;
    for (int i=0; i<weaponNumber;i++) weapons[i]=i;
  }
  
  // If I don't do that, it'll read the array definer in super class
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }      
}

class BioWar extends WeaponsList {
  int weapons[]= {weaponNuke,
                  weaponHumanAttractor,
                  weaponHumanRepulsor,
                  weaponZombieAttractor,
                  weaponZombieRepulsor,
                  weaponInoculate,
                  weaponBioweapon};
  
  BioWar(){
    name = s_List_Biowar;
    description = desc_List_Biowar;
  }
  
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }      
}

class Army extends WeaponsList {
  int weapons[]= {weaponNuke,
                  weaponTroopers,
                  weaponResupply,
                  weaponSoldierAttractor,
                  weaponSoldierRepulsor,
                  weaponHeadquarter,
                  weaponHumanAttractor,
                  weaponHumanRepulsor};
  
  Army(){
    name=s_List_Army;
    description=desc_List_Army;
  }
  
  // If I don't do that, it'll read the array definer in super class
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }      
}

class Demolition extends WeaponsList {
  int weapons[]= {weaponQuickBomb,
                weaponBomb,
                weaponSlowBomb,
                weaponMine,
                weaponBigMine,
                weaponMineField,
                weaponDirt,
                weaponConcrete};
  
  Demolition(){
    name=s_List_Demolition;
    description=desc_List_Demolition;
  }
  
  // If I don't do that, it'll read the array definer in super class
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }      
}

class RandomList extends WeaponsList{
  int listSize,listFirst;
  int[] weapons;
  
  RandomList(){
    listSize=5;
    listFirst=1;
    weapons = new int[listSize];
    rand = true;
    name=s_List_Nuke4;
    description=desc_List_Nuke4; 
    weapons[0]=weaponNuke;
    shuffle();
  }
  
  public void shuffle(){
    WeaponManager wManager = new WeaponManager();
    
    for (int i=1;i<listSize;i++){
      weapons[i]=-1;
    }
    
    //println(wManager.weaponNameFromRef(2));
    
    for (int i=listFirst;i<listSize;i++){
      int temp = (int) random(weaponNumber);
      boolean ok =false;
      while (!ok){
        int j=0;
        for (j=0;j<i;j++){ // test if weapon hasn't been added yet
          if (temp == weapons[j]) break;
        }
        if (j==i) { // Ok, the weapon hasn't been added yet
          Weapon weapon = (Weapon) wManager.loadOneWeapon(temp);
          if (weapon.checkRequiredWeapon(this)){
            ok=true;
            weapons[i]=temp;
          }
        }
        if (!ok) temp = (int) random(weaponNumber);
      }
    }
  }
  
  // If I don't do that, it'll read the array definer in super class
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }
}

class FullRandomList extends RandomList{
  
  
  FullRandomList(){
    listSize=6;
    listFirst=0;
    weapons=new int[listSize];
    rand = true;
    name=s_List_Random6;
    description=desc_List_Random6;
    shuffle();
    
  }
   
  // If I don't do that, it'll read the array definer in super class
  public int length(){
    return weapons.length;
  }
  
  public int get(int i){
    return weapons[i];
  }
}

// Class used to create weapons
class WeaponManager{
  World wld;
  
  WeaponManager(){
    wld=null;
  }
  
  WeaponManager(World _wld){
    wld=_wld;
  }
  
    // returns the name of a weapon from its reference
  public String weaponNameFromRef(int weaponRef){
    Weapon weapon =loadOneWeapon(weaponRef);
    if (weapon!=null) return weapon.name;
    
    return s_Weapon_Unknown;
  }
  
  // Returns an instance of a weapon, according to the reference
  public Weapon loadOneWeapon(int weaponRef){
    Weapon weapon=null;  
    if (weaponRef==weaponNuke) weapon = new Weapon(wld);
    else if (weaponRef==weaponTroopers) weapon = new Troopers(wld);
    else if (weaponRef==weaponResupply) weapon = new Resupply(wld);
    else if (weaponRef==weaponSoldierAttractor) weapon = new DropSoldierAttractor(wld);
    else if (weaponRef==weaponSoldierRepulsor) weapon = new DropSoldierRepulsor(wld);
    else if (weaponRef==weaponQuickBomb) weapon = new DropQuickBomb(wld);
    else if (weaponRef==weaponBomb) weapon = new DropDevice(wld);
    else if (weaponRef==weaponSlowBomb) weapon = new DropSlowBomb(wld);
    else if (weaponRef==weaponMine) weapon = new DropMine(wld);
    else if (weaponRef==weaponBigMine) weapon = new DropBigMine(wld);
    else if (weaponRef==weaponMineField) weapon = new DropMineField(wld);
    else if (weaponRef==weaponDirt) weapon = new Dirt(wld);
    else if (weaponRef==weaponConcrete) weapon = new Concrete(wld);
    else if (weaponRef==weaponHumanAttractor) weapon = new DropHumanAttractor(wld);
    else if (weaponRef==weaponHumanRepulsor) weapon = new DropHumanRepulsor(wld);
    else if (weaponRef==weaponZombieAttractor) weapon = new DropZombieAttractor(wld);
    else if (weaponRef==weaponZombieRepulsor) weapon = new DropZombieRepulsor(wld);
    else if (weaponRef==weaponInoculate) weapon = new Inoculate(wld);
    else if (weaponRef==weaponBioweapon) weapon = new Bioweapon(wld);
    else if (weaponRef==weaponHeadquarter) weapon = new DropSoldierHQ(wld);

    return weapon;
  }
}

// This object contains every weapons that the player can use
// It holds instance of every existing weaponsList, 
// and an instance of every weapon from the loaded weapons list
class WeaponStock{
  int ammo,ammoInit;
  int ammoRestock,restockRate;
  int ammoCycle;
  int weaponCycle;
  int listCycle;
  ArrayList weaponsList;
  ArrayList weapons; // list of Weapons
  String listName;
  int listRestock[]={0,150,60,35,15,7,3};
  String listRestockName[]={s_AmmoRestock_None,s_AmmoRestock_VerySlow,s_AmmoRestock_Slow,s_AmmoRestock_Normal,
                            s_AmmoRestock_Quick,s_AmmoRestock_VeryQuick,s_AmmoRestock_ExtremellyQuick};
  int listAmmoInit[]={0,5,10,20,50,100,500};
  World wld;
  
  
  WeaponStock(World _wld){
    ammoInit = defaultAmmoStart;
    ammo = defaultAmmoStart;
    restockRate = defaultAmmoRestock;
    setRestock(restockRate);
    weaponCycle=0;
    listCycle=0;
    loadWeaponsList();
    WeaponsList wList = (WeaponsList) weaponsList.get(listCycle);
    loadWeapons(wList);
    listName=wList.name;
    wld=_wld;
  }
  
  // reninits when startgina new game
  public void reinit(){
    setAmmo(ammoInit);
    weaponCycle=0;
  }
  
  public void setRestock(int restock){
    restockRate=restock;
    if (restockRate>=0 && restockRate<listRestock.length) ammoRestock = listRestock[restockRate];
    else {
      restockRate=defaultAmmoRestock;
      ammoRestock = listRestock[restockRate];
    }
  }
  
  public void setRestockName(String buttonName){
    cp5.controller(buttonName).setValueLabel(listRestockName[restockRate]);
  }
  
  public void setAmmo(int _ammo){
    ammoInit=_ammo;
    if (ammoInit>=0 && ammoInit<listAmmoInit.length) ammo = listAmmoInit[ammoInit];
    else {
      ammoInit=defaultAmmoStart;
      ammo = listAmmoInit[ammoInit];
    }
  }
  
  public void setAmmoInitName(String buttonName){
    cp5.controller(buttonName).setValueLabel(str(ammo));
  }
  
  // Load an instance of every weapons list
  // !!Weapons lists only hold reference to existing weapons, but no instance
  private void loadWeaponsList(){
    weaponsList= new ArrayList();
    weaponsList.add(new FullHolster()); // It should be the first, to allow menu creation
    weaponsList.add(new CustomList()); // It should be the second, for "Action mode" use
    weaponsList.add(new BioWar());
    weaponsList.add(new Army());
    weaponsList.add(new Demolition());
    weaponsList.add(new RandomList());
    weaponsList.add(new FullRandomList());
  }
  
  
  // This is here that an instance of every weapon from the list is created
  private void loadWeapons(WeaponsList list){
    WeaponManager wManager = new WeaponManager(wld);
    
    weapons = new ArrayList();
    for (int i=0;i<list.length();i++){
      Weapon weapon = wManager.loadOneWeapon(list.get(i));
      if (weapon !=null) weapons.add(weapon);
    }
  }
  
  public void loadWeapons(int i){
    WeaponsList wList = (WeaponsList) weaponsList.get(i);
    if (wList != null) {
      loadWeapons(wList);
      listCycle = i;
      weaponCycle = 0;
    }
  }
  
  public void setWeapon(int value){
    if ((value-1)>=0 && (value-1)<weapons.size()) weaponCycle = value-1;
    else if (value==0 && weapons.size()>=9) weaponCycle = 9; 
  }
  
  public void cycle(){
    weaponCycle++;
    if (weaponCycle>=weapons.size()) weaponCycle=0;
  }
  
  public void cycleBack(){
    weaponCycle--;
    if (weaponCycle<0) weaponCycle=weapons.size()-1;
  }
  
  public void cycleList(){
    listCycle++;
    if (listCycle>=weaponsList.size()) listCycle=0;
    //WeaponsList wList = (WeaponsList) weaponsList.get(listCycle);
    loadWeapons(listCycle);
    //weaponCycle=0;
    //loadWeapons(wList);
  }
  
  public void cycleListBack(){
    listCycle--;
    if (listCycle<0) listCycle=weaponsList.size()-1;
    //WeaponsList wList = (WeaponsList) weaponsList.get(listCycle);
    loadWeapons(listCycle);
    //weaponCycle=0;
    //loadWeapons(wList);
  }
  
  public int setCustomListValue(int position, int value){
    CustomList wList = (CustomList) weaponsList.get(1);
    return wList.setWeapon(position,value);
  }
  
  
  public void fire(int x, int y, boolean limited){
    if (weapons.size()>0){
      Weapon weapon = (Weapon) weapons.get(weaponCycle);
      
      if (limited){
        if (ammo>=weapon.cost){
          weapon.fire(x,y);
          ammo -= weapon.cost;
        }
      }
      else weapon.fire(x,y);
    }
  }
  
  public void fireLeft(int x, int y, boolean limited){
    weaponCycle=0;
    fire(x, y, limited);
  }
  
  public void fireRight(int x, int y, boolean limited){
    weaponCycle=1;
    fire(x, y, limited);
  }
  
  public String listName(){
    if (weaponsList.size() >0){
      WeaponsList wList = (WeaponsList) weaponsList.get(listCycle);
      return wList.name;
    }
    else return "No list";
  }
  
  public String name(){
    if (weapons.size() >0){
      Weapon weapon = (Weapon) weapons.get(weaponCycle);
      return weapon.name;
    }
    else return "No weapon";
  }
  
  public int cost(){
    if (weapons.size() >0){
      Weapon weapon = (Weapon) weapons.get(weaponCycle);
      return weapon.cost;
    }
    else return 0;
  }
  
  public void reStock(){
    if (ammoRestock!=0){
      if (ammoCycle>=ammoRestock){
        ammo++;
        ammoCycle=0;
      }
      ammoCycle++;
    }
  }
}

class Weapon {
  int cost =1;
  String name = s_Weapon_Nuke;
  String description = desc_Weapon_Nuke;
  int radiusFix =6;
  int radiusVariable =10;
  int type=weaponNuke;
  World wld;
  
  Weapon(World _wld){
    wld=_wld;
  }
  
  int requiredWeapon[] = {};
  
  public void drawEffect(int mx,int my,int radius){
    fill(empty);
    stroke(hit);
    for (int i=0;i<scale;i++){
      ellipse(mx,my,radius*2-i,radius*2-i);
    }    
  }
  
  public void countData(Being being){
    if (being instanceof Human)  wld.numKilledHumans++;
    if (being instanceof Zombie) wld.numKilledZombies++;
  }
  
  public void impact(Being being,int mx, int my){
     being.die(type);
  }
  
  public int fire(int mx, int my){
    // nuke characteristics
    int radius=((int)random(radiusVariable)+radiusFix)*scale;
    
    drawEffect(mx,my,radius);
    
    // killing the zombies and people in the radius
    // cycles through
    for(int i=0;i<num;i++){
      if(checkBeingInArea(wld.beings[i],mx,my,radius)){
        wld.beings[i].drawBeingOutside(); // because they have been erased when the effect had been drawn
        countData(wld.beings[i]);
        impact(wld.beings[i],mx,my);
       }
    }
    return cost;
  }
  
  // Sends description of the weapon
  public String description(){
    String s = description+"\nCost : "+cost;
    return s;
  }
  
  // Sends the description if that weapon can not be selected
  public String descriptionUnavailable(){
    int[] _requiredWeapon = getRequiredWeapon();
    WeaponManager wManager = new WeaponManager(wld);
    
    String s = desc_Weapon_Unavailable; 
    int i=0;
    for (i=0; i< _requiredWeapon.length-2;i++) s += "\""+wManager.weaponNameFromRef(_requiredWeapon[i])+"\", ";
    if (i==_requiredWeapon.length-2) {
      s += "\""+wManager.weaponNameFromRef(_requiredWeapon[i])+"\" "+desc_Weapon_Unavailable_Or;
      i++;
    }
    if (i==_requiredWeapon.length-1) s += "\""+wManager.weaponNameFromRef(_requiredWeapon[i])+"\" "+desc_Weapon_Unavailable_End;
    

    return s;
  }
  
  // Used to have the correct table from the subclass
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
  
  // returns false if that weapon requires an other to be selected, that is not is a list
  // wList : list of weapons selected
  public boolean  checkRequiredWeapon(WeaponsList wList){
    
    int[] _requiredWeapon = getRequiredWeapon();
    
    // That weapon does not need any other weapon
    if ( _requiredWeapon.length == 0) {
      return true;
    }
    
    for (int i=0; i<_requiredWeapon.length; i++){
      for (int j=0; j<wList.length();j++){
        if (_requiredWeapon[i]==wList.get(j)) return true;
      }
    }
    
    return false;
  }
  
  // returns false if that weapon requires an other to be selected, that is not is a list
  // wList : list of weapons selected
  // position : position in the list being tested, so that it is not possible to replace 
  // "soldier" with something requiring the soldier
  public boolean  checkRequiredWeapon(WeaponsList wList, int position){
    //return checkRequiredWeapon(wList.getList(),position,wList.length());
    int[] _requiredWeapon = getRequiredWeapon();
    
    // That weapon does not need any other weapon
    if ( _requiredWeapon.length == 0) {
      return true;
    }
    
    for (int i=0; i<_requiredWeapon.length; i++){
      for (int j=0; j<wList.length();j++){
        if (position!=j & _requiredWeapon[i]==wList.get(j)) return true;
      }
    }
    
    return false;
  }
}


class Dirt extends Weapon {
  int requiredWeapon[] = {};
  
  Dirt(World _wld){
    super(_wld);
    cost = 1;
    radiusFix = 6;
    radiusVariable = 5;
    name = s_Weapon_DropDirt;
    description = desc_Weapon_DropDirt;
    type=weaponDirt;
  }
  
  public void drawEffect(int mx,int my,int radius){
    fill(sand);
    stroke(sand);
    for (int i=0;i<scale;i++){
      ellipse(mx,my,radius*2-i,radius*2-i);
    }    
  }
  
  public void impact(Being being,int mx,int my){
    if (!(being instanceof Zombie)){
      being.die(weaponDirt);
    }
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Concrete extends Weapon {
  int requiredWeapon[] = {};
  
  Concrete(World _wld){
    super(_wld);
    cost = 2;
    radiusFix = 4;
    radiusVariable = 4;
    name = s_Weapon_PourConcrete;
    description = desc_Weapon_PourConcrete;
    type = weaponConcrete;
  }
  
  public void drawEffect(int mx,int my,int radius){
    fill(concrete);
    stroke(concrete);
    for (int i=0;i<scale;i++){
      ellipse(mx,my,radius*2-i,radius*2-i);
    }    
  }
  
  public void impact(Being being,int mx,int my){
    if (!(being instanceof Zombie)){
      being.die(weaponConcrete);
    }
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Troopers extends Weapon {
  int requiredWeapon[] = {};
  
  Troopers(World _wld){
    super(_wld);
    cost = 2;
    radiusFix=8;
    radiusVariable=4;
    name = s_Weapon_Soldier;
    description = desc_Weapon_Soldier;
    type = weaponTroopers;
  }
  
  public void drawEffect(int mx,int my,int radius){
  }
  public void impact(Being being,int mx,int my){
    being.equip();
  }
  public void countData(Being being){
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Inoculate extends Weapon {
  int requiredWeapon[] = {};
  
  Inoculate(World _wld){
    super(_wld);
    cost =1;
    radiusFix=4;
    radiusVariable=2;
    name = s_Weapon_InoculateHumans;
    description = desc_Weapon_InoculateHumans;
    type = weaponInoculate;
  }
  public void drawEffect(int mx,int my,int radius){
  }
  public void impact(Being being,int mx,int my){
    if (being instanceof Human) being.infect(0,isweakenedZombie);
  }
  public void countData(Being being){
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Resupply extends Weapon {
  int requiredWeapon[] = {weaponTroopers,weaponHeadquarter};
  
  Resupply(World _wld){
    super(_wld);
    cost =1;
    radiusFix=40;
    radiusVariable=0;
    name = s_Weapon_TroopSupply;
    description = desc_Weapon_TroopSupply;
    type = weaponResupply;
  }
  public void drawEffect(int mx,int my,int radius){
  }
  public void impact(Being being,int mx,int my){
    if (being instanceof Soldier) being.equip();
  }
  public void countData(Being being){
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Bioweapon extends Weapon {
  int requiredWeapon[] = {};
  
  Bioweapon(World _wld){
    super(_wld);
    cost =5;
    radiusFix=4;
    radiusVariable=2;
    name = s_Weapon_Bioweapon;
    description = desc_Weapon_Bioweapon;
    type = weaponBioweapon;
  }
  public void drawEffect(int mx,int my,int radius){
  }
  public void impact(Being being,int mx,int my){
    if (being instanceof Zombie) being.infect(0,ismadZombie);
  }
  public void countData(Being being){
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropDevice extends Weapon{
  int requiredWeapon[] = {};
  
  DropDevice(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_Bomb;
    description = desc_Weapon_Bomb;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new Device(x,y,wld));
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropSlowBomb extends Weapon{
  int requiredWeapon[] = {};
  
  DropSlowBomb(World _wld){
    super(_wld);
    cost=2;
    name= s_Weapon_BigBomb;
    description = desc_Weapon_BigBomb;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new SlowBomb(x,y,wld));
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropQuickBomb extends Weapon{
  
  DropQuickBomb(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_QuickBomb;
    description = desc_Weapon_QuickBomb;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new QuickBomb(x,y,wld));
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropMine extends Weapon{
  
  DropMine(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_Mine;
    description= desc_Weapon_Mine;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new Mine(x,y,wld));
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropBigMine extends Weapon{
  
  DropBigMine(World _wld){
    super(_wld);
    cost=2;
    name= s_Weapon_BigMine;
    description= desc_Weapon_BigMine;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new BigMine(x,y,wld));
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropMineField extends Weapon{
  int ratio; //radius/number of mines
  
  DropMineField(World _wld){
    super(_wld);
    cost=3;
    radiusFix=20;
    radiusVariable=8;
    ratio=4;
    name= s_Weapon_Minefield;
    description= desc_Weapon_Minefield;
  }
  
  public int fire(int x, int y){
    
    int radius=((int)random(radiusVariable)+radiusFix)*scale;
    for (int i=0;i<(radius/ratio);i++){
      wld.devices.add(new Mine(x+(int)random(radius)-radius/2,
                  y+(int)random(radius)-radius/2,wld));
    }
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropHumanAttractor extends Weapon {
  int strength;
  
  DropHumanAttractor(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_MeetingPoint;
    description= desc_Weapon_MeetingPoint;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new Pulse(x,y,strength,wld));
    strength++;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropHumanRepulsor extends Weapon {
  int strength;
  
  DropHumanRepulsor(World _wld){
    super(_wld);
    cost=2;
    name= s_Weapon_ForbiddenArea;
    description= desc_Weapon_ForbiddenArea;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new HumanRepulsor(x,y,strength,wld));
    strength--;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropSoldierAttractor extends Weapon {
  int strength;
  int requiredWeapon[] = {weaponTroopers,weaponHeadquarter};
  
  DropSoldierAttractor(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_GroupTroops;
    description= desc_Weapon_GroupTroops;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new SoldierAttractor(x,y,strength,wld));
    strength++;
    return cost;
  }
    // Obliged to add this to have the correct array to be taken into account.
  /*boolean checkRequiredWeapon(WeaponsList wList){
    return _checkRequiredWeapon(wList,requiredWeapon);
  }
  boolean checkRequiredWeapon(WeaponsList wList, int position){
    return _checkRequiredWeapon(wList, position, requiredWeapon);
  }*/
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropSoldierHQ extends Weapon {
  int strength;
  
  DropSoldierHQ(World _wld){
    super(_wld);
    cost=5;
    name= s_Weapon_Headquarter;
    description= desc_Weapon_Headquarter;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new SoldierHQ(x,y,strength,wld));
    strength++;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropSoldierRepulsor extends Weapon {
  int strength;
  int requiredWeapon[] = {weaponTroopers,weaponHeadquarter};
  
  DropSoldierRepulsor(World _wld){
    super(_wld);
    cost=1;
    name= s_Weapon_Disband;
    description= desc_Weapon_Disband;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new SoldierRepulsor(x,y,strength,wld));
    strength++;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropZombieAttractor extends Weapon {
  int strength;
  
  DropZombieAttractor(World _wld){
    super(_wld);
    cost=2;
    name= s_Weapon_ZombieBait;
    description= desc_Weapon_ZombieBait;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new ZombieAttractor(x,y,strength,wld));
    strength++;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class DropZombieRepulsor extends Weapon {
  int strength;
  
  DropZombieRepulsor(World _wld){
    super(_wld);
    cost=3;
    name= s_Weapon_ZombieRepellent;
    description= desc_Weapon_ZombieRepellent;
    strength=0;
  }
  
  public int fire(int x, int y){
    wld.devices.add(new ZombieRepulsor(x,y,strength,wld));
    strength--;
    return cost;
  }
  
  public int[] getRequiredWeapon(){
    return requiredWeapon;
  }
}

class Device extends Weapon {
  int x,y;
  int timer=0; // number of frames (increases when paused)
  int life; // number of games frames (does not increases when paused)
  int colour,colour1,colour2;
  boolean destroyed=false;
  
  Device(World _wld){
    super(_wld);
  }
  
  Device(int _x, int _y, World _wld){
    super(_wld);
    x=_x;
    y=_y;
    radiusFix=20;
    radiusVariable=8;
    life=100; 
    colour1=color(254,0,0);
    colour2=color(255,255,0);
    type = weaponBomb;
  }
  
  // draws the device on the screen. 
  // blinks even if game paused
  public void drawDevice(){
    // Chooses the color
    if (life<20 && life >=0){
      if (timer%4==0) colour=colour1;
      else colour=colour2;
    }
    else if (life<=50 && life >=0){
      if (timer%8<4) colour=colour1;
      else colour=colour2;
    }
    else if (life<=100 && life >=0){
      if (timer%16<8) colour=colour1;
      else colour=colour2;
    }
    else {
      if (timer%32<16) colour=colour1;
      else colour=colour2;
    }
    
    // Draws the device
    for (int i=0;i<scale*2;i++){
      for (int j=0;j<scale*2;j++){
        set(x+i,y+j,colour);
      }
    }
    timer++;
  }
  
  public void eraseDevice(){
    for (int i=0;i<scale*2;i++){
      for (int j=0;j<scale*2;j++){
        set(x+i,y+j,empty);
      }
    }
  }
  
  // every game frame, checks what the device does
  public void act(){
    if (life ==0) {
      fire(x,y);
      destroyed=true;
      eraseDevice();
    }
    else life--;
  } 
}

class SlowBomb extends Device {
  
  SlowBomb(int _x, int _y,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    radiusFix=25;
    radiusVariable=10;
    life=150; 
    colour1=color(254,0,0);
    colour2=color(255,255,0);
    type=weaponSlowBomb;
  }
}

class QuickBomb extends Device {
  
  QuickBomb(int _x, int _y,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    radiusFix=15;
    radiusVariable=7;
    life=50;
    colour1=color(254,0,0);
    colour2=color(255,255,0);
    type=weaponQuickBomb;
  }
}

class Mine extends Device {
  boolean changed = false;
  int distance,delay;
  
  Mine(World _wld){
    super(_wld);
  }
  
  Mine(int _x, int _y,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    distance=8;
    radiusFix=10;
    radiusVariable=4;
    delay =15;
    life=100; // so blinks slowly
    colour1=color(0,254,0);
    colour2=color(255,255,0);
    type=weaponMine;
  }
  
  public void act(){
    if (timer%4==0 && !changed){
      for (int i=0; i<num;i++){
        if (wld.beings[i] instanceof Zombie && checkBeingInArea(wld.beings[i],x,y,distance)){
          life = delay; 
          changed=true;
        }
      }
    }
    if (changed) {
      if (life ==0) {
        eraseDevice();
        fire(x,y);
        destroyed = true;
      }
      else life --;
    }
  }
}

class BigMine extends Mine {
  
  BigMine(int _x, int _y,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    radiusFix=18;
    radiusVariable=8;
    distance=12;
    delay =25;
    life=200; // so blinks slowly
    colour1=color(0,254,0);
    colour2=color(255,255,0);
    type=weaponBigMine;
  }
}

class Pulse extends Device {
  int cycle;
  int strength;
    
  Pulse(World _wld){
    super(_wld);
  }
  
  Pulse(int _x, int _y, int _strength,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=40+(int)random(4);
    radiusFix=20;
    radiusVariable=0;
    life=600; 
    colour1=human;
    colour2=color(0,254,0);
    type=weaponHumanAttractor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Human && !(being instanceof Soldier) ){
      being.attract(mx,my,50,strength);
    }
  }
  
  public void drawEffect(int mx,int my,int radius){
  }
  
  public void countData(Being being){
  }
  
  public void act(){
    if (life%cycle==0) {
      fire(x,y);
    }
    life--;
    if (life ==0) {
      eraseDevice();
      destroyed=true;
    }
  } 
}

class HumanRepulsor extends Pulse{
  
 
  HumanRepulsor(int _x, int _y, int _strength, World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=40+(int)random(4);
    radiusFix=20;
    radiusVariable=0;
    life=400; 
    colour1=human;
    colour2=color(254,0,0);
    type=weaponHumanRepulsor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Human && !(being instanceof Soldier) ){
      being.attract(mx,my,-50,strength);
    }
  }
}

class SoldierAttractor extends Pulse{
   
  SoldierAttractor(int _x, int _y, int _strength, World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=100;
    radiusFix=40;
    radiusVariable=0;
    life=200; 
    colour1=soldier;
    colour2=color(0,254,0);
    type=weaponSoldierAttractor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Soldier){
      being.attract(mx,my,20,strength);
    }
  }
}

class SoldierHQ extends Pulse{
  int radiusTrain = 25;
  
  SoldierHQ(int _x, int _y, int _strength,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=200;
    radiusFix=40;
    radiusVariable=0;
    life=1000; 
    colour1=soldier;
    colour2=color(0,254,0);
    type=weaponHeadquarter;
  }
  
  public void impact(Being being,int mx, int my){
      if (being instanceof Soldier){
        being.attract(mx,my,10,strength);
        being.equip();
      }
      if ((being instanceof Human) && !(being instanceof Soldier) 
          && checkBeingInArea(being,mx,my,radiusTrain)) being.equip();
  }
}

class SoldierRepulsor extends Pulse{
 
  SoldierRepulsor(int _x, int _y, int _strength,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=100;
    radiusFix=20;
    radiusVariable=0;
    life=200; 
    colour1=soldier;
    colour2=color(254,0,0);
    type=weaponSoldierRepulsor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Soldier){
      being.attract(mx,my,-30,strength);
    }
  }
}

class ZombieAttractor extends Pulse{
 
  ZombieAttractor(int _x, int _y, int _strength,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=25+(int)random(4);
    radiusFix=20;
    radiusVariable=0;
    life=500; 
    colour1=zombie;
    colour2=color(0,254,0);
    type=weaponZombieAttractor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Zombie){
      being.attract(mx,my,50,strength);
    }
  }
}

class ZombieRepulsor extends Pulse{
 
  ZombieRepulsor(int _x, int _y, int _strength,World _wld){
    super(_wld);
    x=_x;
    y=_y;
    strength=_strength;
    cycle=25+(int)random(4);
    radiusFix=20;
    radiusVariable=0;
    life=400; 
    colour1=zombie;
    colour2=color(254,0,0);
    type=weaponZombieRepulsor;
  }
  
  public void impact(Being being,int mx, int my){
    if (being instanceof Zombie){
      being.attract(mx,my,-50,strength);
    }
  }
}
// Classes that enables to draw a graph from a list of values.
// There are 2 important class :
// Data, wich stores the data
// Graph, which displays the data

// That class stores the values
class value{
  int x;
  value(int _x){
    x = _x;
  }  
};

// gets the closest (smaller) power of 10 of a positive integer
public int closestDecimal(int value){
  int i=0;
  
  while (pow(10,i)<value)i++;   
  return (int) pow(10,i-1);
}

// Class data, which stores an undetermined number of int
// It knows which is the max and minimum value
class Data{
  ArrayList values;
  int dataMax;
  int dataMin;
  int dataColor;
  String name;
  
  Data(int _dataColor,String _name){
    values = new ArrayList();
    dataMax=0; // will be changed when first data will be added
    dataMin=0; // idem
    dataColor=_dataColor;
    name=_name;
  }

  public void add(int x){
    if (values.size()==0){ // when first data added
      dataMax=x;
      dataMin=x;
    }
    else {
      if (x>dataMax) dataMax=x;
      if (x<dataMin) dataMin=x;
    }
    values.add(new value(x));
  }  
  
  public int get(int i){
    value x = (value) values.get(i);
    return x.x;
  }
  
  public int getLast(){
    value x = (value) values.get(values.size()-1);
    return x.x;
  }
  
  public int min(){
    return dataMin;
  }
  
  public int max(){
    return dataMax;
  }
  
  public int size(){
    return values.size();
  }
  
  /*int color(){
    return dataColor;
  }*/
  
  // randomly fills the data with values between 0 and 99
  // mostly for testing purpose
  public void Random(int dataSize){
    for (int i=0;i<dataSize;i++){
      if (i==0)add((int)random(10));
      else add(get(i-1)+(int)random(10));
    }
  }
}

class Graph{
  int x,y,width,height;
  int bgColor,drColor;
  
  ArrayList datas; // passed by reference. So they are stored out of this object
  PFont f;
  
  Graph(int _x, int _y, int _width, int _height,ArrayList _datas ){
    f=fNormal;
    bgColor = color(100);
    drColor = color(0); 
    x = _x;
    y = _y;
    width = _width;
    height = _height;
    datas = _datas;
  }
  
  // sets the background color
  public void bgColor(int _bgColor){
    bgColor = _bgColor;
  }
  
  // sets the color the axis will be drawn
  public void drColor(int _drColor){
    drColor = _drColor;
  }
  
  //draws the graph
  public void draw(){
    float step,factor,value;
    int space = 5;
    int fontSize = 10;
    int spaceLeft,spaceBottom;
    int i,yMax, xMax, yUnit, xUnit;
    int spaceLegend = 80;
    Data data;
        
    // draws the area
    rectMode(CORNER);
    noStroke();
    fill(bgColor);
    rect(x,y,width,height);
    
    if (datas!=null){
      // draws the legend
      textFont(f,fontSize);
      for (i=0;i<datas.size();i++){
        data = (Data) datas.get(i);
        fill(data.dataColor);
        textAlign(CENTER);
        String s = data.name + ": "+ data.getLast();
        text(s,x+width-spaceLegend/2,y+height/2+(i+1)*fontSize-datas.size()*fontSize/2);
      }
      
      // Room to leave left and bottom
      yMax=0;
      xMax=0;
      for (i=0;i<datas.size();i++) {
        data = (Data) datas.get(i);
        if (yMax<data.max()) yMax=data.max();
        if (xMax<data.size()) xMax=data.size();
      }
      yUnit = closestDecimal(yMax);
      String tempText = yUnit+"";
      spaceLeft=space+(int)textWidth(tempText);
      
      xUnit = closestDecimal(max(xMax,width-spaceLegend-spaceLeft));
      spaceBottom=space+fontSize;
      
      // finds how to scale the graph
      if (xMax<width-(spaceLeft+spaceLegend)){
        step = 1;
      }
      else step = (float)xMax/(width-(spaceLeft+spaceLegend));
      factor = (float)yMax/(height-(space+spaceBottom));
      
      // draws the canvas
      stroke(drColor);
      line(x+spaceLeft,y+height-spaceBottom,x+spaceLeft,y+space);
      line(x+spaceLeft,y+height-spaceBottom,x+width-spaceLegend,y+height-spaceBottom);   
      
      // draws the units on y axis
      fill(drColor);
      i=0;
      while (i<yMax){
        value = (float)i/factor;
        line(x+spaceLeft,y+height-(spaceBottom+1)-(int)value,x+spaceLeft+3,y+height-(spaceBottom+1)-(int)value);
        tempText = i+"";
        textAlign(RIGHT);
        text(tempText,x+spaceLeft-2,y+height-spaceBottom-(int)value+fontSize/2);
        i+=yUnit;
      }
   
      i=0;
      while (i<max(xMax,width-spaceLegend-spaceLeft)){ // that bit could be optimized
        value = (float)i/step;
        line(x+spaceLeft+(int)value,y+height-spaceBottom,x+spaceLeft+(int)value,y+height-spaceBottom+3);
        tempText = i+"";
        textAlign(CENTER);
        text(tempText,x+spaceLeft+(int)value,y+height-space+2);
        i+=xUnit;
      }   
          
      // plot the values
      for (i=0;i<datas.size();i++) {
        data = (Data) datas.get(i);
        float j;
        stroke(data.dataColor);
        int k=0;
        j=step*k;
        while (j<data.size()){
          value = (float)data.get((int)j)/factor;
          point(x+spaceLeft+1+k,y+height-(spaceBottom+1)-(int)value);
          k++;
          j = step*k;
        }
      }
    }
  }   
}
// Main menu
String s_Menu_Action = "Action";
String desc_Menu_Action = "Select two weapons, and kill zombies in realtime fashion";
String s_Menu_Strategy = "Strategy";
String desc_Menu_Strategy = "Choose a list of weapons, and try to control the zombie outbreak in turn based mode";
String s_Menu_Options = "Options";
String desc_Menu_Options = "Change some parameters of the game, such as the size of the window, speed of the game...";

String desc_Button_Unknown = "No information available";

// Action menu
String s_Menu_Action_Start = "Start !";
String desc_Menu_Action_Start = "Attaaaaaack !";
String s_Menu_Action_Left = "Left";
String desc_Menu_Action_Left = "Select the weapons used when clicking the left mouse button";
String s_Menu_Action_Right = "Right";
String desc_Menu_Action_Right = "Select the weapons used when clicking the right mouse button";
String s_Menu_Action_Speed = "Speed";
String desc_Menu_Action_Speed = "Set the game speed (in frame per second). The higher, the faster.";
String s_Menu_Action_Ammo_Unlimited = "Unlimited";
String s_Menu_Action_Ammo_Limited = "Limited";
String desc_Menu_Action_Ammo = "Toggle between limited and unlimited ammo";
String s_Menu_Action_Default = "Default";
String desc_Menu_Action_Default = "Reset default values";
String s_Menu_Action_Back = "Back";
String desc_Menu_Action_Back = "Go back to general screen";
String s_Menu_Action_More = "More...";
String desc_Menu_Action_More ="Over with mouse to display more weapons";
String s_Menu_AmmoInit = "Initial ammo";
String desc_Menu_AmmoInit = "Number of ammunitions you start with. The bigger, the more you will have."; 
String s_Menu_AmmoRestock = "Ammo restock";
String desc_Menu_AmmoRestock = "Rate at which you get new ammunitions. The bigger, the quicker you will get new ones";
String desc_Menu_Action_AmmoOff = "You have selected unlimited ammunitions. That parameter is of no use.";

String s_Menu_Mission = "Mission";
String desc_Menu_Mission = "Select you mission";

String s_Speed_VerySlow = "Very slow";
String s_Speed_Slow = "Slow";
String s_Speed_Normal = "Normal";
String s_Speed_Quick = "Quick";
String s_Speed_VeryQuick = "Very Quick";

String s_AmmoRestock_None = "None";
String s_AmmoRestock_VerySlow = "Very slow";
String s_AmmoRestock_Slow = "Slow";
String s_AmmoRestock_Normal = "Normal";
String s_AmmoRestock_Quick = "Quick";
String s_AmmoRestock_VeryQuick = "Very quick";
String s_AmmoRestock_ExtremellyQuick = "Too quick";

// Strategy menu
String s_Menu_Strategy_Start = "Start !";
String desc_Menu_Strategy_Start = "Lets carefully nuke some zombies";
String s_Menu_Strategy_Default = "Default";
String desc_Menu_Strategy_Default = "Reset default values";
String s_Menu_Strategy_Back = "Back";
String desc_Menu_Strategy_Back = "Go back to general screen";
String s_Menu_Strategy_TurnLength ="Turn length";
String desc_Menu_Strategy_TurnLength ="Number of simulation iteration between every turn";
String s_Menu_Strategy_InitialTurnLength ="First turn";
String desc_Menu_Strategy_InitialTurnLength ="Length of the first turn";
String s_Menu_Strategy_Weapons ="Weapons";
String desc_Menu_Strategy_Weapons = "Choose your weapons from a range of selections";

// Option menu
String s_Menu_Option_Default = "Default";
String desc_Menu_Option_Default = "Reset default values";
String s_Menu_Option_Back = "Back";
String desc_Menu_Option_Back = "Go back to general screen";

// Weapons
String s_Weapon_Nuke = "Nuke";
String desc_Weapon_Nuke = "A small nuclear warhead shot directly from the sky";
String s_Weapon_Soldier = "Train soldiers";
String desc_Weapon_Soldier = "Train and equip some citizens to be soldiers."
+" They have a limited number of ammuninitions, but can share them."
+" Be carefull though : they are not immunized to the infection.";
String s_Weapon_TroopSupply = "Troops supply";
String desc_Weapon_TroopSupply = "Supply your troops in ammunitions";
String s_Weapon_GroupTroops = "Group troops";
String desc_Weapon_GroupTroops = "Gather all troops from the area to one point";
String s_Weapon_Disband = "Disband troops";
String desc_Weapon_Disband = "Spread soldiers away from that area";
String s_Weapon_Headquarter = "Headquarter";
String desc_Weapon_Headquarter = "A temporary building that will continuously train and supply soldiers. Very efficient...";
String s_Weapon_QuickBomb = "Quick bomb";
String desc_Weapon_QuickBomb = "A small bomb and cheap bomb, that will explode shortly after having been dropped";
String s_Weapon_Bomb = "Bomb";
String desc_Weapon_Bomb = "A regular sized bomb, that will explode some after having been dropped";
String s_Weapon_BigBomb = "Big bomb";
String desc_Weapon_BigBomb = "A large bomb, that will explode a long time after it has been dropped. Plan ahead !";
String s_Weapon_Mine = "Mine";
String desc_Weapon_Mine = "That device will explode when a zombie will come close to it";
String s_Weapon_BigMine = "Big mine";
String desc_Weapon_BigMine = "A bigger mine, causing more damage";
String s_Weapon_Minefield = "Minefield";
String desc_Weapon_Minefield = "Many regular mines dropped in an area";
String s_Weapon_DropDirt = "Drop dirt";
String desc_Weapon_DropDirt = "Drops construction material from the sky to hinder zombies progression."
+" It will crush humans, but zombies are tough and might survive when cast into that material";
String s_Weapon_PourConcrete = "Pour concrete";
String desc_Weapon_PourConcrete = "Pour good old concrete to block zombies progression."+
" It is tougher than regular construction material, but poured in smaller quantity";
String s_Weapon_MeetingPoint = "Meeting Point";
String desc_Weapon_MeetingPoint = "Group civilians from an area to gather to one point";
String s_Weapon_ForbiddenArea = "Forbidden Area";
String desc_Weapon_ForbiddenArea = "Civilians should avoid that area fro some time";
String s_Weapon_ZombieBait = "Zombie bait";
String desc_Weapon_ZombieBait = "Attracts some zombies of the area to one point";
String s_Weapon_ZombieRepellent = "Zombie repellent";
String desc_Weapon_ZombieRepellent = "Repels most zombies of the area away from that point";
String s_Weapon_InoculateHumans = "Inoculate humans";
String desc_Weapon_InoculateHumans = "Evil, but efficient defense consiting in inoculating humans with a weakened zombie virus :"+
" that tuned down infection will spread locally, preventing those humans to turn into regular zombies";
String s_Weapon_Bioweapon = "Bioweapon";
String desc_Weapon_Bioweapon = "Innoculate a bio engineered virus, designed to infect zombies only."
+" It will alter their behaviour, so they will attack regular zombies and ignore humans."+
" Efficient to get rid of existing zombies, but will rarely stop zombies progression.";

// List
String s_List_FullArsenal = "Full Arsenal";
String desc_List_FullArsenal = "Simply go fighting zombies with every existing weapons. "
+"It might be a bit tedious to cycle the whole list, though";
String s_List_Custom = "Custom List";
String desc_List_Custom = "Choose any weapon you want. Click once to select, and again to remove a weapon";
String s_List_Biowar = "Bio war";
String desc_List_Biowar = "You bio engineers managed ot alter the virus. Infect zombies and " 
+"control the population to limit infection spreading";
String s_List_Army = "Army";
String desc_List_Army = "Control the infection with trusty soldiers";
String s_List_Demolition = "Demolition";
String desc_List_Demolition = "Bombs, concrete, mines. Plenty of funny things to build and destruct";
String s_List_Nuke4 = "Nuke + 4";
String desc_List_Nuke4 = "The nuke and 4 random weapons. Click again to have a new selection";
String s_List_Random6 = "6 Random";
String desc_List_Random6 = "6 random weapons. Click again to have a new selection of weapons";

String s_Weapon_Unknown = "Unknown Weapon";
String desc_Weapon_Unavailable = "That weapon can't be selected yet : you need to select it together with ";
String desc_Weapon_Unavailable_Or = "or ";
String desc_Weapon_Unavailable_End = ".";

// Panel Strategy
String s_Menu_PanelStrategy_NextTurn = "Next turn";
String desc_Menu_PanelStrategy_NextTurn = "Click the screen to play. Then press the button to move to next turn";
String desc_Menu_PanelStrategy_NextTurn_Unavailable = "Wait for next turn to be able to play. Watch patiently and enjoy the show !";
String s_Menu_PanelStrategy_NextWeapon = "-->";
String desc_Menu_PanelStrategy_NextWeapon = "Select next weapon";
String s_Menu_PanelStrategy_PreviousWeapon = "<--";
String desc_Menu_PanelStrategy_PreviousWeapon = "Select previous weapon";

// Panel Action
String s_Menu_PanelAction_Pause = "Pause";
String s_Menu_PanelAction_Continue = "Continue";
String s_Menu_PanelAction_Start = "Start";
String desc_Menu_PanelAction_Pause = "Pause/continue the game";
//String desc_Menu_PanelAction_Continue = "Continue the game";
String s_Text_PanelAction_Paused = "- game paused -";
String s_Text_PanelAction_Left = "Left: ";
String s_Text_PanelAction_Right = "Right: ";

// Panel Finished
String s_Text_PanelFinished_Win = "You won !";
String s_Text_PanelFinished_Loose = "You have lost...";
String s_Menu_PanelFinished_Restart = "Restart";
String desc_Menu_PanelFinished_Restart = "Restart the game, with the same settings";
String s_Menu_PanelFinished_BackMenu = "Back to menu";
String desc_Menu_PanelFinished_BackMenu = "Go back to the menu";
String s_Menu_PanelFinished_KeepGoing = "Keep watching";
String s_Menu_PanelFinished_Pause = "Pause";
String desc_Menu_PanelFinished_KeepGoing = "Keep watching the survivors struggle against the infection. But you can't interract anymore";

// Panel General
String s_Text_Panel_Cost = "Cost: ";
String s_Text_Panel_CostUnlimited = "Cost: -";
String s_Text_Panel_Ammo = "Ammo: ";
String s_Text_Panel_AmmoUnlimited = "Ammo: -";

// Panel Info
String s_Menu_PanelInfo_Stat = "Statistics";
String desc_Menu_PanelInfo_Stat = "Visualise some statistics about the game";
String s_Menu_PanelInfo_Mission = "Mission";
String desc_Menu_PanelInfo_Mission = "Description of your objectives, and how far you are from acheiving them."; 
String s_Menu_PanelInfo_Score = "Score";
String desc_Menu_PanelInfo_Score = "When you game is finished, displays the score.";
String s_Menu_PanelInfo_Victory= "Victory";
String s_Menu_PanelInfo_Failure= "Failure";
String s_Menu_PanelInfo_Warning= "Warning";
String s_Menu_PanelInfo_FinishedScore= "Click area to view your score";

// Panel Info Mission
String s_InfoMission_WinningCondition = "Win conditions  :";
String s_InfoMission_LoosingCondition ="Loose conditions :";
String s_InfoMission_Misc ="Other events :";


// Score
String s_Score_HumansAlive = "Humans still alive";
String s_Score_HumansKilled = "Humans you have killed";
String s_Score_ZombiesAlive = "Zombies alive";
String s_Score_ZombiesKilled = "Zombies killed";
String s_Score_Special = "Special events";
String s_Score_MissionAccomplishment = "Mission accomplishment";
String s_Score_Won = "You won !";
String s_Score_Lost = "You failed...";
String s_Score_Draw = "Draw";

String s_InfoScore_Unfinished = "The game is still running. There is no score yet.";
String s_InfoScore_Number = "Number";
String s_InfoScore_Multiplier = "Multiplier";
String s_InfoScore_Score = "Score";
String s_InfoScore_FinalScore = "Final score";

// Event
String s_Event_None = "Nothing.";
String s_Event_Infect_1 = "Infect ";
String s_Event_Infect_2 = " humans";
String s_Event_Win = "You win";
String s_Event_Loose = "You loose";
String s_Event_BombZombies = "The army bombs the city";

String s_Event_Condition_ZombieDead = "When no zombies are left";
String d_Event_Condition_ZombieDead = "There are no zombies left";
String s_Event_Condition_ZombieMax_1 = "When there are more than "; 
String s_Event_Condition_ZombieMax_2 = " zombies in the city";
String w_Event_Condition_ZombieMax_1 = "There are close to ";
String w_Event_Condition_ZombieMax_2 = " zombies !";
String d_Event_Condition_ZombieMax_1= "There are more than ";
String d_Event_Condition_ZombieMax_2 = " zombies in the city";
String s_Event_Condition_HumanKilledMax_1 = "When you have killed more than "; 
String s_Event_Condition_HumanKilledMax_2 = " humans";
String w_Event_Condition_HumanKilledMax_1 = "You have almost killed  ";
String w_Event_Condition_HumanKilledMax_2 = " humans !";
String d_Event_Condition_HumanKilledMax_1 = "You have killed more than "; 
String d_Event_Condition_HumanKilledMax_2 = " humans";
String s_Event_Condition_HumanDead_1 = "When there are ";
String s_Event_Condition_HumanDead_2 = " survivors or less";
String w_Event_Condition_HumanDead_1 = "There is close to ";
String w_Event_Condition_HumanDead_2 = " survivors...";
String d_Event_Condition_HumanDead_1 = "There are ";
String d_Event_Condition_HumanDead_2 = " survivors or less";
String s_Event_Condition_Time_1 = "After ";
String s_Event_Condition_Time_2 = " minutes";
String w_Event_Condition_Time_1 = "Almost ";
String w_Event_Condition_Time_2 = " minutes have passed !";
String d_Event_Condition_Time_1 = "More than ";
String d_Event_Condition_Time_2 = " minutes have passed";

String s_Mission_Regular = "Resist";
String desc_Mission_Regular = "The army is preparing its reponse to the infection. Hold the zombies as long as you can";
String s_Mission_Second = "Alone";
String desc_Mission_Second = "You are alone to eradicate the infection before the population is wipped out."
+" Please note that killing innocent citizens would not be appreciated...";
String s_Mission_Third = "Control";
String desc_Mission_Third = "That city is considered as doomed. But for global health reasons, you should not allow the zombie population to rise too high.";
String s_Mission_Vanilla = "Vanilla";
String desc_Mission_Vanilla = "The most basic mission : have at least one survivor alive when all zombies have died.";
String s_Mission_MadScientist = "Mad scientist";
String desc_Mission_MadScientist = "You are a mad scientist and should try to eradicate the population before all zombies have died. The less points you have, the better.";
/*
Todo :

04/12/2012 :
------------
- Screen and mission now display points awarded for mission completion


Things to do to finish version :
--------------------------------
- Color 
-- choose new color sheme
- Mission
-- Propose different missions

Remarks :
-------------------
- In strategy, default does not reset custom list
- In custom list, should not be able to deselect weapon required for other selected weapon (or should deselect everyother)
- In action, should not be allowed to select twice the same weapon
- Maybe consider allowing custom list in action ?
- Pause button highlight in panels
- some work on colors and general look... So far, it really looks ugly.
- Warning : could write a specific description of what triggered the warning


Optimisation :
--------------
- Profiling
-- This is the first step : knowing where time is being spent
VisualVM should allow to do that : http://visualvm.java.net/profiler.html

- Graph
-- ArrayList.size() is efficient enough (does not calculate again 
the value). But I should avoid calling the function again and 
again in the looping condition
-- General review of efficiency
-- Should the graph be drawn every tick ?

- Data
-- The way it is done now :
--- Game initialisation : create the various data fields.
--- Every turn : when weapon kills, is stored
--- Every turn : number of each class is computed then stored (rem : could be improved)
--- When being changes status, remembers what it was, and what killed it (saved in being)

-- Reasons for storing data :
--- End of game. Conditions on number of zombies or being. Present system is sufficient
--- Flavour text at the end of the game. From number of killed people... Close to be sufficient. Could be improved :
---- Number of killed per weapon (comment such as bombong artist,...)
---- Number of infected
--- Beings feat : recalls which being have behaved extraordinally (who killed the most zombies,...)
--- Weapons facts : recalls how many times a weapon has been used, and people it killed.
--- Graph : displays story line of different data.

-- Data to store and places to save that :
--- Being specific stats : in beings. 
---- Storyline (store events with date)
---- Synthetic data (total number of kills) (computed when storyline event created)
---- latest status (same as now ->what killed them, and previous type)
---- Fields for synthetic values adapt (new can be created when being changes type)
Allows to select exceptional beings, display their history, display info on map
--- Weapon specific data : in weapons ?
---- Number of time fired
---- Number of being killed/transformed/impacted
-> for devices, and things done by beings (soldier and zombies), need to link to weapon.
---- Storyline : give these values with date + synthetic
---- Total for weapons calculated
--- General population data
---- computed every turn (or few turns)
---- stores general data about the different population total and killed

- Mission :
*- Stores initial human

-- Event : known / hidden / random 
*-- Add zombies (?) (initial event)
--- immigrants (zombies / humans / soldier)
--- Wind (zombie move toward one direction)
--- Time : Bomb all zombies, score on number of surving humans (but dead by support don't count in score)
-> winning tested after event.

-- Ending conditions. When met, pauses the game display score, then proposes to continue.
*-- Zombies : All dead -> Win 
(--- Zombies : Number escaped -> loose (all map bombed) -> pause, then propose to continue)
*-- Human : All human dead -> loose 
*-- Human : number dead -> loose 
*-- Human : Number of human killed by player -> loose 
(--- Human : Number human saved -> win (bomb zombies))
(--- VIP : Number of Vip killed -> loose (Bomb zombies))
(--- VIP : Number of Vip saved -> win )


-- Score -> Allways same way to count
--- number human alive (2 pt par humain)
--- number human killed (-10 pt par humain) 
--- number zombies alive (-200 pour premier, puis augmentation diminue)
--- number of zombies killed (1 pt)
--- (number zombies escaped) (-400 pour premier, puis diminue)
--- (number human saved) (5 pt par humain)

-- Award :
(later)

Heat map (zombie / human location):
----------
Map to give direction for closer zombies / humans
-> possibility is spread smell from human with smaller increment every step (expensive).
-> Barycentre zombie par rapport position soldat. Mais pas de pathfinding.
-> Mix between spreading smell, over small area / updated every n turn for every being (not too often) for short distance + 
-> or communication between being (no path finding).

Structure :
-----------

- 


UI :
-----

- Navigation :
-- Screens :
--- First Screen
---- Game name (+ animation ?)
---- Select game style
----- Options
      - Scale (slider)
      - Size
        Slider for Width and Heigth. With max corresponding
        to screen
      - Human density (slider)
      - Zombie density (Z per human, or Z per square pixel ?)
      - Default button
      - Back to main screen
----- Help (?)

- Option Screen
-- When ? when starting, when pressing a key, when clicking,...
-- Can select differnt things to display at the bottom of the 
   screen (graph, ...)

- When paused, proposes different display mode
-- death reason (color code)
-- death density (number of zombies on one pixel)
-- humans (+ belief ?)
-- zombies (+ age)
-- Ammo
-- Devices

- No special effect layer (no sfx)


Game :
------

- Weapons :
-- Examples :
--- Foam :
    Expenses in wall
    Only grows on empty tiles (kills people). Drops objects at
    the limit, expense on empty, die when surrounded. Number 
    of objects to be placed controlled by central object
--- Dig :
    Place 2 objects. A rectangle is built between both. 
    Surface area is the same
--- Build :
    Same as dig, except that it builds (dirt, concrete)
--- Corridor :
    Same as dig, except that it builds a corridor (surrounded 
    with concrete
--- Paratroopers
    Click + Being
    Drops soldiers (instead of training people)
--- Rambo
    Click + Being
    Drops of tough soldier
    Large field of vision (everything in front of him), 
    but limited distance (5 ?). Moves forward if further than 5.
    Backward if closer than 4.
--- Information (changes the behaviour in an area)
    Click
    Reduces Panick
--- Path
    First, attracts people, then sends to second (is there a 
    chance for people to get confused and forget ?)
--- Take off
    Collects people in area (maybe coupled with attracktor)
--- Troop transport
    Collects troops (soldiers,...) and possibility once loaded
    to transport them to any other place of the map (create
    new option in weapons list)
--- Vaccin
    Click (area)
    Instead of turning into zombies, vaccinated people will
    just die
    Different type of vaccin -> when bitten, can turn into dead or
    other type of zombie (sleeping, bioweapon,...)
    
--- Doctor
    Click Being (chance of beings in area to be turned into 
    doctor)
    Move toward other humans (soldiers,...) to vaccinate them
    
--- Diagnostic
    Device that lets you know health level of humans nearby (displays specific color, instead basic type)
   
-- Disater
--- Infection : people infecting each other and then turning
    into zombies. To investigated(?) (could drop a dead body,
    that will rise again. Information, might teacj people to 
    butcher those fresh corpses.)
    Maybe also add behaviour of poeple killing each other even
    if not sick.

-- Somehow allowing user weapons customisation ? (adding effects
and damages to each other).
--- Either through parameters
--- Or through modules, combined together
--- need to create interface for that

- Data
-- save and display more data : max killed humans, max shot zombies,... 
-- Weapon like system, that gives stat in an area when clicked. 
  (Problem : how to display area selected without layers ?

- Behaviour
-- Maybe change color so only one or 2 parameters to recognize, and third to store some data 
(such as heading)
-- Investigating flocking. Might be tough there 
  (idea : different color depending on direction)
-- Slightly different than flocking, but have remote zombies attacking where survivors are 
(Should it be all zombies going toward closed humans, or a huge group moving being formed once a while ?)
-- Causalties if touched by panic, 
-- might also alter sensibility to attractors / repulsors
-- Investigating path finding for some weapons/beings

- Being history
-- Separate module storing stat and beings history (before and during game)
-- Being created with past (first name, surname, relatives, neighbours, job, age)
-- Remember if interaction with relative
-- Question how to iniate that cleverly ?
-> place all beings, then create story. (can choose people nearby)
-> create one after the other (risk of not spreading people evenly in town)
-> Simulate life year per year or decade per decade. But tough to control the xact numer in the end. 
Can check at some times and send immigrants / have emigrants. (create model that defines how many of each class I should have
every year/decade.
Check that website for first names/age :
http://www.ssa.gov/OACT/babynames/decades/index.html
http://www.prenoms.com/v2/services-prenom/top-des-prenoms.asp?theme=msn&year=
http://names.mongabay.com/most_common_surnames.htm
http://surnames.behindthename.com/top/lists/1000frs2005.php
http://www.insee.fr/fr/themes/tableau.asp?reg_id=0&ref_id=NATTEF02236

- Beings structure
-- Maybe devide being into modules, so easier to try different combination of zombies
-> see java interface

- Alternate Zombies :
-- Eat people (kill them) instead of changing them into zombies
--- Some dead might turn into zombie later (maybe usefull to chop them down)
--- Or dead might infect people, who then might turn into zombies
-- Mobs, or detection of living from far way ?
-- Possibility of evolution ?
--- Change infection type (target, kill/infect ratio, 
-- Infection transmitted by humans contact or not ?
--- Die, seems to die then turn into zombie, turn into zombie straight away ?
--> might need to think about zombie/diversity soon

- Other People type :
-- injured. They move slowly. Zombies feed on them (but can they turn into zombies ?)
Could be rescued by survivor ?

- UI :
-- Possibility to create scenario (parameters from start, with a bit of text)
-- Possibility to have scenario follow each other ? or meta game ?
-- Possibility to have alternative skin (multiculutre, or love spreading instead of zombies)
-- Language selection

Info position :
618-10 : 608 -> 76
138-10 = 128 -> 8,5

*/
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "ZombieInfection" });
  }
}
